# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz>
"""
$Id: CacheManager.py,v 1.5.4.1.2.1 2005/03/25 04:07:47 fafhrd91 Exp $
"""

import time
from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo
from thread import allocate_lock

from Products.StandardCacheManagers.RAMCacheManager import \
     CacheException, RAMCache, RAMCacheManager, \
     CacheEntry, ObjectCacheEntries, _marker

from debug import log
from StringIO import StringIO

class CMFBoardCache (RAMCache):
    # Note the need to take thread safety into account.
    # Also note that objects of this class are not persistent,
    # nor do they make use of acquisition.
    max_age = 0

    def getObjectCacheEntries(self, ob, hint=(), create=0):
        """
        Finds or creates the associated ObjectCacheEntries object.
        Remember to lock writelock when calling with the 'create' flag.
        """
        cache = self.cache
        path = ob.getPhysicalPath()
        oc = cache.get((path, hint), None)
        if oc is None:
            if create:
                cache[(path, hint)] = oc = ObjectCacheEntries(path)
            else:
                return None
        return oc

    def g1etCacheReport(self):
        """
        Reports on the contents of the cache.
        """
        rval = []
        for oc in self.cache.values():
            size = 0
            ac = 0
            views = []
            for entry in oc.entries.values():
                size = size + entry.size
                ac = ac + entry.access_count
                view = entry.view_name or '<default>'
                if view not in views:
                    views.append(view)
            views.sort()
            info = {'path': '/'.join(oc.physical_path),
                    'hits': oc.hits,
                    'misses': oc.misses,
                    'size': size,
                    'counter': ac,
                    'views': views,
                    'entries': len(oc.entries)
                    }
            rval.append(info)
        return rval

    def ZCache_invalidate(self, ob):
        '''
        Invalidates the cache entries that apply to ob.
        '''
        path = ob.getPhysicalPath()
        # Invalidates all subobjects as well.
        
        self.writelock.acquire()
        #print self.cache
        try:
            
            for p, oc in self.cache.items():
                pp = oc.physical_path
                #print 'ZCache_invalidate'
                #print p
                #print '-------------------------------------'
                #print pp
                #print '-------------------------------------'
                #print path
                #print '-------------------------------------'
                if pp[:len(path)] == path:
                    del self.cache[p]
        finally:
            self.writelock.release()

    def ZCache_invalidate_by_hint(self, ob, hint=()):
        '''
        Invalidates the cache entries that apply to ob.
        '''
        #print 'ZCache_invalidate_by_hint', ob, hint
        
        path = ob.getPhysicalPath()
        self.writelock.acquire()
        try:
            for p, oc in self.cache.items():
                pp = oc.physical_path
                if (pp[:len(path)] == path) and (p[1] == hint):
                    del self.cache[p]
        finally:
            self.writelock.release()

    def ZCache_get(self, ob, view_name='', keywords=None,
                   mtime_func=None, default=None):
        '''
        Gets a cache entry or returns default.
        '''
        here = keywords.get('here', None)
        if here is None:
            hint = ''
        else:
            try:
                hint = here.getPhysicalPath()
            except:
                hint = here
        
        oc = self.getObjectCacheEntries(ob, hint)
        if oc is None:
            return default
        lastmod = ob.ZCacheable_getModTime(mtime_func)
        index = oc.aggregateIndex(view_name, ob.REQUEST,
                                  self.request_vars, keywords)
        entry = oc.getEntry(lastmod, index)
        if entry is _marker:
            return default
        if self.max_age > 0 and entry.created < time.time() - self.max_age:
            # Expired.
            self.writelock.acquire()
            try:
                oc.delEntry(index)
            finally:
                self.writelock.release()
            return default
        oc.hits = oc.hits + 1
        entry.access_count = entry.access_count + 1
        return entry.data

    def ZCache_set(self, ob, data, view_name='', keywords=None,
                   mtime_func=None):
        '''
        Sets a cache entry.
        '''
        #print 'ZCache_set', ob
        here = keywords.get('here', None)
        if here is None:
            hint = ''
        else:
            try:
                hint = here.getPhysicalPath()
            except:
                hint = here

        now = time.time()
        if self.next_cleanup <= now:
            self.cleanup()
            self.next_cleanup = now + self.cleanup_interval

        lastmod = ob.ZCacheable_getModTime(mtime_func)
        self.writelock.acquire()
        try:
            oc = self.getObjectCacheEntries(ob, hint, create=1)
            try:
                index = oc.aggregateIndex(view_name, ob.REQUEST,
                                          self.request_vars, keywords)
                oc.setEntry(lastmod, index, data, view_name)
                oc.misses = oc.misses + 1
            except:
                # sometimes error happen
                pass
        finally:
            self.writelock.release()


caches = {}

class CMFBoardCacheManager (RAMCacheManager):
    meta_type = 'CMFBoard RAM Cache Manager'
    security = ClassSecurityInfo()

    def __init__(self, ob_id):
        self.id = ob_id
        self.title = ''
        self._settings = {
            'threshold': 1000,
            'cleanup_interval': 300,
            'request_vars': ('AUTHENTICATED_USER', 'SESSION', 'HTTP_ACCEPT_LANGUAGE',
                             'b_start', 'portal_status_message'),
            'max_age': 3600,
            }
        self.__cacheid = '%s_%f' % (id(self), time.time())

    security.declarePrivate('ZCacheManager_getCache')
    def ZCacheManager_getCache(self):
        cacheid = self.__cacheid

        try:
            return caches[cacheid]
        except KeyError:
            cache = CMFBoardCache()
            cache.initSettings(self._settings)
            caches[cacheid] = cache
            return cache

    security.declarePublic('Dump')
    def Dump(self):
        """ """
        out = StringIO()

        items = self.ZCacheManager_getCache().cache.items()
        items.sort()

        for k, v in items:
            print >> out, '"%s": "%s"'%(k, v)

        log(out.getvalue())

InitializeClass(CMFBoardCacheManager)

manage_addCMFBoardCacheManagerForm = DTMLFile('zpt/addCMFBoardRCM', globals())

def manage_addCMFBoardCacheManager(self, id, REQUEST=None):
    'Adds a RAM cache manager to the folder.'
    self._setObject(id, CMFBoardCacheManager(id))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)
