# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nikolay Kim <fafhrd@legco.biz>
"""
$Id: CacheManagerMisc.py,v 1.3 2004/04/27 06:56:20 fafhrd91 Exp $
"""

import time
from Acquisition import aq_base
from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo
from thread import allocate_lock

from Products.StandardCacheManagers.RAMCacheManager import \
     CacheException, RAMCache, RAMCacheManager, \
     CacheEntry, ObjectCacheEntries, _marker

from debug import log
from StringIO import StringIO

class RAMCacheMisc (RAMCache):
    max_age = 0

    def ZCache_get(self, ob, view_name='', keywords=None,
                   mtime_func=None, default=None):
        """ """
        oc = self.getObjectCacheEntries(ob)
        if oc is None:
            return default

        base = aq_base(ob)
        lastmod = max(getattr(base, '_p_mtime', 0), 0)
        klass = getattr(base, '__class__', None)
        if klass:
            lastmod = max(getattr(klass, '_p_mtime', lastmod), lastmod)
        
        index = oc.aggregateIndex(view_name, {}, self.request_vars, keywords)
        entry = oc.getEntry(lastmod, index)

        #print 'ZCache_get', index
        
        if entry is _marker:
            return default
        if self.max_age > 0 and entry.created < time.time() - self.max_age:
            # Expired.
            self.writelock.acquire()
            try:
                oc.delEntry(index)
            finally:
                self.writelock.release()
            return default
        oc.hits = oc.hits + 1
        entry.access_count = entry.access_count + 1
        return entry.data

    def ZCache_set(self, ob, data, view_name='', keywords=None,
                   mtime_func=None):
        '''
        Sets a cache entry.
        '''
        #print 'ZCache_set'
        now = time.time()
        if self.next_cleanup <= now:
            self.cleanup()
            self.next_cleanup = now + self.cleanup_interval

        base = aq_base(ob)
        lastmod = max(getattr(base, '_p_mtime', 0), 0)
        klass = getattr(base, '__class__', None)
        if klass:
            lastmod = max(getattr(klass, '_p_mtime', lastmod), lastmod)

        self.writelock.acquire()
        try:
            oc = self.getObjectCacheEntries(ob, create=1)
            index = oc.aggregateIndex(view_name, {}, self.request_vars, keywords)
            #print index

            oc.setEntry(lastmod, index, data, view_name)
            oc.misses = oc.misses + 1
        finally:
            self.writelock.release()

        #print '----------------'

caches = {}
PRODUCT_DIR = __name__.split('.')[-2]

class RAMCacheManagerMisc (RAMCacheManager):
    meta_type = 'CMFBoard Misc Cache Manager'

    def __init__(self, ob_id):
        self.id = ob_id
        self.title = ''
        self._settings = {
            'threshold': 1000,
            'cleanup_interval': 300,
            'request_vars': (),
            'max_age': 10800,
            }
        self.__cacheid = '%s_%f' % (id(self), time.time())

    ZCacheManager_getCache__roles__ = ()
    def ZCacheManager_getCache(self):
        cacheid = self.__cacheid
        try:
            return caches[cacheid]
        except KeyError:
            cache = RAMCacheMisc()
            cache.initSettings(self._settings)
            caches[cacheid] = cache
            return cache

InitializeClass(RAMCacheManagerMisc)


manage_addRAMCacheManagerMiscForm = DTMLFile('zpt/addRCMMisc', globals())

def manage_addRAMCacheManagerMisc(self, id='CMFBoardCache2', REQUEST=None):
    'Adds a RAM cache manager to the folder.'
    self._setObject(id, RAMCacheManagerMisc(id))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)
