# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
"""
$Id: DiscussionFilter.py,v 1.10.6.1 2005/01/25 06:59:12 fafhrd91 Exp $
"""

import os
from Acquisition import aq_parent, aq_inner
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.Archetypes.config import TOOL_NAME

from Products.CMFCore.utils import _checkPermission, getToolByName
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent, ManagePortal
from Products.CMFCore.Expression import Expression, createExprContext

from config import _zptdir
#from ForumContentTopic import ForumContentTopic

class ForumDiscussionFilter( SimpleItem ):
    """ """
    meta_type = 'ForumDiscussionFilters'

    security = ClassSecurityInfo()

    manage_options = (({'label': 'Filters', 'action':'manage_editFilters_form'},) +
                      SimpleItem.manage_options)

    manage_editFilters_form = PageTemplateFile( 'editFilters', _zptdir )

    _filters = []

    id = 'discussion_filters'

    def __init__(self):
        self.title = 'Discussion filters'

    security.declarePrivate( 'isDiscussionAllowedFor' )
    def isDiscussionAllowedFor( self, content, portal ):
        filters = self._filters

        if filters:
            context = createExprContext(aq_parent(aq_inner(content)), portal, content)

            for rec in self._filters:
                if rec['expr_ob'](context):
                    if rec['forum'] == '__block__':
                        return {'status': '__block__', 'allowed': 0}
                    else:
                        return {'status': '__created__', 'allowed': 1}

        return {'status': None}

    security.declarePrivate('createDiscussionFor')
    def createDiscussionFor(self, content, portal):
        """ check filters and create topic """
        filters = self._filters

        if filters:
            context = createExprContext(aq_parent(aq_inner(content)), portal, content)

            for rec in self._filters:
                if rec['expr_ob'](context):
                    if rec['forum'] == '__block__':
                        return {'status': '__block__', 'topic':None}
                    else:
                        return {'status': '__created__',
                                'topic': self._create_topic(content, rec['forum'])}

        return {'status': None}
    
    security.declarePrivate('getForumForDiscussion')
    def getForumForDiscussion(self, content, portal):
        """ check filters and create topic """
        filters = self._filters

        if filters:
            context = createExprContext(aq_parent(aq_inner(content)), portal, content)

            for rec in self._filters:
                if rec['expr_ob'](context):
                    if rec['forum'] == '__block__':
                        return {'status': '__block__', 'forum':None}
                    else:
                        return {'status': '__forum__', 'forum': rec['forum']}

        return {'status': None}
    

    def _create_topic(self, content, forum):
        forum_ob = getToolByName(self, TOOL_NAME).lookupObject(forum)

        if forum_ob is None:
            return None

        # create topic
        topic_ob = forum_ob._addTopic('discussion', '', language=content.Language(),
                                      portal_type=ForumContentTopic.meta_type,
                                      content_uid = self.UID(),
                                      text_format=getattr(content,'text_format','text/plain'))
        return topic_ob


    security.declareProtected(ManagePortal, 'manage_addFilter')
    def manage_addFilter(self, filter, REQUEST=None):
        """ """
        expr = filter.expr
        forum = filter.forum
        
        try:
            expr_ob = Expression(expr.strip())

            if forum != '__block__':
                tool = getToolByName(self, TOOL_NAME)
                forum_ob = tool.lookupObject(forum)
                if forum_ob is None:
                    raise Exception, "Can't find forum"
            rec = {'expr': expr, 'expr_ob': expr_ob, 'forum': forum}
            self._filters = self._filters + [rec]
            msg = 'Filter added'
        except Exception, e:
            msg = str(e)

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(
                '%s/manage_editFilters_form?manage_tabs_message=%s'%
                (self.absolute_url(), msg))

    security.declareProtected(ManagePortal, 'manage_deleteFilters')
    def manage_deleteFilters(self, ids, REQUEST=None):
        """ """
        if type(ids) == type(''):
            ids = [ids]

        ids = [int(i) for i in ids]

        filters = self._filters
        new_filters = []
        for idx in range(len(filters)):
            if idx in ids:
                continue

            new_filters.append(filters[idx])
            
        self._filters = new_filters

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(
                '%s/manage_editFilters_form?manage_tabs_message=%s'%
                (self.absolute_url(), 'Filters deleted'))

    security.declareProtected(ManagePortal, 'manage_updateFilters')
    def manage_updateFilters(self, filters, REQUEST=None):
        """ """
        self._filters = []
        for filter in filters:
            self.manage_addFilter(filter)

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(
                '%s/manage_editFilters_form?manage_tabs_message=%s'%
                (self.absolute_url(), 'Filters updated'))
        
    
    security.declareProtected(ManagePortal, 'manage_listFilters')
    def manage_listFilters(self):
        """ """
        return tuple(self._filters)


InitializeClass( ForumDiscussionFilter )
