# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nikolay Kim <fafhrd@legco.biz> 
"""
$Id: DiscussionTool.py,v 1.36.4.1.2.3 2005/04/07 18:23:54 fafhrd91 Exp $
"""
import traceback
from Globals import InitializeClass
from Acquisition import aq_inner, aq_parent, aq_base
from AccessControl import ClassSecurityInfo
from OFS.PropertyManager import PropertyManager

from Products.CMFCore.Expression import Expression
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View, ManagePortal, ModifyPortalContent
from Products.CMFDefault.DiscussionTool import DiscussionTool, DiscussionNotAllowed

from utils import cb_isinstance
#from converters import convert
from DiscussionFilter import ForumDiscussionFilter
from DiscussionItem import ForumDiscussionContainer, DiscussionItemWrapper


class ForumDiscussionTool( DiscussionTool, PropertyManager ):
    """ """
    meta_type = 'ForumDiscussionTool'

    security = ClassSecurityInfo()

    manage_options = DiscussionTool.manage_options + PropertyManager.manage_options

    _actions = [ActionInformation(id='reply'
                                , title='Reply'
                                , action=Expression(
                text='string: ${object_url}/discussion_reply_form')
                                , condition=Expression(
                text='python: object is not None and ' +
                'portal.portal_discussion.isDiscussionAllowedFor(object)')
                                , permissions=('Reply to item',)
                                , category='object'
                                , visible=1
                                 )]

    _properties = (
        {'id': 'only_published', 'type': 'boolean', 'mode': 'rw'},
        {'id': 'auto_create_topics', 'type': 'boolean', 'mode': 'rw'},
        {'id': 'auto_converting', 'type': 'boolean', 'mode': 'rw'},
        )
    
    _default = ''
    _default_vm = 'nested'
    
    only_published = 0
    auto_create_topics = 1
    auto_converting = 0
    
    _type_vm = {}
    _type_map  = {}

    def __init__(self):
        self.id = 'portal_discussion'

    security.declareProtected(ManagePortal, 'edit_tool')
    def edit_tool(self, default, default_vm):
        """ """
        request = self.REQUEST
        traverse = self.unrestrictedTraverse

        self._default = default
        self._default_vm = default_vm

        map = {}
        map_vm = {}
        for tp in getToolByName(self, 'portal_types').listTypeInfo():
            id = tp.getId()
            f_var = 'tp_%s'%id
            vm_var = 'vm_%s'%id
            if request.has_key(f_var):
                forum = request[f_var]
                map[id] = forum
                
            if request.has_key(vm_var):
                vm = request[vm_var]
                map_vm[id] = vm

        self._type_map = map
        self._type_vm = map_vm

    security.declareProtected(View, 'getDefaultForum')
    def getDefaultForum(self):
        """ """
        return self._default

    security.declareProtected(View, 'getForumForType')
    def getForumForType(self, tp):
        """ """
        return self._type_map.get(tp, '')

    security.declareProtected(View, 'getViewModeForType')
    def getViewModeForType(self, tp, default=0):
        """ """
        try:
            vm = self._type_vm.get(tp, '')
            if vm or default:
                return vm
        except:
            pass

        return self._default_vm

    security.declarePublic('getViewMode')
    def getViewMode(self, content):
        """ """
        return self.TypeViewMode(content.getPortalTypeName())

    security.declareProtected(View, 'getDefaultViewMode')
    def getDefaultViewMode(self):
        """ """
        return self._default_vm
    
    security.declarePublic( 'getDiscussionFor' )
    def getDiscussionFor(self, content):
        """
            Return the talkback for content, creating it if need be.
        """
        try:
            if cb_isinstance(content, DiscussionItemWrapper):
                pass
            else:
                if not self.isDiscussionAllowedFor( content ):
                    raise DiscussionNotAllowed
        except:
            if not self.isDiscussionAllowedFor( content ):
                raise DiscussionNotAllowed

        talkback = getattr( content, 'talkback', None )

        if talkback is None:
            return self._createDiscussionFor( content )
        else:
            try:
                if talkback.meta_type != ForumDiscussionContainer.meta_type:
                    # try convert
                    if self.auto_converting:
                        return convert(self, content, talkback)

                return talkback
            except:
                return self._createDiscussionFor( content )

        return talkback

    security.declarePublic( 'overrideDiscussionFor' )
    def overrideDiscussionFor(self, content, allowDiscussion):
        """ """
        mtool = getToolByName( self, 'portal_membership' )
        if not mtool.checkPermission(ModifyPortalContent, content):
            raise "Unauthorized"

        if allowDiscussion is None or allowDiscussion == 'None':
            if hasattr(content, 'allow_discussion'):
		try:
            	    del content.allow_discussion
		except:
		    pass
        else:
            content.allow_discussion = int(allowDiscussion)

    security.declarePublic( 'isDiscussionAllowedFor' )
    def isDiscussionAllowedFor( self, content ):
        '''
            Returns a boolean indicating whether a discussion is
            allowed for the specified content.
        '''
        if content.meta_type == 'DiscussionItemWrapper':
            return 1
        
        allowed = 0
        object = content
        portal = aq_parent(aq_inner(self))

        # disable discussion for filtered types
        if content.portal_type in portal.portal_cmfboard.filtered_discussion_types:
            return 0

        # check published
        if self.only_published:
            try:
                if self.portal_workflow.getInfoFor(content,'review_state') != 'published':
                    return 0
            except:
                return 0

        # check default
        if hasattr( content, 'allow_discussion' ):
            return content.allow_discussion
        typeInfo = getToolByName(self, 'portal_types').getTypeInfo( content )
        if typeInfo:
            return typeInfo.allowDiscussion()

        # check discussion filter
        while 1:
            object = aq_parent(aq_inner(object))

            filter = getattr(object, 'discussion_filters', None)

            if filter is not None:
                if not cb_isinstance(filter, ForumDiscussionFilter):
                    continue

                topic_rec = filter.isDiscussionAllowedFor(content, portal)
                if topic_rec['status'] == None:
                    # goto next filter
                    continue
                else:
                    # return state topic
                    allowed = topic_rec['allowed']
                    break
            else:
                break

        return allowed


    security.declarePrivate( '_createDiscussionFor' )
    def _createDiscussionFor( self, content ):
        """ """
        if not self.isDiscussionAllowedFor( content ):
            raise DiscussionNotAllowed

        content.talkback = ForumDiscussionContainer(getattr(aq_base(self),'talkback',None))
        content.talkback.manage_afterAdd(content.talkback, content)
        return content.talkback

    security.declareProtected(ModifyPortalContent, 'cookReply')
    def cookReply(self, reply, text_format=None):
        """ """
        if text_format is not None:
            return

        try:
            reply.getField('text_format').set(reply, text_format)
        except:
            pass


InitializeClass( ForumDiscussionTool )
