__version__ = '$Revision: 1.56.4.5.2.14 $'[11:-2]

import traceback
from cStringIO import StringIO
from Acquisition import aq_base
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.CMFBoard import config
from Products.CMFBoard.config import *
from Products.CMFBoard.ForumPermissions import *
from Products.CMFBoard.DiscussionTool import ForumDiscussionTool
from Products.CMFBoard.ForumWorkflow import \
     topic_workflow_id, topic_workflow, forum_workflow_id, forum_workflow, \
     message_workflow_id, message_workflow
from Products.CMFBoard.mailtemplates import templates
from Products.CMFBoard.CacheManagerMisc import manage_addRAMCacheManagerMisc
from Products.CMFBoard.UtilitiesTool import UtilitiesTool

try:
    from Products.PortalTransport.utils import install_mail_templates
except:
    def install_mail_templates(self, portal, template, out):
        pass
    

def install(self):
    out = StringIO()

    portal = getToolByName(self, 'portal_url').getPortalObject()

    installTypes(self, out, listTypes(PROJECTNAME), PROJECTNAME)
    install_subskin(self, out, GLOBALS)
    install_global_properties(self, out)

    #if hasattr(portal, 'portal_quickinstaller'):
    #   try:
    #	    portal.portal_quickinstaller.installProduct('PortalTransport')
    #	except:
    #       pass

    if not hasattr(portal, 'CMFBoardCache2'):
        try:
            manage_addRAMCacheManagerMisc(portal)
        except:
            pass

    if not hasattr(portal, UtilitiesTool.id):
        portal.manage_addProduct[PROJECTNAME].manage_addTool(
            UtilitiesTool.meta_type, None)
        print >> out, "Successfully installed CMFBoard Tool."

    install_skins(self, portal, out)
    install_workflow(self, out)
    install_member_properties(self, out)

    try:
        install_mail_templates(self, portal, templates, out)
    except:
        pass

    # register folderish classes in use_folder_contents
    props = portal.portal_properties.site_properties
    use_folder_tabs = list(props.use_folder_tabs)
    if 'ForumFolder' not in use_folder_tabs:
        use_folder_tabs.append('ForumFolder')
        props.use_folder_tabs=tuple(use_folder_tabs)

    # don't show ForumMessage in navtree
    props = portal.portal_properties.navtree_properties
    metaTypesNotToList = list(props.metaTypesNotToList)
    for tp in ('ForumPoll','ForumTopic',
               'ForumContentTopic','ForumMessage','ForumContentMessage'):
        if tp not in metaTypesNotToList:
            metaTypesNotToList.append(tp)
    props.metaTypesNotToList = tuple(metaTypesNotToList)

    # install left slots
    left_slots = list(getattr(portal, 'left_slots', []))
    for portlet in portlets:
        if portlet not in left_slots:
            try:
                left_slots = left_slots + [portlet]
            except:
                left_slots = [portlet]
    portal.left_slots = tuple(left_slots)

    print >> out, "Successfully installed left slots."

    install_actions(self, portal, out)

    print >> out, "Successfully installed %s." % PROJECTNAME
    
    return out.getvalue()

def uninstall(self):
    out = StringIO()

    # uninstall configs
    try:
        self.portal_controlpanel.unregisterApplication(PROJECTNAME)
    except:
        pass

    # uninstall left_slots
    portal = getToolByName(self, 'portal_url').getPortalObject()
    left_slots = list(getattr(portal, 'left_slots', []))
    
    for portlet in portlets + menu_portlets:
        try:
            left_slots.remove(portlet)
        except:
            pass
    portal.left_slots = tuple(left_slots)
    print >> out, "Successfully uninstalled left slots."

    # unregister folderish classes in use_folder_contents
    props = portal.portal_properties.site_properties
    use_folder_tabs=list(props.use_folder_tabs)
    if 'ForumFolder' in use_folder_tabs:
        use_folder_tabs.remove('ForumFolder')
        props.use_folder_tabs=tuple(use_folder_tabs)

    print >> out, "Successfully uninstalled %s." % PROJECTNAME
    return out.getvalue()

def install_actions(self, portal, out):
    try:
        portal.portal_controlpanel.registerConfiglet(**cmfboard_configlet)
    except:
        pass

    try:
        portal.portal_controlpanel.registerConfiglet(**forum_preferences)
    except:
        pass

def install_workflow(self, out):
    wtool = self.portal_workflow

    for r in forumRoles:
        try:
            wtool._addRole(r)
        except:
            pass

    if topic_workflow_id in wtool.objectIds():
        wtool._delObject(topic_workflow_id)

    wtool.manage_addWorkflow(id=topic_workflow_id, workflow_type=topic_workflow)
    wtool.setChainForPortalTypes(('ForumPoll',), topic_workflow_id)
    wtool.setChainForPortalTypes(('ForumTopic',), topic_workflow_id)
    wtool.setChainForPortalTypes(('ForumContentTopic',), topic_workflow_id)

    if forum_workflow_id in wtool.objectIds():
        wtool._delObject(forum_workflow_id)

    wtool.manage_addWorkflow(id=forum_workflow_id, workflow_type=forum_workflow)
    wtool.setChainForPortalTypes(('ForumNB',), forum_workflow_id)
    wtool.setChainForPortalTypes(('ForumFolder',), 'folder_workflow')

    if message_workflow_id in wtool.objectIds():
        wtool._delObject(message_workflow_id)

    wtool.manage_addWorkflow(id=message_workflow_id, workflow_type=message_workflow)
    wtool.setChainForPortalTypes(('ForumMessage',), message_workflow_id)
    wtool.setChainForPortalTypes(('ForumContentMessage',), message_workflow_id)

    wtool.setChainForPortalTypes(('ForumAttachment',), '')
    
    # in case the workflows have changed, update all workflow-aware objects
    #wtool.updateRoleMappings()

    print >> out, "Successfully installed custom workflows."


def install_member_properties(self, out):
    mdtool = getToolByName(self, 'portal_memberdata')
    for prop, tp, val in config.memberData_properties:
        if not mdtool.hasProperty(prop):
            mdtool._setProperty(prop, val, tp)

    print >> out, "Successfully installed member properties."


def install_global_properties(self, out):
    # install cmfboard global properties
    if not hasattr(self.portal_properties, 'cmfboard_properties'):
        self.portal_properties.addPropertySheet(
            'cmfboard_properties', 'CMFBoard properties')

    props = self.portal_properties.cmfboard_properties
    for prop, tp, val in config.cmfboard_properties:
        if not props.hasProperty(prop):
            props._setProperty(prop, val, tp)

    print >> out, "Successfully installed global forum properties."

def install_skins(self, portal, out):
    dtool = portal.portal_discussion
    skinstool = portal.portal_skins

    skins = skinstool.getSkinSelections()

    default = skinstool.getDefaultSkin()
    default_path = [i.strip() for i in skinstool.getSkinPath(default).split(',')]

    for skinName, skinPath in predefined_skins_path:
        if skinName not in skins:
            path=list(default_path)
            try:
                idx = path.index('custom')+1
                path = path[:idx] + list(skinPath) + path[idx:]
            except ValueError:
                path = path + list(skinPath)

            path = ','.join(path)
            skinstool.addSkinSelection(skinName, path)
    
    print >> out, "Successfully installed forum skins."
    

def dump(self):
    out = StringIO()
    print >>out, '-------------- begin dump ---------------------'
    print >>out, '----- object: %s ----------------'%'/'.join(self.getPhysicalPath())
    print >>out, '----- classes: %s ---------------'%str(self.__class__.__bases__)
    print >>out, '----- classes: %s ---------------'%str(self.__class__)

    items = self.__dict__.items()
    items.sort()
    for key, val in items:
        print >>out, key, repr(val), '<br />'

    print '--------------------------------------------------'

    return out.getvalue()

from Products.CMFBoard.ForumMessage import ForumMessage
from Products.CMFBoard.ForumContentTopic import ForumContentMessage

def fix_portal_type(self):

    out = StringIO()

    for brain in self.portal_catalog(portal_type='ForumNB'):
        try:
            forum = brain.getObject()
            if forum is None:
                continue
        except:
            continue

        for topic_id in forum.objectIds():
            topic = getattr(forum, topic_id, None)
            if topic is None:
                continue

            msg = getattr(topic, '0000')
            if msg is None:
                continue

            if isinstance(msg, ForumMessage) and msg.portal_type == 'ForumContentMessage':
                print >>out, msg.absolute_url()
                msg.portal_type = 'ForumMessage'
                msg.indexObject()
            elif isinstance(msg, ForumContentMessage) and msg.portal_type == 'ForumMessage':
                print >>out, msg.absolute_url()
                msg.portal_type = 'ForumContentMessage'
                msg.indexObject()

        try:
            del topic._tree_unindex[1970255172]
        except:
            pass

        index = topic._tree_index
        for k, v in index.items():
            try:
                v.remove(1970255172)
            except:
                pass
                
    return out.getvalue()
