# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.kz> 
"""
$Id: migration21to22.py,v 1.1.2.20 2005/05/26 03:52:06 fafhrd91 Exp $
"""

import traceback, sys
from Acquisition import aq_base
from Globals import PersistentMapping
from AccessControl.SecurityManagement import getSecurityManager
from Products.CMFCore.utils import getToolByName
from Products.CMFBoard.HitsCounter import hits_counter
from Products.CMFBoard.ForumMessage import initForumMessage, ForumMessage
from Products.CMFBoard.ForumContentTopic import ForumContentMessage
from OFS.ObjectManager import bad_id

def check_filename(msg, filename):
    new_filename = ''
    for ch in filename:
        if bad_id(ch):
            new_filename += '-'
        else:
            new_filename += ch

    idx = 0
    prefix = ''
    while hasattr(aq_base(msg), prefix + new_filename):
        prefix = '%d'%idx
        idx += 1

    return prefix + new_filename


def migrate21To22(self, out):
    """ Migrate CMFBoard 2.1 to 2.2 """
    out = sys.stdout    

    atool = self.portal.archetype_tool
    wftool = self.portal.portal_workflow
    wf = wftool.getWorkflowById('forum_message_workflow')

    self.portal.REQUEST.set('count_user_posts', 0)

    for forum_brain in self.forum_brains:
        try:
            forum_obj = forum_brain.getObject()
            if forum_obj is None:
                continue
        except:
            continue

        #print >>out, forum_obj.absolute_url()
        
        major, minor = forum_obj._internal_version

        if major == 2 and minor == 1:
            # migrate Attachments size
            file_size = getattr(forum_obj, 'file_size', 0)
            image_size = getattr(forum_obj, 'image_size', 0)
            if file_size > image_size:
                forum_obj.attachment_size = file_size
            else:
                forum_obj.attachment_size = image_size

            #print >>out, "Forum file_size and image_size migrated"

            # Migrate hit counter
            if hasattr(aq_base(forum_obj), 'hits'):
                try:
                    length = forum_obj.__dict__['hits']()
                    hits_counter.set(forum_obj.UID(), length, forum_obj._getParent())
                    del forum_obj.hits
                except:
                    pass

            #print >>out, "Forum hits_count migrated"

            for topic_id in forum_obj.objectIds():
		topic = forum_obj._getOb(topic_id)
                if hasattr(aq_base(topic), 'hits'):
                    try:
                        length = topic.__dict__['hits']()
                        hits_counter.set(topic.UID(), length, topic._getParent())
                        del topic.hits
                    except:
                        pass

            #print >>out, "Forum topics hits_count migrated"

            tool = getToolByName(topic, 'portal_types')
            tp_msg = tool.getTypeInfo('ForumMessage')
            tp_cmsg = tool.getTypeInfo('ForumContentMessage')

            # Migrate topic to '0000' message
            for topic_id in forum_obj.contentIds():
		topic = forum_obj._getOb(topic_id)
                topic.rebuildIndex(0)
                topic_created = topic.creation_date
                topic_modified = topic.modification_date

                if hasattr(aq_base(topic), '0000'):
                    continue

                #print >>out, "Begin migrate topic %s"%topic.absolute_url()

                content_topic = topic.portal_type == 'ForumContentTopic'

                if content_topic:
                    kwargs = {
                        'id': '0000',
                        'title': topic.title,
                        'text_text_format': 'text/plain',
                        'authorname': '', 'message_icon': '', 'parsing_flag': 0,
                        'signature': 0
                        }
                else:
                    kwargs = {
                        'id': '0000',
                        'title': topic.title,
                        'text': topic.text.getRaw(),
                        'text_text_format': topic.text.getContentType(),
                        'authorname': topic.authorname,
                        'message_icon': getattr(topic,'message_icon',''),
                        'parsing_flag': getattr(topic,'parsing_flag',0),
                        'signature': topic.signature
                        }
                if type(kwargs['text_text_format']) != type(''):
                    new_mt = str(aq_base(kwargs['text_text_format']))
                    if new_mt[0] == '<':
                        if kwargs['text'].find('<br />'):
                            kwargs['text_text_format'] = 'text/html'
                        else:
                            kwargs['text_text_format'] = 'text/plain'
                    else:
                        kwargs['text_text_format'] = new_mt
                    #print >>out, 'mimetype rewrited'
                    
                # create first message
                if content_topic:
                    message = ForumContentMessage('0000')
                    kwargs['message_type'] = 'ForumContentMessage'
                else:
                    message = ForumMessage('0000')
                    kwargs['message_type'] = 'ForumMessage'

                setattr(topic, '0000', message)
                message = getattr(topic, '0000')
                message.manage_fixupOwnershipAfterAdd()

                if hasattr(message, '__ac_local_roles__'):
                    if message.__ac_local_roles__ is None:
                        user=getSecurityManager().getUser()
                        if user is not None:
                            name=user.getUserName()
                            if name != 'Anonymous User':
                                message.manage_setLocalRoles(name, ['Owner'])

                message.manage_afterAdd(message, topic)
                message = initForumMessage(topic, message, **kwargs)
                message._anonymous = getattr(aq_base(topic), '_anonymous', 0)

                if not content_topic:
                    message.file = getattr(aq_base(topic), 'file', None)
                    message.image = getattr(aq_base(topic), 'image', None)
                else:
                    # migrate discussion item reference
                    obs = topic.getRefs('cmfboard_topic')
                    if obs:
                        talkback = obs[0]
                        talkback.topic_uid = topic.UID()
                        topic.content_uid = talkback.UID()
                        topic.deleteReference(talkback, 'cmfboard_topic')

                try: del topic.file
                except: pass

                try: del topic.image
                except: pass
                
                try: del topic.image_thumbnail
                except: pass

                if content_topic:
                    tp = tp_cmsg
                else:
                    tp = tp_msg

                message = tp._finishConstruction(message)
                message.creation_date = topic.creation_date
                message.modification_date = topic.creation_date

                # change ownership
                user = topic.getWrappedOwner()
                if user:
                    message.changeOwnership(user)

                #print >>out, "Complete 0000 message migration"
                    
                # Migrate tree
                def migrateTree(message):
                    """ """
                    children = message.getBRefs('parent')
                    if type(children) is not (type([]) or type(())):
                        children = [children,]

                    if message.portal_type in ('ForumMessage', 'ForumContentMessage'):
                        parent = getattr(message.getTopic(), '0000', message)
                    else:
                        parent = message
                        message = getattr(message.getTopic(), '0000')
                        
                    for ob in children:
                        #print ob, ob.id, parent.getId()
                        topic.insertObject(ob)
                        topic.removeMessageFromTree(ob)
                        topic.insertMessageIntoTree(ob, parent)
                        migrateTree(ob)

                migrateTree(topic)

                #print >>out, "Complete topic tree migration"
                
                # Migrate attachemnts
                for msg in [getattr(topic, '0000')] + topic.contentValues():
                    file = getattr(aq_base(msg), 'file', None)
                    image = getattr(aq_base(msg), 'image', None)

                    if file is not None:
		        data = getattr(aq_base(file), 'data', '')
                        mimetype = file.content_type
                        filename = file.filename.strip().split('\\')[-1]

                        if not filename:
                            filename = 'file'
                        filename1 = check_filename(msg, filename)
                            
                        msg.invokeFactory('ForumAttachment', filename1)
                        attach = getattr(msg, filename1)
                        attach._setFile(str(data),filename=filename,mimetype=mimetype)
                        if user:
                            attach.changeOwnership(user)
                        del msg.file
                        
                    if image is not None:
                        data = getattr(aq_base(image), 'data', '')
                        mimetype = image.content_type
                        filename = image.filename.strip().split('\\')[-1]
                        if not filename:
                            filename = 'image'
                        filename1 = check_filename(msg, filename)
			msg.invokeFactory('ForumAttachment', filename1)
                        attach = getattr(msg, filename1)
                        attach._setFile(str(data),filename=filename,mimetype=mimetype)
                            
                        if user:
                            attach.changeOwnership(user)
                        del msg.image

                    # store originals dates
                    cooked_text = getattr(aq_base(msg), 'cooked_text', '')
                    created = msg.creation_date
                    modified = msg.modification_date
                    try:
                        msg._updateSchema()
                    except:
                        #print >>out, 'error'
                        pass

                    msg.cooked_text = cooked_text
                    msg.creation_date = created
                    msg.modification_date = modified

                    # update permissions for messages
                    wf.updateRoleMappingsFor(message)
                    if hasattr(aq_base(message), 'reindexObject'):
                        # Reindex security-related indexes
                        try:
                            # fix _md archetype atributte error
                            if not hasattr(aq_base(self), '_md'):
                                msg._md = PersistentMapping()
                                msg._p_changed = 1

                            msg.reindexObject(idxs=['allowedRolesAndUsers'])
                        except TypeError:
                            pass

                    # check if msg in topic tree
                    # during Archetypes migration we can lost references
                    # if so while tree migration not all messages processed
                    if not topic.isInTree(msg.getId()):
                        #add to topic tree
                        topic.insertMessageIntoTree(msg)

                try:
                    topic._updateSchema()
                except:
                    #print >>out, 'topic error'
                    pass

                topic.creation_date = topic_created
                topic.modification_date = topic_modified
                topic.rebuildIndex()

                #print >>out, "Complete message file and image migration"
                #print >>out, "Topic migration completed succesfull"
                #print >>out
            
            # Migrate forum catalog
            try:
                forum_obj._delObject('forum_catalog')
            except:
                pass
            
            forum_obj.rebuildIndex()

            try:
                forum_obj._updateSchema()
            except:
                traceback.print_exc()
                #print >>out, 'forum error'

    # set version as 2.2
    self.setInternalVersion((2,2))
