import traceback
from Acquisition import aq_base
from Products.CMFCore.utils import getToolByName
from BTrees.OOBTree import OOBTree
from BTrees.Length import Length
from cStringIO import StringIO

from migration21to22 import migrate21To22

class MigrationManager:
    """
    Manage migration between version of CMFBoard.
    """
    
    def __init__(self, portal):
        self.portal = portal
        # Get portal_catalog
        portal_catalog = getToolByName(self.portal, 'portal_catalog')
        
        # Get all forum folders
        self.forum_folder_brains = portal_catalog(meta_type='ForumFolder')
        
        # Get all forumNb
        self.forum_brains = portal_catalog(meta_type='ForumNB')
        
        # Get all topics
        self.forum_topic_brains = portal_catalog(meta_type='ForumTopic')
        
        # Get all polls
        self.forum_poll_brains = portal_catalog(meta_type='ForumPoll')
        
        # Get all messages
        self.forum_message_brains = portal_catalog(meta_type='ForumMessage')
    
    
    def setInternalVersion(self, internal_version, force=0):
        """
        Set _internal_version for forum folder and forumnb
        """
        
        new_major, new_minor = internal_version
        
        for forum_folder_brain in self.forum_folder_brains:
            try:
                folder_obj = forum_folder_brain.getObject()
                if folder_obj is None:
                    continue # For ghost objects
            except:
                continue # For ghost objects
                
            major, minor = getattr(aq_base(folder_obj), '_internal_version', (0,0))
            
            if (force or (new_major > major) or
                ((new_major == major) and (new_minor >= minor))):
                folder_obj._internal_version = internal_version
                
        for forum_brain in self.forum_brains:
            try:
                forum_obj = forum_brain.getObject()
                if forum_obj is None:
                    continue
            except:
                continue # For ghost objects
                
            major, minor = getattr(aq_base(forum_obj), '_internal_version', (0,0))

            if (force or (new_major > major) or
                ((new_major == major) and (new_minor >= minor))):
                forum_obj._internal_version = internal_version

    def migrate14To20(self, out):
        """
        Migrate CMFBoard 1.4 to 2.0
        """
        
        # Migrate schema
        for forum_brain in self.forum_brains:
            try:
                forum_obj = forum_brain.getObject()
                if forum_obj is None:
                    continue
            except:
                continue # For ghost objects
            
            if not hasattr(aq_base(forum_obj), '_internal_version') or aq_base(forum_obj)._internal_version == (1, 4):
                if hasattr(aq_base(forum_obj), 'forum_email'):
                    del forum_obj.forum_email
                
                if not hasattr(aq_base(forum_obj), 'replies'):
                    forum_obj.replies = Length()
            
        # set version as 2.0
        self.setInternalVersion((2,0))
    
    def migrate20To21(self, out):
        """
        Migrate CMFBoard 2.0 to 2.1
        """
        
        # Migrate poll
        for forum_poll_brain in self.forum_poll_brains:
            try:
                forum_poll_obj = forum_poll_brain.getObject()
                if forum_poll_obj is None:
                    continue
            except:
                continue # for ghost objects
            
            major, minor = forum_poll_obj.getForum()._internal_version
            
            if major == 2 and minor == 0:
                if not hasattr(forum_poll_obj, '_member_votes'):
                    forum_poll_obj._member_votes = OOBTree()

        # set version as 2.1
        self.setInternalVersion((2,1))

    def migrate21To22(self, out):
        migrate21To22(self, out)
    
    def migrate(self, out, force=0):
        """
        Migrate all CMFboard objects
        """

        self.setInternalVersion((1,4), force)
            
        self.migrate14To20(out)
        self.migrate20To21(out)
        self.migrate21To22(out)
        
        
    
def migrate(portal, force=0):
    out = StringIO()

    # block message send
    portal.REQUEST.block_subscription = 1
    
    # Migrate system
    migration = MigrationManager(portal)
    migration.migrate(out, force)

    #raise 'Error', 'error'
    
    print >>out, "CMFBoard Migration Successful"
    return out.getvalue()
