# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
"""
$Id: ForumAttachment.py,v 1.8.2.7 2005/05/26 03:52:06 fafhrd91 Exp $
"""

import traceback
from cStringIO import StringIO

from OFS.Image import Image, File
from OFS.ObjectManager import bad_id
from ZPublisher.HTTPRequest import FileUpload

from Acquisition import aq_parent, aq_base
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.CMFCore.CMFCorePermissions import View

from utils import cb_isinstance


try:
    import PIL.Image
    has_pil=1
except:
    # no PIL, no scaled versions!
    has_pil=None

schema = BaseSchema + Schema((
    StringField('title', searchable=1,
                widget=StringWidget(label_msgid='label_title',
                                    description_msgid='help_title',
                                    i18n_domain='plone')),

    StringField('description',
                searchable = 1,
                widget = TextAreaWidget(label_msgid = 'label_description',
                                        description_msgid = 'help_description',
                                        i18n_domain = 'plone')),
    FileField('file',
              validators=('checkAttachmentSize',),
              mutator='_setFile',
              widget = FileWidget(label_msgid = 'label_file',
                                  description_msgid = 'help_file',
                                  i18n_domain = 'plone')),
    ))


class ForumAttachment(BaseContent):
    """Attachment in ForumMessage """
    
    security = ClassSecurityInfo()

    schema = schema
    global_allow = 0

    image = None
    
    def _setFile(self, value, **kwargs):
        file = self.getField('file')
        file.set(self, value, **kwargs)

        try:
            data = str(file.get(self).data)
        except:
            self.image = None
            return
        
        mimetype = file.getContentType(self)
        if has_pil and mimetype.startswith('image/'):
            w, h = 128, 128
            keys = {'height':int(h), 'width':int(w)}

            pilfilter = 0
            if PIL.Image.VERSION >= "1.1.3":
                pilfilter = 1

            original_file = StringIO(data)
	    try:
                image = PIL.Image.open(original_file)
        	image = image.convert('RGB')
        	image.thumbnail((128, 128), pilfilter)
        	thumb = StringIO()
        	image.save(thumb, "JPEG", quality=88)
        	thumb.seek(0)
        	self.image = Image('image', '', thumb, 'image/jpeg')
	    except:
		self.image = None
        else:
            self.image = None

        # rename attach to filename
	try:
	    isIDAutoGenerated = self.isIDAutoGenerated(self.getId())
	except:
	    isIDAutoGenerated = 0
        if isIDAutoGenerated:
            filename = self.getId()
            if cb_isinstance(value, FileUpload):
                filename = value.filename
            elif cb_isinstance(value, FileType):
                filename = value.name
            
            new_filename = ''
            for ch in filename:
                if bad_id(ch):
                    new_filename += '-'
                else:
                    new_filename += ch

            idx = 0
            prefix = ''
            while hasattr(aq_base(self), prefix + new_filename):
                prefix = '%d'%idx
            idx += 1

            filename = prefix + new_filename

            if self.REQUEST.form.get('id'):
                return
            elif filename == self.getId():
                return
            elif filename:
                aq_parent(self).manage_renameObject(self.getId(), filename)

    security.declareProtected(View, 'hasThumbnail')
    def hasThumbnail(self):
        return self.image is not None

    security.declareProtected(View, 'getThumbnailTag')
    def getThumbnailTag(self):
        try:
            return self.image.tag()
        except:
            return ''

    security.declareProtected(View, 'getSize')
    def getSize(self):
        file = self.getField('file').get(self)
        try:
            return file.get_size()
        except:
            return 0

    security.declareProtected(View, 'get_size')
    get_size = getSize

    security.declareProtected(View, 'index_html')
    def index_html(self, REQUEST, RESPONSE):
        """ """
        return self.file.index_html(REQUEST, RESPONSE)

    def indexObject(self):
        BaseContent.indexObject(self)
        #self.update_cache('message_changed')

    def unindexObject(self):
        BaseContent.unindexObject(self)
        #self.update_cache('message_changed')

    def reindexObject(self, idxs=[]):
        BaseContent.reindexObject(self, idxs)
        #self.update_cache('message_changed')


registerType(ForumAttachment)
