# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.kz> 
"""
$Id: ForumCatalog.py,v 1.5.6.9 2005/05/24 20:07:35 fafhrd91 Exp $
"""

from types import StringType
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CatalogTool import CatalogTool
from Products.CMFCore.CMFCorePermissions import View, ManagePortal

from BTrees.OOBTree import OOBTree
from BTrees.IOBTree import IOBTree
from BTrees.OIBTree import OISet
from BTrees.IIBTree import IIBTree, IISet, IITreeSet, \
     union, multiunion, intersection, weightedIntersection
from ListSupport import ListSupport, allowedRolesAndUsers


class ForumCatalog(CatalogTool): pass

class ForumCatalogSupport(ListSupport):

    security = ClassSecurityInfo()

    def printIndex(self):
        """ """
        for key, val in self._list_index.items():
            print key, val
            
        for key, val in list(self._review_index.items()):
            print key, list(val.keys())

    def _clear(self):
        ListSupport._clear(self)
        
        # review_state index
        review_index = OOBTree()
        review_index['admin'] = IITreeSet()
        review_index['announce'] = IITreeSet()
        review_index['published'] = IITreeSet()
        review_index['locked'] = IITreeSet()
        self._review_index = review_index

        # sticky index
        self._sticky_on = IITreeSet()
        self._sticky_off = IITreeSet()

    def _get_time_key(self, obj, id, meth='modified', attr='modification_date'):
        return ListSupport._get_time_key(self, obj, id, meth, attr)

    def _insert(self, id, mtime, object):
        ListSupport._insert(self, id, mtime, object)
        
        #=============================
        # add to review_state index
        #=============================
        wtool = getToolByName(self, 'portal_workflow')
        review_state = wtool.getInfoFor(object, 'review_state', '')
        if review_state != 'initial':
            set = self._review_index.get(review_state, None)
            if set is None:
                self._review_index[review_state] = IITreeSet((id,))
            else:
                set.insert(id)

        #=============================
        # add to sticky index
        #=============================
        sticky_flag = object.sticky_flag

        if sticky_flag:
            index = self._sticky_on
        else:
            index = self._sticky_off
        index.insert(id)

    def _remove(self, id, mtime, object):
        ListSupport._remove(self, id, mtime, object)

        # remove from review_state index
        for key, set in self._review_index.items():
            try:
                set.remove(id)
            except KeyError:
                pass

        # remove from sticky index
        for index in (self._sticky_on, self._sticky_off):
            try:
                index.remove(id)
            except KeyError:
                pass

    security.declarePrivate('getObjectsList')
    def getObjectsList(self):
        allowed = None
        user_roles = self._get_user_roles()

        index = self._list_access_index
        for role in user_roles:
            set = index.get(role, None)
            if set is not None:
                allowed = union(allowed, set)
       
        unindex = self._list_unindex
        review_index = self._review_index

        topics1 = review_index['admin']

        if allowed is not None:
            topics1 = intersection(topics1, allowed)

        topics1 = IISet(weightedIntersection(
            unindex, topics1, 1, 0)[1].values()).keys()
        topics1 = list(topics1)
        #print 'admin', topics1

        states = []
        for st in review_index.keys():
            if st != 'admin':
                states.append(review_index[st])

        reviews = multiunion(states)

        #print reviews
        #ListSupport.printIndex(self)
        #print list(self._list_access_index.keys())
        #print allowedRolesAndUsers(self['0002'])
        #print [hash(r) for r in allowedRolesAndUsers(self['0002'])]

        #if allowed is not None:
        #    topics2 = IISet(weightedIntersection(
        #        topics2, allowed, 1, 0)[1].values()).keys()

        topics2 = IISet(weightedIntersection(
            unindex,
            intersection(intersection(self._sticky_on, reviews), allowed), 1, 0
            )[1].values()).keys()
        
        #print 'test1', list(topics2)

        topics2 = list(topics2)
        #print 'on', topics2
        
        topics3 = IISet(weightedIntersection(
            unindex,
            intersection(intersection(self._sticky_off, reviews),allowed), 1, 0
            )[1].values()).keys()
        topics3 = list(topics3)
        #print 'off', topics3
        
        return LazyList(self, topics1, topics2, topics3)

    security.declareProtected(ManagePortal, 'rebuildIndex')
    def rebuildIndex(self):
        """ drop index and rebuild new """
        self._clear()

        for obj in self.contentValues():
            self.insertObject(obj)

InitializeClass(ForumCatalogSupport)


class LazyList:

    def __init__(self, cat, topics1, topics2, topics3):
        self._list = cat

        topics1.reverse()
        topics2.reverse()
        topics3.reverse()

        topics = topics1 + topics2 + topics3

        self._len = len(topics)
        self._topics = topics

    def __len__(self):
        return self._len

    def __getitem__(self, key):
        hashId = self._list._list_index[self._topics[key]]
        return self._list._hash_map[hashId]

    def getNext(self, id):
        id = self._list._list_unindex[hash(id)]

        try:
            val = self[self._topics.index(id) + 1]
            return val
        except:
            return None

    def getPrev(self, id):
        id = self._list._list_unindex[hash(id)]

        try:
            idx = self._topics.index(id) - 1
            if idx >= 0:
                val = self[idx]
                return val
        except:
            pass

        return None
