# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz>
"""
$Id: ForumContentTopic.py,v 1.27.4.3.2.7 2005/05/24 20:07:35 fafhrd91 Exp $
"""

import random, traceback
from Acquisition import aq_parent, aq_base
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.config import TOOL_NAME as ARCHETYPE_TOOL
from Products.Archetypes.interfaces.referenceable import IReferenceable

from config import *
from utils import forum_parser
from ForumTopic import ForumTopic, initForumTopic
from ForumMessage import ForumMessage
from ForumPermissions import ModifyTopic


def addForumContentTopic(self, id, title, description='', sticky=0, **kwargs):
    self._setObject(id, ForumContentTopic(id))
    topic = getattr(self, id)

    topic = initForumTopic(self, topic, title, sticky, description,
                           message_klass=ForumContentMessage,
                           message_type='ForumContentMessage',
                           **kwargs)
    return topic


class ForumContentTopic(ForumTopic):
    """ """
    meta_type = 'ForumContentTopic'
    archetype_name = 'Forum Content Topic'

    global_allow = 0
    filter_content_types = 1

    security = ClassSecurityInfo()

    actions = ForumTopic.actions + (
            { 'id': 'link',
              'name': 'Link',
              'action': 'string:$object_url/forum_content_link_form',
              'permissions': (ModifyTopic,)
              },
            )

    topic_type = 1

    content_uid = None
    content_path = ''

    def _get_content(self):
        if self.content_uid:
            try:
                talkback = self.archetype_tool.lookupObject(self.content_uid)

                # set talkback
                uid = self.UID()
                if (talkback.topic_uid is None) or (talkback.topic_uid != uid):
                    talkback.topic_uid = uid
                    
                return talkback.getContent()
            except:
                pass
            
        else:
            # old system
            obs = self.getRefs('cmfboard_topic')
            if obs:
                talkback = obs[0]
                talkback.topic_uid = self.UID()
                self.content_uid = talkback.UID()
                self.deleteReference(talkback, 'cmfboard_topic')
                
                try:
                    return talkback.getContent()
                except:
                    return talkback
            else:
                return None

        return None

    def initializeArchetype(self, **kwargs):
        if kwargs.has_key('content_uid'):
            # link to DiscussionContainer
            self.content_uid = kwargs['content_uid']

        ForumTopic.initializeArchetype(self, **kwargs)

    security.declareProtected(View, 'Title')
    def Title(self):
        """ """
        try:
            ob = self._get_content()
            return ob.Title()
        except:
            return 'Broken content!!!'

    security.declareProtected(View, 'Description')
    def Description(self):
        """ """
        try:
            ob = self._get_content()
            return ob.Description()
        except:
            return ''

registerType(ForumContentTopic)


class ForumContentMessage(ForumMessage):

    security = ClassSecurityInfo()

    schema = BaseContent.schema

    actions = (
            { 'id': 'view',
              'name': 'View',
              'action': 'string:$object_url/',
              'permissions': (View,)
              },
            )

    Creator = BaseFolder.Creator

    parent = None

    security.declareProtected(View, 'Title')
    def Title(self, encoding=''):
        """ """
        try:
            ob = self.getTopic()._get_content()
            return ob.Title()
        except:
            return 'Broken content!!!'

    security.declareProtected(View, 'Format')
    def Format(self):
        """ return mimetype of text """
        try:
            # strange CMFDefault.DublinCore.Format returns only text/plain or text/html
            content = self.getTopic()._get_content()
            if hasattr(aq_base(content), 'text_format'):
                format = content.text_format
                if format == 'structured-text':
                    return 'text/structured'
            return content().Format()
        except:
            return 'text/html'

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self, encoding=None):
        """ """
        try:
            ob = self.getTopic()._get_content()
            if hasattr(aq_base(ob), 'CookedBody'):
                return ob.CookedBody()
            
            elif hasattr(aq_base(ob), 'getField'):
		try:
            	    return ob.getField('text').get(
                        ob, mimetype='text/html', encoding=encoding)
		except:
		    pass

            content = 'original: [url href="%s"]%s[/url] \n %s'% (
                ob.absolute_url(), getattr(aq_base(ob), 'title', ''),
                ob.Description())
            return forum_parser(self, content)
        except:
            #log_exc("Can't find content")
            return "Can't find content"

    def getSignature(self):
        return 0

registerType(ForumContentMessage)


def modify_fti(fti):
    for a in fti['actions']:
        if a['id'] in ('references', 'metadata', 'edit'):
            a['visible'] = 0
    return fti
