# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz>
"""
$Id: ForumExtensibleSchema.py,v 1.3.4.2 2004/10/29 02:53:25 fafhrd91 Exp $
"""

from Globals import InitializeClass
from ExtensionClass import Base
from Acquisition import ImplicitAcquisitionWrapper, aq_acquire
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from config import *
from ForumPermissions import *

field_types = {
    'StringField':     StringField,
    'IntegerField':    IntegerField,
    'FloatField':      FloatField,
    'FixedPointField': FixedPointField,
    'BooleanField':    BooleanField,
    'LinesField':      LinesField,
    'DateTimeField':   DateTimeField,
    }

widget_types = {
    'String':      lambda :StringWidget(),
    'Select':      lambda :SelectionWidget(format='select'),
    'Flex':        lambda :SelectionWidget(format='flex'),
    'Radio':       lambda :SelectionWidget(format='radio'),
    'Textarea':    lambda :TextAreaWidget(),
    'Calendar':    lambda :CalendarWidget(),
    'Boolean':     lambda :BooleanWidget(),
    'MultiSelect': lambda :MultiSelectionWidget(),
    'Keywords':    lambda :KeywordWidget(),
    'Richtext':    lambda :RichWidget(),
    'Password':    lambda :PasswordWidget(),
    'Lines':       lambda :LinesWidget(),
    'Visual':      lambda :VisualWidget(),
    'Epoz':        lambda :EpozWidget(),
    'Image':       lambda :ImageWidget(),
    'Integer':     lambda :IntegerWidget(),
    'Decimal':     lambda :DecimalWidget(),
    'Reference':   lambda :ReferenceWidget(),
    }

cache = { }

def clear_cache(uid):
    cache = {}
    
    l = len(uid)
    
    for p, k in cache.items():
        if p[:l] == uid:
            del cache[p]


_marker = []

class ForumSchemaHolder(Base):

    _v_ext_schema = None

    def _get_schema(self, uid):
        #print uid
        _v_ext_schema = self._v_ext_schema
        if _v_ext_schema is None:
            return None
        
        return _v_ext_schema.get(uid, None)

    def _set_schema(self, uid, schema):
        _v_ext_schema = self._v_ext_schema
        if _v_ext_schema is None:
            _v_ext_schema = {}
        
        _v_ext_schema[uid] = schema
        self._v_ext_schema = _v_ext_schema

class ForumExtensibleSchema(Base):

    security = ClassSecurityInfo()

    def _get_schema_uid(self):
        uid = self.getPhysicalPath()
        return uid

    # code from PloneColelctorNG
    def _gen_ext_schema(self):
        uid = self._get_schema_uid()
        
        cache_data = cache.get(uid, _marker)
        if cache_data is not _marker:
            return cache_data

        tp = self.portal_type
        semata_default = '%s extension'%tp

        tp = tp.replace(' ', '_').lower()

        meth = getattr(self, '%s_schema_extension'%tp, None)

        if meth is not None:
            res = meth()

            fields = []
            for field_info in res:
                field_name = field_info['name']
                
                field_tp = field_types.get(field_info['type'], None)
                if not field_tp:
                    raise TypeError, 'Unknown filed type'
                
                kw = {'default': field_info.get('default', ''),
                      'schemata': field_info.get('schemata', semata_default),
                      'createindex': field_info.get('createindex', 0),
                      'required': field_info.get('required', 0),
                      }

                widget_tp = field_info['widget']
                widget_meth = widget_types.get(widget_tp, None)
                if not widget_meth:
                    raise TypeError, 'Unkown widget type'

                widget = widget_meth()
                widget.label = field_info.get('label', field_name)
                widget.label_msgid = 'label_' + widget.label
                widget.i18n_domain = 'CMFBoard'

                widget.visible = 1

                kw['widget'] = widget
                
                #if FD.has_key('widgetsize'):
                #    widget.size = FD.widgetsize
                #    widget.rows = 5
                #    widget.cols = 60
        
                #elif FD.has_key('widgetcols') and FD.has_key('widgetrows'):
                #    widget.rows = FD['widgetrows']
                #    widget.cols = FD['widgetcols']
                #    widget.size = 60
                #else:
                #    raise RuntimeError

                # build DisplayList instance for SelectionWidgets
                if widget_tp in ('Radio', 'Select', 'MultiSelect'):
                    vocab = field_info.get('vocabulary', [])

                    # The vocabulary can either be a list of string of 'values'
                    # or a list of strings 'key|value' or a list with *one*
                    # string 'method:<methodname>'. 'method:<methodname>' is used
                    # specify a method that is called to retrieve a DisplayList
                    # instance

                    if len(vocab) == 1 and vocab[0].startswith('method:'):
                        dummy,method = vocab[0].split(':')
                        kw['vocabulary'] = method.strip()
                    else:
                        l = []
                        for line in vocab:
                            line = line.strip()
                            if not line: continue
                            if line.find('|') == -1:
                                k = v = line
                            else:
                                k,v = line.split('|', 1)
                            l.append( (k,v))
                            
                        kw['vocabulary'] = DisplayList(l)

                field = field_tp(field_name, **kw)
                fields.append(field)

            if fields:
                schema = Schema(field)
                cache[uid] = schema
                return schema

        cache[uid] = None
        return None
    
    def Schema(self):
        """ """
        try:
            get_schema = aq_acquire(self, '_get_schema')
        except AttributeError:
            return ImplicitAcquisitionWrapper(self.schema, self)

        uid = self._get_schema_uid()
        _v_ext_schema = get_schema(uid)

        if _v_ext_schema is None:
            ext_schema = self._gen_ext_schema()

            if ext_schema is not None:
                for field in ext_schema.fields():
                    name = field.getName()
                    
                    method = lambda self=self, name=name, *args, **kw: \
                             self.Schema()[name].get(self, *args, **kw)
                    accessor_name = '_v_%s_accessor'%name
                    setattr(self, accessor_name, method)
                    field.accessor = accessor_name
                    field.edit_accessor = accessor_name

                    method = lambda value, self=self, name=name, *args, **kw: \
                             self.Schema()[name].set(self, value, *args, **kw)
                    mutator_name = '_v_%s_mutator'%name
                    setattr(self, mutator_name, method)
                    field.mutator = mutator_name

                    try:
                        field.get(self)
                    except:
                        field.set(self, field.default)

                _v_ext_schema = self.schema.copy() + ext_schema
                _v_ext_schema.initializeLayers(self, None, None)
            else:
                _v_ext_schema = self.schema

            aq_acquire(self, '_set_schema')(uid, _v_ext_schema)

        return ImplicitAcquisitionWrapper(_v_ext_schema, self)

    security.declareProtected(View, 'getExtSchemaValue')
    def getExtSchemaValue(self, name, default=_marker):
        """ """
        # fixme: check security
        try:
            schema = self.Schema()
            return schema[name].get(self)
        except Exception, msg:
            print msg
            return default
        

InitializeClass(ForumExtensibleSchema)


class MessageExtensibleSchema(ForumExtensibleSchema):
    
    def _get_schema_uid(self):
        uid = self.getForum().getPhysicalPath()
        return uid + (self.portal_type,)
