# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz>
"""
$Id: ForumFileField.py,v 1.8.4.2 2004/10/22 05:05:25 fafhrd91 Exp $
"""

from types import FileType
from urllib import quote as url_quote

from OFS.Image import File
from webdav.common import rfc1123_date
from ZPublisher.HTTPRequest import FileUpload
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Field import ObjectField, FileField
from Products.Archetypes.Widget import FileWidget
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View

from utils import cb_isinstance

class ForumFileField(FileField):

    def set(self, instance, value, **kwargs):
        if not value:
            return
        
        if value == 'DELETE_FILE':
            ObjectField.set(self, instance, None, **kwargs)
            return

        try:
            value.seek(0)
        except:
            pass

        if not kwargs.has_key('mimetype'):
            kwargs['mimetype'] = None

        f_name = ''
        if cb_isinstance(value, FileUpload):
            f_name = value.filename
        elif cb_isinstance(value, FileType):
            f_name = value.name

        try:
            isFile = cb_isinstance(value, File) or cb_isinstance(value, ForumFile)
        except:
            isFile = 0

        if isFile:
            mimetype = value.content_type
        else:
            #value, mimetype = self._process_input(
            #    value, default=self.default, **kwargs)
            results = self._process_input(value, default=self.default, **kwargs)
            value    = results[0]
            mimetype = results[1]
            # filename = results[2]
            value = ForumFile(self.getName(), '', value, mimetype, filename=f_name)

        kwargs['mimetype'] = mimetype
        ObjectField.set(self, instance, value, **kwargs)


class ForumFileWidget(FileWidget):

    def process_form(self, instance, field, form, empty_marker=None):
        ## check to see if the delete hidden was selected
        delete = form.get('%s_delete' % field.getName(), empty_marker)
        if delete is not empty_marker: return "DELETE_FILE", {}

        return FileWidget.process_form(self, instance, field, form, empty_marker)


class ForumFile(File):
    """ """

    security = ClassSecurityInfo()

    def __init__(self, id, title, file, content_type='', precondition='', filename=''):
        File.__init__(self, id, title, file,
                      content_type=content_type, precondition=precondition)

        self.filename = filename.strip().split('\\')[-1]

    security.declareProtected(View, 'Filename')
    def Filename(self):
        """ """
        return self.filename

    security.declareProtected(View, 'ContentType')
    def ContentType(self):
        """ """
        return self.content_type

    security.declareProtected(View, 'getMimetypeIcon')
    def getMimetypeIcon(self):
        """ """
        mtr = getToolByName(self, 'mimetypes_registry', None)
        if mtr is None:
            return 'unknown.png'

        mti = mtr.lookup(self.content_type)[0]
        return mti.icon_path

    def index_html(self, REQUEST, RESPONSE):
        """ added Contnt-Disposition header """

        filename = self.filename
        if filename:
            RESPONSE.setHeader('Content-Disposition','inline;filename="%s"' % filename)

        return File.index_html(self, REQUEST, RESPONSE)
