# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim  
"""
$Id: ForumMailReceiver.py,v 1.7.6.2 2005/04/07 17:40:27 fafhrd91 Exp $
"""

import traceback
from ExtensionClass import Base
from Globals import InitializeClass
from Acquisition import aq_base, aq_inner
from AccessControl import ClassSecurityInfo
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.PermissionRole import rolesForPermissionOn

from Products.Archetypes.public import *
from Products.Archetypes.config import TOOL_NAME
from Products.Archetypes.Referenceable import Referenceable

from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFCore.utils import getToolByName

from Products.PortalTransport.interfaces.mailreceiver import IMailReceiver
from Products.PortalTransport.PortalTransportPermissions import AllowSubscription

from email.Header import decode_header

from debug import log
from config import *
from ForumPermissions import *

class MailReceiver( Base ):
    """ """

    __implements__ = (IMailReceiver,)

    security = ClassSecurityInfo()

    security.declarePrivate('message_new')
    def message_new(self, user, msg, tool, charset):
        """ receive mail object """
        # check permission "Add forum topic"
        if not self._checkUserPermission(user, AddTopic, self):
            tool.replyError(PROJECTNAME, error_templates[1],
                            {'forum_title': self.Title(),
                             'forum_url': self.absolute_url(),
                             }, msg, self)
            log("ForumMailReceiver: Member doesn't have permission to add new topic")
            return

        # check subscription
        newSecurityManager(None, user) #fixme: must be removed
        if not self.isSubscribed():
            tool.replyError(PROJECTNAME, error_templates[2],
                            {'forum_title': self.Title(),
                             'forum_url': self.absolute_url(),
                             }, msg, self)
            log('ForumMailReceiver: member not subscribed.')
            return

        # create topic from message
        title = tool.getMessageSubject(msg, charset)
        text, text_format = tool.getParsedBody(msg, charset)
        topic = self.manage_addTopic(title, text,
                                     text_text_format=text_format,
                                     user_object=user)
        self._load_attachments(getattr(topic, '0000'), attachments)
        return

    security.declarePrivate('message_reply')
    def message_reply(self, user, msg, tool, charset):
        """ receive mail object """
        at_tool = getToolByName(self, TOOL_NAME)

        # find message
        uid = decode_header(msg['In-Reply-To'])[0][0][1:-1].split('@')[0];
        message = at_tool.lookupObject(uid)
        log('ForumMailReceiver: lookupKey=|' + decode_header(msg['In-Reply-To'])[0][0][1:-1].split('@')[0] + '|')
        log('ForumMailReceiver: message=|' + str(message) + '|')

        topic = None
        if message is None:
            # maybe uid_catalog lookup failed? Try manual lookup in self:
            topic = getattr(self, uid, None);
            if topic is None:
                if msg.has_key('References'):
                    #try to find topic by references
                    pass
            else:
                log('ForumMailReceiver: found topic in folder.')
        else:
            topic = message.getTopic()

	if topic is None:
            # message not found generate error message
            # TODO: should we insert message as a new topic instead of
            # just creating an error message?
            tool.replyError(PROJECTNAME, error_templates[3],
                            {'forum_title': self.Title(),
                             'forum_url': self.absolute_url(),
                             }, msg, self)
            log('ForumMailReceiver: In-Reply-To message not found.')
            return

        # check if topic is from current forum
        if topic.getForum().UID() != self.UID():
            tool.replyError(PROJECTNAME, error_templates[3],
                            {'forum_title': self.Title(),
                             'forum_url': self.absolute_url(),
                             }, msg, self)
            log('ForumMailReceiver: Topic not found')
            return

        # check permission "Add forum topic reply"
        if not self._checkUserPermission(user, AddTopicReply, topic):
            tool.replyError(PROJECTNAME, error_templates[4],
                            {'forum_title': self.Title(),
                             'forum_url': self.absolute_url(),
                             }, msg, self)
            log("ForumMailReceiver: Member doesn't have permission to add reply")
            return

        # check subscription
        newSecurityManager(None, user) #fixme: must be removed
        if not (self.isSubscribed() or topic.isSubscribed()):
            tool.replyError(PROJECTNAME, error_templates[2],
                            {'forum_title': self.Title(),
                             'forum_url': self.absolute_url(),
                             }, msg, self)
            log('ForumMailReceiver: Member is not subscribed')
            return

        # create reply
        title = tool.getMessageSubject(msg, charset)
        text, text_format = tool.getParsedBody(msg, charset)

        # fixme: need remove user_object!!! see ForumTopic._addReply
        ob = topic.manage_addReply(title, text, uid,
                                   text_text_format=text_format, user_object=user)
        attachments = tool.getAttachments(msg)
        self._load_attachments(ob, attachments)
        return

    def _load_attachments(self, message, attachments):
        """ load message attachments into forum message """
        for attach in attachments:
            id, data, mimetype, disposition = attach
            if disposition == 'attachment':
                ids = message.objectIds()
                new_id = id
                idx = 0
                while new_id in ids:
                    new_id = '%s%s'%(idx, id)

                try:
                    message.invokeFactory('ForumAttachment', new_id)
                    m_attach = message[new_id]
                    m_attach._setFile(attach, mimetype=mimetype)
                except Exception, e:
                    #log_exc(e)
                    pass

    security.declarePrivate('message_command')
    def message_command(self, user, msg, command, tool, portal, charset):
        """ recieve command by email """
        if command == 'subscribe':
            self.addSubscribers((user.getUserName(),))
        elif command == 'unsubscribe':
            self.removeSubscribersByUserId((user.getUserName(),))

    security.declarePrivate('message_bounce')
    def message_bounce(self, user, msg, tool, portal):
        """ recieve bounced email """
        # by default, simply unsubscribe
        log('unsubscribe: ' % user.getUserName())
        self.removeSubscribersByUserId((user.getUserName(),))

    security.declarePrivate('_checkUserPermission')
    def _checkUserPermission(self, user, permission, obj, StringType = type('')):
        roles = rolesForPermissionOn(permission, obj)
        if type(roles) is StringType:
            roles=[roles]
        log('_checkUserPermission: permission='+ str(permission) + ', obj=' + str(obj) + ', roles=' + str(roles) + '.');
        log('user:' + str(user) + ', allowed: ' + str(user.allowed( obj, roles )) + '.')
        if user.allowed( obj, roles ):
            return 1
        return 0

InitializeClass(MailReceiver)
