# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
"""
$Id: ForumMessage.py,v 1.91.4.5.2.25 2005/05/24 20:07:36 fafhrd91 Exp $
"""

import traceback, copy
from Acquisition import aq_inner, aq_base, aq_acquire
from AccessControl import ClassSecurityInfo
from OFS.Image import File, FileUpload
from Products.Archetypes.public import *
from Products.Archetypes.Field import decode, encode

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from AccessControl.PermissionRole import rolesForPermissionOn

from ForumPermissions import *
from ForumExtensibleSchema import MessageExtensibleSchema
from uihelpers.MessageView import MessageView
from config import CACHE_ID2
from forum_codes import parser
from utils import forum_parser, forum_html_tidy, wrap_filename, cb_isinstance

schema = BaseSchema + Schema((
    StringField('title', required=1, searchable=1,
                write_permission=ModifyPortalContent,
                widget=StringWidget(label_msgid='label_title',
                                    description_msgid='help_title',
                                    i18n_domain='plone')),
    
    StringField('authorname', default='',
                write_permission=ModifyPortalContent,
                widget=StringWidget(label='Author name',
                                    label_msgid='label_author_name',
                                    i18n_domain='CMFBoard')),
        
    StringField('message_icon', default='',
                write_permission=ModifyPortalContent,
                widget=StringWidget(label='Message icon')),
    
    BooleanField('parsing_flag', default=1,
                write_permission=ModifyPortalContent,
                widget=BooleanWidget()),

    TextField('text', default='', searchable=1,
                validators = ('checkTextSize',),
                accessor = 'EditableBody', mutator = 'setText',
                write_permission = ModifyPortalContent,
                default_content_type = 'text/plain',
                default_output_type = 'text/html',
                allowable_content_types = ('text/structured',
                                         'text/restructured',
                                         'text/html',
                                         'text/plain'),
                widget=RichWidget(label='Body text')),
    
    IntegerField('signature', default=1,
                write_permission=ModifyPortalContent,
                widget=IntegerWidget()),
    ))


def addForumMessage(self, id, parent=None, **kwargs):
    if self.portal_type in ('ForumTopic','ForumPoll','ForumContentTopic'):
        id = self.getNextId()
        
    self._setObject(id, ForumMessage(id, parent))
    initForumMessage(self, getattr(self, id), message_type='ForumMessage',
                     **kwargs)
    return id

def initForumMessage(self, message, text='',
                     text_text_format='', attach='',
                     message_type='ForumMessage', **kwargs):

    # initialize archetype
    message.initializeArchetype(
        text=text, text_text_format=text_text_format, attach=attach, **kwargs)

    # check anonymous
    mtool = getToolByName(self, 'portal_membership')

    if mtool.isAnonymousUser():
        message._anonymous = 1

    # process text, because if i pass text and text_text_format vars in
    # initializeArchetypes, mimetype of text will be default rather than text_text_format
    if text:
        message.setText(text, mimetype=text_text_format)
	message.reindexObject(('SearchableText', 'Descrption'))

    # create attachment
    if attach:
        f_name = ''
        if cb_isinstance(attach, FileUpload):
            f_name = attach.filename
        elif cb_isinstance(attach, FileType):
            f_name = attach.name

        try:
            attach.seek(0)
        except:
            pass

        fileName = wrap_filename(f_name)
        if fileName:
            message.invokeFactory('ForumAttachment', fileName)
            m_attach = message[fileName]
            m_attach._setFile(attach)

    # init cmf portal type
    tool = getToolByName(self, 'portal_types')
    tp = tool.getTypeInfo(message_type)
    if tp is None:
        raise 'ValueError', 'No such content type: %s'%message_type
    message = tp._finishConstruction(message)
    
    return message


class ForumMessage(MessageExtensibleSchema, BaseFolder, MessageView):

    meta_type = 'ForumMessage'
    archetype_name = 'Forum Message'

    schema = schema

    global_allow = 0
    content_icon = 'discussionitem_icon.gif'
    allowed_content_types = ['ForumAttachment']
    default_page = ''

    actions = (
            { 'id': 'view',
              'name': 'View',
              'action': 'string:$object_url/',
              'permissions': (View,)
              },
            { 'id': 'edit',
              'name': 'Edit',
              'action': 'string:$object_url/forum_message_edit',
              'permissions': (ModifyPortalContent,)
              },
            { 'id': 'reply',
              'name': 'Reply',
              'action': 'string:$object_url/forum_reply_form',
              'permissions': (AddTopicReply,)
             },
            { 'id': 'preview',
              'name': 'Preview',
              'action': 'string:$object_url/forum_message_preview',
              'permissions': (ModifyPortalContent,)
              },
            { 'id': 'delete',
              'name': 'Delete',
              'action': 'string:$object_url/forum_message_delete_form',
              'permissions': (DeleteReply,)
              },
            )

    security = ClassSecurityInfo()

    sended = 1
    message_icon = ''
    parsing_flag = 1
    signature = 1

    _anonymous = 0

    __call__ = MessageView.view
    index_html = None

    parent = '0000'

    def __init__(self, id, parent=None):
        self.parent = parent or '0000'
        BaseFolder.__init__(self, id)

    def _setId(self, *args, **kwargs):
        pass

    security.declareProtected(ModifyPortalContent, 'allowDiscussion')
    def allowDiscussion(self, allowDiscussion=None, **kw):
        """ override Archetype's allowDiscussion method to handle traceback """
        if allowDiscussion is not None:
            try:
                allowDiscussion = int(allowDiscussion)
            except (TypeError, ValueError):
                allowDiscussion = allowDiscussion.lower().strip()
                d = {'on' : 1, 'off': 0, 'none':None, '':None}
                allowDiscussion = d.get(allowDiscussion, None)
        dtool = getToolByName(self, 'portal_discussion')
        try:
            dtool.overrideDiscussionFor(self, allowDiscussion)
        except:
            pass

    def manage_afterAdd(self, item, container):
        BaseFolder.manage_afterAdd(self, item, container)

        # insert message to tree
        topic = self.getTopic()
        topic.insertMessageIntoTree(self, getattr(topic, str(self.parent), None))

        # update member posts count
        if self.REQUEST.get('count_user_posts',1):
            mtool = getToolByName(self, 'portal_membership')
            user = mtool.getAuthenticatedMember()

            if not mtool.isAnonymousUser() and user:
                try:
                    posts = int(user.getProperty('fc_posts', 0)) + 1
                except:
                    posts = 1
                try:
                    user.setMemberProperties({'fc_posts': posts})
                except:
                    pass

            # change forum replies counter
            self.getForum().changeRepliesCount(1)

    def manage_beforeDelete(self, item, container):
        # remove message from tree
        topic = self.getTopic()
        try:
            self.parent = topic.removeMessageFromTree(self)
        except:
            pass

        # update member posts count
        mtool = getToolByName(self, 'portal_membership')
        user = mtool.getMemberById(self.getOwner())
        if user is not None:
            try:
                posts = int(user.getProperty('fc_posts', 1)) - 1
                if posts >= 0:
                    user.setMemberProperties({'fc_posts': posts})
            except:
                user.setMemberProperties({'fc_posts': 0})

        # change forum replies counter
        self.getForum().changeRepliesCount(-1)

        BaseFolder.manage_beforeDelete(self, item, container)

    security.declareProtected(DeleteReply, 'manage_deleteMessage')
    def manage_deleteMessage(self, REQUEST=None):
        """ """
        topic = self.getTopic()
        topic.manage_delObjects([self.getId()])

        try:
            self.update_cache('message_deleted')
        except:
            pass

        return topic

    security.declareProtected(View, 'getEditableData')
    def getEditableData(self):
        """ return message editable data """
        return { 'title': self.Title(),
                 'text': self.getRawText(),
                 'text_format': self.getContentType('text'),
                 'authorname': self.getAuthorname(),
                 'message_icon': self.getMessage_icon(),
                 'parsing_flag': self.getParsing_flag(),
                 'signature': self.getSignature(),
                 }

    security.declareProtected(View, 'Title')
    def Title(self):
        """ """
        return self.getField('title').get(self)

    security.declareProtected(View, 'Format')
    def Format(self):
        """ return mimetype of text """
        return self.getField('text').getContentType(self)

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self, encoding=None):
        """ """
        if hasattr(aq_base(self), 'cooked_text'):
            return encode(self.cooked_text, self, encoding=encoding)
        else:
            return ''

    security.declareProtected(View, 'getParsingFlag')
    def getParsingFlag(self, REQUEST=None):
        """ """
        forum = self.getForum()
        forum_parsing_flag = forum.getField('parsing_flag').get(forum)

        if not forum_parsing_flag:
            if REQUEST is not None:
                return REQUEST.get(
                    'parsing_flag',
                    aq_base(self).getField('parsing_flag').get(self))
            else:
                return aq_base(self).getField('parsing_flag').get(self)
        elif forum_parsing_flag == 'on':
            return 1
        else:
            return 0

    security.declareProtected(View, 'getText')
    def getText(self, **kwargs):
        """ """
        return self.getField('text').get(self, **kwargs)

    security.declareProtected(ModifyPortalContent, 'setText')
    def setText(self, value, mimetype='', **kwargs):
        """ """
        text_field = aq_base(self).getField('text')

        if not mimetype:
            mimetype = text_field.getContentType(self)

        if mimetype == 'text/html':
            # validate html with mxTidy
            value = decode(value, self, **kwargs)
            errors, value, errordata = forum_html_tidy(self, value)

        text_field.set(self, value, mimetype=mimetype)

        # set cooked text and parse smiles, etc
        cooked_text = decode(
            text_field.get(self, mimetype='text/html'), self, **kwargs)

        if self.getParsingFlag(self.REQUEST):
            if mimetype == 'text/plain':
                self.cooked_text = forum_parser(self, value, 1, mimetype, 1)
            else:
                self.cooked_text = forum_parser(self, cooked_text, 1, mimetype)
        else:
            self.cooked_text = cooked_text

        try:
            self.update_cache('message_updated')
        except:
            pass

    security.declareProtected(View, 'getParent')
    def getParent(self):
        """ return parent of this message for tree view """
        return self.getTopic().getParent(self.getId())

    security.declareProtected(View, 'getChildren')
    def getChildren(self):
        """ return children of this message (tree view)"""
        return self.getTopic().getChildren(self.getId())

    security.declareProtected(View, 'Creator')
    def Creator(self):
        """ return creator name it could be anonymous, owner, or member supplied name """
	if hasattr(aq_base(self), 'creator'):
	    if self.creator == 'Anonymous User':
		self._anonymous = 1
		del self.creator

        authorname = aq_base(self).getField('authorname').get(self).strip()
        if self._anonymous:
            if authorname:
                return authorname
            else:
                return 'Anonymous User'
	else:
            if authorname:
                return authorname
            else:
                return BaseFolder.Creator(self)

    security.declareProtected(View, 'isAnonymousCreator')
    def isAnonymousCreator(self):
        """ """
        return self._anonymous

    security.declareProtected(View, 'getMessage')
    def getMessage(self):
        """ """
        return aq_inner(self)

    security.declareProtected(View, 'getSignature')
    def getSignature(self, portal=None):
        """ get member signature for this mesage """
        if portal is None:
            portal = getToolByName(self, 'portal_url').getPortalObject()
        
        cache = None
        try:
            manager = aq_acquire(portal, CACHE_ID2, containment=1, default=None)
        except:
            manager = None
        if manager is not None:
            cache = manager.ZCacheManager_getCache()
            if cache is not None:
                uid = self.UID()
                data = cache.ZCache_get(portal, 'signature', keywords={'UID': uid})
                if data is not None:
                    return data

        sig = ''
        signature = aq_base(self).signature
        if signature and not self._anonymous:
            try:
                signature = int(signature) - 1
            except:
                self.signature = 0
                signature = 0
            mtool = portal.portal_membership

            member = mtool.getMemberById(self.Creator())
            if member:
                member_name = member.getUserName()

                sigs = member.getProperty('signatures', [])
                try:
                    sig = sigs[signature]
                except:
                    sig = ''

        if cache is not None:
            cache.ZCache_set(portal, sig, 'signature', keywords={'UID': uid})

        return sig

    def indexObject(self):
        self.getTopic().insertObject(self)
        BaseFolder.indexObject(self)
        #self.update_cache('message_changed')

    def unindexObject(self):
        self.getTopic().removeObject(self)
        BaseFolder.unindexObject(self)
        #self.update_cache('message_changed')

    def reindexObject(self, idxs=[]):
        self.getTopic().insertObject(self)
        BaseFolder.reindexObject(self, idxs)
        #self.update_cache('message_changed')

    security.declareProtected(View, 'notifyStateChanged')
    def notifyStateChanged(self, sci):
        """ send message to subscribers when message state change """
        try:
            # check for block sending, usually on migration
            if self.REQUEST.block_subscription:
                self.sended = 1
                return
        except:
            pass
        
        sended = self.sended

        if sci.old_state.getId() == 'initial':
            sended = 0
        
        new_state = sci.new_state.getId()

        try:
            # send notification
            if not sended:
                if new_state in ('published','visible','announce','admin','published'):
                    # send message
                    self.sended = 1
                    topic = self.getTopic()
                    parent = topic.getParent(self.getId())
                    parent_msg = getattr(topic, parent)
                    self.getForum().sendMessage(self, parent_msg)

                elif new_state == 'waitreview':
                    # send notification about moderation
                    self.sended = 0
                    self.getForum().sendModeratedNotification(self, self.getParent())
        except:
            traceback.print_exc()
            #log_exc('Sending email to subscribers faild.')

        #ctool = getToolByName(self, 'portal_catalog')
	#print ctool
        #ctool.indexObject(self)

        try:
            self.update_cache('message_updated')
        except:
            pass


registerType(ForumMessage)


def modify_fti(fti):
    for a in fti['actions']:
        if a['id'] in ('references', 'metadata'):
            a['visible'] = 0
    return fti
