# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
"""
$Id: ForumPoll.py,v 1.19.4.6.2.1 2005/01/23 11:34:50 fafhrd91 Exp $
"""

import types, operator
from DateTime import DateTime
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View

from BTrees.Length import Length
from BTrees.IOBTree import IOBTree 
from BTrees.OOBTree import OOBTree

from config import *
from ForumTopic import ForumTopic, initForumTopic
from ForumPermissions import ModifyTopic

def addForumPoll(self, id, title, description='', sticky=0, **kwargs):
    self._setObject(id, ForumPoll(id))
    topic = getattr(self, id)

    return initForumTopic(self, topic, title, sticky,
                          description, 'ForumPoll', **kwargs)


class ForumPoll(ForumTopic):

    meta_type = 'ForumPoll'
    archetype_name = 'Forum Poll'

    global_allow = 0
    filter_content_types = 1
    allowed_content_types = []
    content_icon = 'fcPoll_icon_noans.gif'

    schema = ForumTopic.schema + Schema((
        LinesField('answers', required=1, searchable=1,
                   write_permission=ModifyTopic,
                   widget=LinesWidget(description="A list of alternatives. One alternative per row.",
                                      label_msgid="label_answers",
                                      description_msgid="help_answers",
                                      i18n_domain="CMFBoard"),
                   ),
        IntegerField('minimum_answers', required=1,
                   default=1,
                   write_permission=ModifyTopic,
                   widget=IntegerWidget(description="Minimum answers a user can choose.",
                                        label_msgid="label_minimum_answers",
                                        description_msgid="help_minimum_answers",
                                        i18n_domain="CMFBoard"),
                   ),
        IntegerField('maximum_answers', required=1,
                   default=1,
                   write_permission=ModifyTopic,
                   widget=IntegerWidget(description="Maximum answers a user can choose.",
                                        label_msgid="label_maximum_answers",
                                        description_msgid="help_maximum_answers",
                                        i18n_domain="CMFBoard"),
                   ),
        ))
    
    security = ClassSecurityInfo()

    _cache_invalidates = {
        'poll_changed': {'views': ['forum_poll_view', 'forum_topic_content']},
        }


    def __init__(self, oid, **kwargs):
        ForumTopic.__init__(self, oid, **kwargs)
        self._votes = IOBTree()
        self._member_votes = OOBTree()

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self, encoding=None):
        """ """
        msg = ForumTopic.CookedBody(self, encoding)
        return self.forum_poll_view(message=msg)

    security.declareProtected(View, 'isOpen')
    def isOpen(self):
        """ """
        wtool = getToolByName(self, 'portal_workflow')
        return wtool.getInfoFor(self,'review_state','') != 'locked'
    
    security.declareProtected(View, 'getVote')
    def getVote(self):
        """get the vote"""
        mtool = getToolByName(self, 'portal_membership')

        if mtool.isAnonymousUser():
            request = self.REQUEST

            try:
                return int(request.cookies.get(self.UID()))
            except:
                try:
                    return tuple(request.cookies.get(self.UID()))
                except:
                    return None
        else:
            member_id = mtool.getAuthenticatedMember().getId()
            return self._member_votes.get(member_id, None)
            
    security.declareProtected(View, 'unvote')
    def unvote(self):
        """Cancel a vote"""
        if not self.isOpen():
            return

        answer = self.getVote()
        
        if not answer:
            return

        answer_type = type(answer)

        if answer_type in (types.ListType, types.TupleType):
            for item in answer:
                if self._votes.has_key(item):
                    self._votes[item].change(-1)
        else:
            if self._votes.has_key(answer):
                self._votes[answer].change(-1)
        
        mtool = getToolByName(self, 'portal_membership')
        if mtool.isAnonymousUser():
            response = self.REQUEST.RESPONSE
            response.expireCookie(self.UID(), path='/')
        else:
            member_id = mtool.getAuthenticatedMember().getId()
            del self._member_votes[member_id]
            
        self.update_cache('poll_changed')
        
    security.declareProtected(View, 'vote')
    def vote(self, answer):
        """Register a vote"""
        if not self.isOpen():
            return

        answer_type = type(answer)

        if answer_type in (types.ListType, types.TupleType):
            for item in answer:
                if not self._votes.has_key(item):
                    self._votes[item] = Length()
        
                self._votes[item].change(1)
        elif answer_type in (types.IntType,):
            if not self._votes.has_key(answer):
                self._votes[answer] = Length()
    
            self._votes[answer].change(1)
        else:
            raise TypeError, "Bad type for answer."

        mtool = getToolByName(self, 'portal_membership')
        if mtool.isAnonymousUser():
            response = self.REQUEST.RESPONSE

            e = (DateTime('GMT') + 365).rfc822()
            response.setCookie(self.UID(), str(answer), path='/', expires=e)
        else:
            member_id = mtool.getAuthenticatedMember().getId()
            self._member_votes[member_id] = answer
            
        self.update_cache('poll_changed')

    security.declareProtected(View, 'getVotes')
    def getVotes(self):
        """Result of this poll"""

        result = {'total': reduce(
            operator.add, [votes() for votes in self._votes.values()] + [0,0])}

        scores = []

        i = 1
        for answer in self.getAnswers():
            votes = 0
            percent = 0

            if self._votes.has_key(i) and (result['total'] > 0):
                votes = self._votes[i]()
                percent = int(votes*100.0/result['total'])
            
            scores.append({'answer': answer,
                           'votes': votes,
                           'percent': percent})
            i = i+1
            
        result['scores'] = scores

        return result

registerType(ForumPoll)
