# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz>
"""
$Id: ForumSchema.py,v 1.13.4.1.2.3 2005/05/23 17:35:00 fafhrd91 Exp $
"""

from Globals import InitializeClass
from ExtensionClass import Base
from Acquisition import ImplicitAcquisitionWrapper, aq_acquire
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from config import *
from ForumPermissions import *

forumSchema = BaseSchema + Schema((
    StringField('id', required=1, mode='rw',
                accessor='getId', mutator='setId', default=None,
                write_permission = ManageForum,
                widget=IdWidget(label_msgid='label_name',
                                description_msgid='help_name',
                                i18n_domain='plone')),
    StringField('title', required=1, default='Forums',
                searchable=1,
                write_permission=ManageForum,
                widget=StringWidget(label_msgid='label_title',
                                    description_msgid='help_title',
                                    i18n_domain='plone')),
    ImageField('forum_icon', sizes={},
               widget=ImageWidget(label='Forum icon',
                                  label_msgid='label_forum_icon',
                                  description_msgid='help_forum_icon',
                                  i18n_domain='CMFBoard')),
    BooleanField('moderated', required=0, default=0,
                 schemata = 'security',
                 write_permission=ManageForum,
                 widget=BooleanWidget(
                        label='Moderated forum?',
                        description='Forum managers must approve each post.',
                        label_msgid='label_moderated',
                        description_msgid='help_moderated',
                        i18n_domain='CMFBoard')),
    BooleanField('editable', required=0, default=1,
                 schemata = 'security',
                 write_permission=ManageForum,
                 widget=BooleanWidget(
                        label='Users can edit their posts?',
                        description='Only applies to unmoderated forums.',
                        label_msgid='label_editable_forum',
                        description_msgid='help_editable_forum',
                        i18n_domain='CMFBoard')),
    BooleanField('allow_subscription', required=0, default=1,
                 schemata = 'options',
                 write_permission=ManageForum,
                 widget=BooleanWidget(
                        label='Allow subscription',
                        description='Members receive all new posts via email.',
                        label_msgid='label_allow_subscription',
                        description_msgid='help_allow_subscription',
                        i18n_domain='CMFBoard')),
    BooleanField('allow_topic_subscription', required=0, default=1,
                 schemata = 'options',
                 write_permission=ManageForum,
                 widget=BooleanWidget(
                        label='Allow topic subscription',
                        description='Members receive all new replies '
                                    'to subscribed topic.',
                        label_msgid='label_allow_topic_subscription',
                        description_msgid='help_allow_topic_subscription',
                        i18n_domain='CMFBoard')),

    IntegerField('forum_page_size', required=0, default=20,
                 schemata="design",
                 write_permission=ManageForum,
                 widget=IntegerWidget(
                        label='Forum page size',
                        description='Number of topics on page.',
                        label_msgid='label_forum_page_size',
                        description_msgid='help_forum_page_size',
                        i18n_domain='CMFBoard')),

    IntegerField('topic_page_size', required=0, default=20,
                 schemata="design",
                 write_permission=ManageForum,
                 widget=IntegerWidget(
                        label='Topic page size',
                        description='Number of replies on page.',
                        label_msgid='label_topic_page_size',
                        description_msgid='help_topic_page_size',
                        i18n_domain='CMFBoard')),

    BooleanField('show_userinfo', required=0, default=1,
                 schemata="design",
                 write_permission=ManageForum,
                 widget=BooleanWidget(
                        label='Show userinfo',
                        description='Enable userinfo in message view.',
                        label_msgid='label_show_userinfo',
                        description_msgid='help_show_userinfo',
                        i18n_domain='CMFBoard')),

    BooleanField('show_wizards', required=0, default=1,
                 schemata="design",
                 write_permission=ManageForum,
                 widget=BooleanWidget(
                        label='Show wizards',
                        description='Enable wizards in various edit forms.',
                        label_msgid='label_show_wizards',
                        description_msgid='help_show_wizards',
                        i18n_domain='CMFBoard')),

    BooleanField('show_message_icons', required=0, default=1,
                 schemata="design",
                 write_permission=ManageForum,
                 widget=BooleanWidget(
                        label='Show messageicons',
                        description='Enable message icons in various edit forms.',
                        label_msgid='label_show_message_icons',
                        description_msgid="help_show_message_icons",
                        i18n_domain='CMFBoard')),
    
    BooleanField('hits_counter', required=0, default=1,
                 schemata="options",
                 write_permission=ManageForum,
                 widget=BooleanWidget(
                        label='Views counter',
                        description='Use Views counter. Disable for high load sites.',
                        label_msgid='label_views_counter',
                        description_msgid='help_views_counter',
                        i18n_domain='CMFBoard')),

    BooleanField('read_status', required=0, default=1,
                 schemata="design",
                 write_permission=ManageForum,
                 widget=BooleanWidget(
                        label='Show topic read status',
                        description='Show topic read status. Disable for high load sites.',
                        label_msgid='label_show_read_status',
                        description_msgid='help_show_read_status',
                        i18n_domain='CMFBoard')),

    BooleanField('contact_admin', required=0, default=1,
                 schemata="design",
                 write_permission=ManageForum,
                 widget=BooleanWidget(
                        label='Contact moderator',
                        description='Allow users send message to forum moderator.',
                        label_msgid='label_contact_moderator',
                        description_msgid='help_contact_moderator',
                        i18n_domain='CMFBoard')),

    BooleanField('allow_signatures', required=0, default=1,
                 schemata="design",
                 write_permission=ManageForum,
                 widget=BooleanWidget(
                        label='Allow member signatures',
                        label_msgid='label_allow_signatures',
                        description_msgid='help_allow_signatures',
                        i18n_domain='CMFBoard')),

    StringField('allow_member_names', required=0, default=0,
                 schemata="design",
                 write_permission=ManageForum,
                 vocabulary=anon_name_modes,
                 widget=SelectionWidget(
                        label='Allow members to specify their name',
                        description='If this is set, members will be given a possibility to specify their name/handle instead thier name.',
                        label_msgid='label_allow_member_names',
                        description_msgid='help_allow_member_names',
                        i18n_domain='CMFBoard')),

    StringField('allow_anon_names', required=0, default=0,
                 schemata="design",
                 write_permission=ManageForum,
                 vocabulary=anon_name_modes,
                 widget=SelectionWidget(
                        label='Allow anonymous users to specify their name',
                        description='If this is set, anonymous users will be given a possibility to specify their name/handle instead of simply being called Anonymous.',
                        label_msgid='label_allow_anon_names',
                        description_msgid='help_allow_anon_names',
                        i18n_domain='CMFBoard')),

    BooleanField('view_mode_selectable', required=0, default=1,
                 schemata="design",
                 write_permission=ManageForum,
                 widget=BooleanWidget(
                        label='Allow members to select forum view mode',
                        label_msgid='label_view_mode_selectable',
                        description_msgid='help_view_mode_selectable',
                        i18n_domain='CMFBoard')),

    IntegerField('topic_hot', required=1, default=300,
                 schemata='options',
                 write_permission=ManageForum,
                 widget=IntegerWidget(
                        label="Hot topic 'Views'",
                        description='Count of views to mark topic as hot.',
                        label_msgid='label_hot_topic_views',
                        description_msgid='help_hot_topic_views',
                        i18n_domain='CMFBoard')),

    IntegerField('topic_noreplies', required=1, default=2,
                 schemata='options',
                 write_permission=ManageForum,
                 widget=IntegerWidget(
                        label="Topic with 'No replies'",
                        description='How many days after which  mark topic as with no replies.',
                        label_msgid='label_topic_noreplies_mark',
                        description_msgid='help_topic_noreplies_mark',
                        i18n_domain='CMFBoard')),
    
    StringField('quickreply_form', required=0,
                schemata='design',
                default='here/forum_quickreply_form/macros/edit_form',
                vocabulary=quickreply_forms,
                write_permission=ManageForum,
                widget=SelectionWidget(
                        label='Quick reply form',
                        description='Form used for quick reply in topic(message) view.',
                        label_msgid='label_quickreply_form',
                        description_msgid='help_quickreply_form',
                        i18n_domain='CMFBoard')),
    
    StringField('view_mode', required=0, default='flat',
                schemata='design',
                vocabulary=view_modes,
                write_permission=ManageForum,
                widget=SelectionWidget(
                        label='Default view mode',
                        description='View mode used in topic(message) view.',
                        label_msgid='label_view_mode',
                        description_msgid='help_view_mode',
                        i18n_domain='CMFBoard')),
    
    StringField('wysiwyg_editor', required=0, default='',
                schemata='design',
                vocabulary=wysiwyg_editors,
                write_permission=ManageForum,
                widget=SelectionWidget(
                        label='Default wysiwyg editor',
                        description='Forum managers can restrict text '
                                    'editor to text area editor.',
                        label_msgid='label_wysiwyg_editor',
                        description_msgid='help_wysiwyg_editor',
                        i18n_domain='CMFBoard')),

    StringField('create_poll', required=0, default='Manager', multiValued=1,
               schemata='security',
               accessor='getCreatePollRolesTopic',
                mutator='setCreatePollRolesTopic',
               vocabulary=roles_attache,
               widget=MultiSelectionWidget(label='Who can create polls',
                                           label_msgid='label_who_can_create_polls',
                                           description_msgid='help_who_can_create_polls',
                                           i18n_domain='CMFBoard')),
               

    StringField('attach_topic', required=0, default='Manager', multiValued=1,
               schemata='security',
               vocabulary=roles_attache,
               mutator='setAttachRolesTopic',
               accessor='getAttachRolesTopic',
               widget=MultiSelectionWidget(label='Who can make attachments to Topic',
                                           label_msgid='label_who_can_attache_topic',
                                           description_msgid='help_who_can_attache_topic',
                                           i18n_domain='CMFBoard')),
    
    StringField('attach_reply', required=0, default='Manager', multiValued=1,
               schemata='security',
               vocabulary=roles_attache,
               mutator='setAttachRolesReply',
               accessor='getAttachRolesReply',
               widget=MultiSelectionWidget(label='Who can make attachments to Reply',
                                           label_msgid='label_who_can_attache_reply',
                                           description_msgid='help_who_can_attache_reply',
                                           i18n_domain='CMFBoard')),
    
    BooleanField('owner_as_topic_manager', required=1, default=1,
                 schemata='security',
                 widget=BooleanWidget(label='Owner of topic also is topic manager',
                                      label_msgid='label_owner_topic_manager',
                                      description_msgid='help_owner_topic_manager',
                                      i18n_domain='CMFBoard')),
    
    StringField('parsing_flag', default='',
                schemata="design",
                vocabulary=parsing_flags,
                widget=SelectionWidget(label='Emoticons',
                                       label_msgid='label_emoticons',
                                       description_msgid='help_emoticons',
                                       i18n_domain='CMFBoard')),
    
    IntegerField('text_size', required=1, default=10240,
                 schemata='options',
                 write_permission=ManageForum,
                 widget=StringWidget(label='Max text size (bytes)',
                                     label_msgid='label_text_size',
                                     description_msgid='help_text_size',
                                     i18n_domain='CMFBoard')),

    IntegerField('attachment_size', required=1, default=153600,
                 schemata='options',
                 write_permission=ManageForum,
                 widget=StringWidget(label='Max attachement size (bytes)',
                                     label_msgid='label_attachement_size',
                                     description_msgid='help_attachement_size',
                                     i18n_domain='CMFBoard')),

    TextField('description', required=0, default='',
              searchable=1,
              write_permission=ManageForum,
              widget=TextAreaWidget(label_msgid='label_description',
                                    description_msgid='help_description',
                                    i18n_domain='plone')),
    ))

