# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nikolay Kim <fafhrd@legco.biz>
"""
$Id: ForumTopic.py,v 1.69.2.7.2.20 2005/05/24 20:07:36 fafhrd91 Exp $
"""

import random, traceback
from DateTime import DateTime
from AccessControl import ClassSecurityInfo
from AccessControl.SecurityManagement import getSecurityManager, newSecurityManager
from Acquisition import aq_parent, aq_inner, aq_base
from Products.Archetypes.public import *
from Products.Archetypes.config import TOOL_NAME
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFCore.utils import getToolByName

from config import *
from ForumPermissions import *
from ForumMessage import ForumMessage, addForumMessage, initForumMessage
from ForumExtensibleSchema import MessageExtensibleSchema
from IdGenerator import IdGenerator
from TreeSupport import TreeSupport
from HitsCounter import HitsCounterTopic
from uihelpers.TopicView import TopicView

try:
    from Subscribeable import Subscribeable
except:
    from SubscribeableStub import Subscribeable


schema = BaseSchema + Schema((
    StringField('title', required=1, searchable=1,
                write_permission=ModifyTopic,
                widget=StringWidget(label_msgid='label_title',
                                    description_msgid='help_title',
                                    i18n_domain='plone')),

    StringField('description', required=0, searchable=1,
                write_permission=ModifyTopic,
                widget=TextAreaWidget(label_msgid='label_description',
                                      description_msgid='help_description',
                                      i18n_domain='plone')),

    BooleanField('sticky_flag', default=0,
                mutator='_change_sticky_flag',
                write_permission=ManageTopic,
                widget=BooleanWidget(label='Sticky',
				     label_msgid="label_sticky",
				     description="This topic will be always on head list",
				     description_msgid="help_sticky",
				     i18n_domain='CMFBoard')),
    ))


def addForumTopic(self, id, title, description='', sticky=0, **kwargs):
    self._setObject(id, ForumTopic(id))
    topic = getattr(self, id)

    return initForumTopic(self, topic, title, sticky, description, **kwargs)

def initForumTopic(self, topic, title, sticky,
                   description, portal_type='ForumTopic',
                   message_klass=ForumMessage, message_type='ForumMessage',
                   **kwargs):
    topic.initializeArchetype(title=title, sticky=sticky,
                              description=description, **kwargs)

    # set owner of topic as topic manager and subscribe
    mtool = getToolByName(self, 'portal_membership')
    user = mtool.getAuthenticatedMember()

    if not mtool.isAnonymousUser():
        if user:
            topic.manage_setLocalRoles(user.getId(), [topicManager, 'Owner'])

            if (user.getProperty('fc_autosubscribe', 0) and
                topic.getForum().allowTopicSubscription()):
                topic.subscribe()
    else:
        topic.anonymous = 1

    # it is needed for restoring old UID when import/export
    if kwargs.has_key('uid'):
        setattr(topic, UUID_ATTR, kwargs['uid'])

    # this is for workflows
    if kwargs.has_key('topic_type'):
        topic.topic_type = kwargs['topic_type']

    # init portal type
    tp = self.portal_types.getTypeInfo(portal_type)
    if tp is None:
        raise 'ValueError', 'No such content type: %s'%portal_type
    topic = tp._finishConstruction(topic)

    # create first message

    # need remove this code
    # it needed for first reply through email gate
    ################# start security hack #############################
    user_object = kwargs.get('user_object', None)
    if user_object is not None:
        newSecurityManager(None, user_object)
    ################### end security hack #############################
    
    message = message_klass('0000')
    setattr(topic, '0000', message)
    message = getattr(topic, '0000')
    message.manage_fixupOwnershipAfterAdd()

    # Try to give user the local role "Owner", but only if
    # no local roles have been set on the object yet.
    if hasattr(message, '__ac_local_roles__'):
        if message.__ac_local_roles__ is None:
            user=getSecurityManager().getUser()
            if user is not None:
                name=user.getUserName()
                if name != 'Anonymous User':
                    message.manage_setLocalRoles(name, ['Owner'])

    message.manage_afterAdd(message, topic)
    message = initForumMessage(topic, message, title=title,
                               message_type=message_type, **kwargs)
    return topic


class ForumTopic(BaseBTreeFolder, #MessageExtensibleSchema, 
                 Subscribeable, HitsCounterTopic,
                 TopicView, IdGenerator, TreeSupport):
    """ """
    meta_type = 'ForumTopic'
    archetype_name = 'Forum Topic'

    __implements__ = (BaseBTreeFolder.__implements__ + Subscribeable.__implements__)

    schema = schema

    global_allow = 0
    filter_content_types = 1
    allowed_content_types = ['ForumMessage']

    actions = (
            { 'id': 'view',
              'name': 'View',
              'action': 'string:$object_url/',
              'permissions': (View,)
              },
            { 'id': 'edit',
              'name': 'Edit',
              'action': 'string:$object_url/base_edit',
              'permissions': (ModifyTopic,)
              },
            { 'id': 'delete',
              'name': 'Delete',
              'action': 'string:$object_url/forum_topic_delete',
              'permissions': (ModifyTopic,)
              },
            )

    content_icon = 'fcTopic_icon_noans.gif'
    default_page = ''

    security = ClassSecurityInfo()

    topic_type = 1
    sticky_flag = 0

    __call__ = TopicView.view
    index_html = None

    security.declareProtected(ModifyTopic, 'processForm')

    def __init__(self, id, **kwargs):
        TreeSupport.__init__(self)
        BaseBTreeFolder.__init__(self, id, **kwargs)

    def _change_sticky_flag(self, value):
        try:
            self.sticky_flag = int(value)
        except:
            self.sticky_flag = 0
        self.modification_date = DateTime()
        self.reindexObject(['sticky_flag', 'modified'])

    def _change_locked_flag(self, value):
        self.locked_flag = value
        self.reindexObject(['locked_flag'])

    def edit(self, **kwargs):
        """
        this is just an alias in Archetypes for the update method but we redefine it
        here the edit method in PortalFolder has being called which only accepts the
        title, and description arguements.
        """
        self.update(**kwargs)

    def manage_beforeDelete(self, item, container):
        getattr(self, '0000').manage_beforeDelete(item, container)

        BaseBTreeFolder.manage_beforeDelete(self, item, container)
        HitsCounterTopic.manage_beforeDelete(self, item, container)

    security.declareProtected(AddTopicReply, 'invokeFactory')
    def invokeFactory( self, type_name, id, RESPONSE=None, *args, **kw):
        '''Invokes the portal_types tool.'''
        if type_name == 'ForumMessage':
            return addForumMessage(self, id, *args, **kwargs)
        else:
            raise 'Error', 'content type creation not allowed'
        
    def _addReply(self, id, **kwargs):
        # need remove this code
        # it needed for first reply through email gate
        ################# start security hack #############################
        user_object = kwargs.get('user_object', None)
        if user_object is not None:
            newSecurityManager(None, user_object)
        ################### end security hack #############################

        id = addForumMessage(self, id, **kwargs)

        #message = addForumMessage(self, id, **kwargs)
        message = getattr(self, self.getLastId())

        try:
            self.update_cache('message_new')
        except:
            pass

	return message

    security.declareProtected(AddTopicReply, 'manage_addReply')
    def manage_addReply(self, title='', text='', reply_to='', **kwargs):
        """ """
        tool = getToolByName(self, TOOL_NAME)

        kwargs['id'] = self.getNextId()

        # generate title
        if not title:
            title = self.Title()
            if title[:3] != 'Re:':
                title = 'Re: ' + title

        # get message to reply
        parent = self._getOb(reply_to, None)
        if parent is None:
            parent = getattr(self, reply_to, None)

        if parent is not None:
            kwargs['parent'] = reply_to
        else:
            # set reply to to "last reply"
            msgId = self.getLastObject()
            kwargs['parent'] = self.getLastObject()
            
        kwargs.update({'title':title, 'text': text})

	msg = self._addReply(**kwargs)

        self.modification_date = msg.created()
        self.reindexObject(['replyCount', 'modified'])

        return msg

    security.declareProtected(ModifyTopic, 'manage_deleteTopic')
    def manage_deleteTopic(self, REQUEST=None):
        """ """
        aq_parent(self).manage_delObjects([self.getId()])

    security.declareProtected(View, 'getTopic')
    def getTopic(self):
        """ """
        return aq_inner(self)

    security.declarePublic('getTopicType')
    def getTopicType(self):
        """ """
        return self.topic_type

    security.declareProtected(ModifyTopic, 'setSubject')
    def setSubject(self, subject):
        """ set subject and reindex """
        self.getField('subject').set(self, subject)
        self.reindexObject()

    security.declarePrivate('getRootId')
    def getRootId(self):
        return '0000'

    #########################################
    # HitsCounter API
    #########################################
    security.declareProtected(View, 'replyCount')
    replyCount = lambda obj: obj._count()

    security.declareProtected(View, 'hasReplies')
    hasReplies = replyCount

    security.declareProtected(View, 'Hits')
    def Hits(self):
        """ return hits and inc """
        self.getForum().Hits()
        return HitsCounterTopic.Hits(self)

    security.declareProtected(View, 'hasReplies')
    hasReplies = replyCount
    """ for compatibility with default discussion tool """


    ##########################################
    # forum catalog support
    ##########################################
    security.declareProtected(ModifyPortalContent, 'indexObject')
    def indexObject(self):
        """ """
        BaseBTreeFolder.indexObject(self)
        self.getForum().insertObject(self)

    security.declareProtected(ModifyPortalContent, 'unindexObject')
    def unindexObject(self):
        """ """
        BaseBTreeFolder.unindexObject(self)
        self.getForum().removeObject(self)

    security.declareProtected(ModifyPortalContent, 'reindexObject')
    def reindexObject(self, idxs=[]):
        """ """
        BaseBTreeFolder.reindexObject(self, idxs)
        self.getForum().insertObject(self)

    #########################################
    # when topic state changed,
    # also change first message state
    #########################################
    security.declareProtected(View, 'notifyStateChanged')
    def notifyStateChanged(self, sci):
        """ send message to subscribers when message state change """
        wtool = getToolByName(self, 'portal_workflow')

        action = sci.transition.id

        msg = getattr(self, self.getFirstObject())
        transitions = wtool.getTransitionsFor(msg)

        for trans in transitions:
            if trans['id'] == action:
                wtool.doActionFor(msg, action)

        try:
            self.update_cache('message_updated')
        except:
            pass

registerType(ForumTopic)


def modify_fti(fti):
    for a in fti['actions']:
        if a['id'] in ('references', 'metadata'):
            a['visible'] = 0
    return fti
