"""
$Id: ForumWorkflow.py,v 1.29.4.1.2.6 2005/05/25 04:43:15 fafhrd91 Exp $
"""

from Products.CMFCore.CMFCorePermissions import \
     View, ModifyPortalContent, AccessContentsInformation, \
     ReviewPortalContent, RequestReview

from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Products.DCWorkflow.Transitions import TRIGGER_AUTOMATIC
from Products.PythonScripts.PythonScript import manage_addPythonScript

from ForumPermissions import *

p_access = AccessContentsInformation
p_modify = ModifyPortalContent
p_view = View
p_atopic = AddTopic
p_apoll = AddPoll
p_announce = AddAnnouncement
p_areply = AddTopicReply
p_mforum = ManageForum
p_mtopic = ManageTopic
p_review = ReviewPortalContent
p_request = RequestReview
p_attach = ModifyAttachments

r_anon = 'Anonymous'
r_manager = 'Manager'
r_member = 'Member'
r_owner = 'Owner'
r_fmanager = forumManager
r_reviewer = forumReviewer
r_freader = forumReader
r_fwriter = forumWriter
r_rwriter = replyWriter
r_tmanager = topicManager


def setupForumWorkflow(wf):
    wf.setProperties(title='Forum Workflow [CMFBoard]')

    for s in ('members', 'public1', 'public2', 'open', 'custom'):
        wf.states.addState(s)
    for t in ('members', 'public1', 'public2', 'open', 'custom'):
        wf.transitions.addTransition(t)
    for v in ('action', 'actor', 'comments', 'review_history', 'time'):
        wf.variables.addVariable(v)
    for p in (p_access, p_view, p_atopic, p_apoll,
              p_areply, p_review, p_announce, p_attach):
        wf.addManagedPermission(p)

    manage_addPythonScript(wf.scripts, 'stateChanged')
    scr = getattr(wf.scripts, 'stateChanged')
    scr.write(
"""##parameters=sci
sci.object.notifyStateChanged()
""")

    # states
    wf.states.setInitialState('public1')

    sdef = wf.states['members']
    sdef.setProperties(
        title='Only members can write and read',
        transitions=('public1','public2', 'open', 'custom'))
    sdef.setPermission(p_view, 0, (
        r_manager, r_fmanager, r_reviewer, r_member, forumReader, forumWriter))
    sdef.setPermission(p_access, 0, (
        r_manager, r_fmanager, r_reviewer, r_member, forumReader, forumWriter))
    sdef.setPermission(p_atopic, 0, (
        r_manager, r_fmanager, r_reviewer, r_member, forumWriter))
    sdef.setPermission(p_areply, 0, (
        r_manager, r_fmanager, r_reviewer, r_member, forumWriter, replyWriter))
    sdef.setPermission(p_apoll, 0, (
        r_manager, r_fmanager, r_reviewer, r_member, forumWriter))
    sdef.setPermission(p_announce, 0, (r_manager, r_fmanager, r_reviewer))
    sdef.setPermission(p_attach, 0, (r_manager, r_fmanager, r_reviewer, r_member, forumWriter))
    sdef.setPermission(p_review, 0, (r_manager, r_fmanager, r_reviewer))

    sdef = wf.states['public1']
    sdef.setProperties(
        title='Only members can write and all can read',
        transitions=('members','public2', 'open', 'custom'))
    sdef.setPermission(p_view, 0, (
        r_anon, r_owner, r_manager, r_fmanager, r_reviewer,
        r_member, forumReader, forumWriter))
    sdef.setPermission(p_access, 1, ())
    sdef.setPermission(p_atopic, 0, (
        r_manager, r_fmanager, r_reviewer, r_member, forumWriter))
    sdef.setPermission(p_areply, 0, (
        r_manager, r_fmanager, r_reviewer, r_member, forumWriter, replyWriter))
    sdef.setPermission(p_apoll, 0, (
        r_manager, r_fmanager, r_reviewer, r_member, forumWriter))
    sdef.setPermission(p_announce, 0, (r_manager, r_fmanager, r_reviewer))
    sdef.setPermission(p_attach, 0, (r_manager, r_fmanager, r_reviewer, r_member, forumWriter))
    sdef.setPermission(p_review, 0, (r_manager, r_fmanager, r_reviewer))

    sdef = wf.states['public2']
    sdef.setProperties(
        title='Like public1 but anonymous can reply',
        transitions=('members','public1', 'open', 'custom'))
    sdef.setPermission(p_view, 0, (
        r_anon, r_owner, r_manager, r_fmanager, r_reviewer,
        r_member, forumReader, forumWriter))
    sdef.setPermission(p_access, 1, ())
    sdef.setPermission(p_atopic, 0, (
        r_manager, r_fmanager, r_reviewer, r_member, forumWriter))
    sdef.setPermission(p_areply, 0, (
        r_anon, r_manager, r_fmanager, r_reviewer, r_member, forumWriter, replyWriter))
    sdef.setPermission(p_apoll, 0, (
        r_manager, r_fmanager, r_reviewer, r_member, forumWriter))
    sdef.setPermission(p_announce, 0, (r_manager, r_fmanager, r_reviewer))
    sdef.setPermission(p_attach, 0, (r_manager, r_fmanager, r_reviewer, r_member, forumWriter))
    sdef.setPermission(p_review, 0, (r_manager, r_fmanager, r_reviewer))

    sdef = wf.states['open']
    sdef.setProperties(
        title='Like public2 but anonymous can start topics',
        transitions=('members','public1', 'public2', 'custom'))
    sdef.setPermission(p_view, 0, (
        r_anon, r_owner, r_manager, r_fmanager, r_reviewer,
        r_member, forumReader, forumWriter))
    sdef.setPermission(p_access, 1, ())
    sdef.setPermission(p_atopic, 0, (
        r_anon, r_manager, r_fmanager, r_reviewer, r_member, forumWriter))
    sdef.setPermission(p_areply, 0, (
        r_anon, r_manager, r_fmanager, r_reviewer, r_member, forumWriter, replyWriter))
    sdef.setPermission(p_apoll, 0, (
        r_anon, r_manager, r_fmanager, r_reviewer, r_member, forumWriter))
    sdef.setPermission(p_review, 0, (r_manager, r_fmanager, r_reviewer))

    sdef = wf.states['custom']
    sdef.setProperties(
        title='Manager can define his own permissions',
        transitions=('public1','public2', 'open', 'members'))
    sdef.setPermission(p_view, 0, (
        r_manager, r_fmanager, r_reviewer, forumReader, forumWriter, replyWriter))
    sdef.setPermission(p_access, 0, (
        r_manager, r_fmanager, r_reviewer, forumReader, forumWriter, replyWriter))
    sdef.setPermission(p_atopic, 0, (
        r_manager, r_fmanager, r_reviewer, forumWriter))
    sdef.setPermission(p_areply, 0, (
        r_manager, r_fmanager, r_reviewer, forumWriter,
        forumReader, replyWriter, r_tmanager))
    sdef.setPermission(p_apoll, 0, (r_manager, r_fmanager, r_reviewer, forumWriter))
    sdef.setPermission(p_announce, 0, (r_manager, r_fmanager, r_reviewer))
    sdef.setPermission(p_attach, 0, (r_manager, r_fmanager, r_reviewer,
                                     forumWriter, r_owner))
    sdef.setPermission(p_review, 0, (r_manager, r_fmanager, r_reviewer))
    sdef.setPermission(p_mforum, 0, (r_manager, r_fmanager))

    # transitions
    tdef = wf.transitions['members']
    tdef.setProperties(
        title='Members forum',
        new_state_id='members',
        after_script_name='stateChanged',
        actbox_name='Members forum',
        actbox_url='%(content_url)s/forum_members',
        props={'guard_permissions':p_mforum})

    tdef = wf.transitions['public1']
    tdef.setProperties(
        title='Public forum',
        new_state_id='public1',
        after_script_name='stateChanged',
        actbox_name='Public1 forum',
        actbox_url='%(content_url)s/forum_public1',
        props={'guard_permissions':p_mforum})

    tdef = wf.transitions['public2']
    tdef.setProperties(
        title='Public forum (Anonymous can reply)',
        new_state_id='public2',
        after_script_name='stateChanged',
        actbox_name='Public2 forum',
        actbox_url='%(content_url)s/forum_public2',
        props={'guard_permissions':p_mforum})
    
    tdef = wf.transitions['open']
    tdef.setProperties(
        title='Open forum (Anonymous can write)',
        new_state_id='open',
        after_script_name='stateChanged',
        actbox_name='Open forum',
        actbox_url='%(content_url)s/forum_open',
        props={'guard_permissions':p_mforum})
    
    tdef = wf.transitions['custom']
    tdef.setProperties(
        title='Custom forum',
        new_state_id='custom',
        after_script_name='stateChanged',
        actbox_name='Custom forum',
        actbox_url='%(content_url)s/forum_custom',
        props={'guard_permissions':p_mforum})

    # variables
    wf.variables.setStateVar('review_state')

    vdef = wf.variables['action']
    vdef.setProperties(description='The last transition',
                       default_expr='transition/getId|nothing',
                       for_status=1, update_always=1)

    vdef = wf.variables['actor']
    vdef.setProperties(description='The ID of the user who performed '
                       'the last transition',
                       default_expr='user/getId',
                       for_status=1, update_always=1)

    vdef = wf.variables['comments']
    vdef.setProperties(description='Comments about the last transition',
                       default_expr="python:state_change.kwargs.get('comment', '')",
                       for_status=1, update_always=1)

    vdef = wf.variables['review_history']
    vdef.setProperties(description='Provides access to workflow history',
                       default_expr="state_change/getHistory",
                       props={'guard_permissions': p_mtopic})

    vdef = wf.variables['time']
    vdef.setProperties(description='Time of the last transition',
                       default_expr="state_change/getDateTime",
                       for_status=1, update_always=1)


def createForumWorkflow(id):
    ob = DCWorkflowDefinition(id)
    setupForumWorkflow(ob)
    return ob

addWorkflowFactory(createForumWorkflow, id='forum_workflow',
                   title='Forum Workflow [CMFBoard]')

forum_workflow = 'forum_workflow (Forum Workflow [CMFBoard])'
forum_workflow_id = 'forum_workflow'


def setupTopicWorkflow(wf):
    wf.setProperties(title='Topic Workflow [CMFBoard]')

    for s in ('initial', 'admin', 'announce', 'waitreview',
              'published', 'locked', 'rejected', 'private', 'retired'):
        wf.states.addState(s)
    for t in ('reject', 'lock', 'submit', 'publish', 'private',
              'autoadmin', 'autoannounce', 'autopublish', 'autosubmit',
              'announce', 'retire'):
        wf.transitions.addTransition(t)
    for v in ('action', 'actor', 'comments', 'review_history', 'time'):
        wf.variables.addVariable(v)
    for p in (p_access, p_view, p_areply, ModifyTopic, DeleteTopic, p_modify):
        wf.addManagedPermission(p)

    manage_addPythonScript(wf.scripts, 'stateChanged')
    scr = getattr(wf.scripts, 'stateChanged')
    scr.write(
"""##parameters=sci
from Products.CMFBoard.utils import forum_stateChanged
forum_stateChanged(sci)
""")

    # states
    wf.states.setInitialState('initial')

    sdef = wf.states['initial']
    sdef.setProperties(
        title='Initial state',
        transitions=('autoadmin','autoannounce', 'autopublish', 'autosubmit'))
    sdef.setPermission(p_access, 0, (r_manager, r_fmanager))
    sdef.setPermission(p_view, 0, (r_manager, r_fmanager))
    sdef.setPermission(p_areply, 0, (r_manager, r_fmanager))
    sdef.setPermission(ModifyTopic, 0, (r_manager, r_fmanager))
    sdef.setPermission(DeleteTopic, 0, (r_manager, r_fmanager))
    sdef.setPermission(p_modify, 0, ())

    sdef = wf.states['admin']
    sdef.setProperties(
        title='Topic for Administrator',
        transitions=('publish',))
    sdef.setPermission(p_access, 0, (r_manager, r_fmanager, r_reviewer, r_owner))
    sdef.setPermission(p_view, 0, (r_manager, r_fmanager, r_reviewer, r_owner))
    sdef.setPermission(p_areply, 0, (r_manager, r_fmanager, r_reviewer, r_owner))
    sdef.setPermission(ModifyTopic, 0, (r_manager, r_fmanager))
    sdef.setPermission(DeleteTopic, 0, (r_manager, r_fmanager))
    sdef.setPermission(p_modify, 0, ())

    sdef = wf.states['announce']
    sdef.setProperties(
        title='',
        transitions=('retire',))
    sdef.setPermission(p_access, 1, ())
    sdef.setPermission(p_view, 1, ())
    sdef.setPermission(p_areply, 0, ())
    sdef.setPermission(ModifyTopic, 0, (r_manager, r_fmanager, r_reviewer))
    sdef.setPermission(DeleteTopic, 0, (r_manager, r_fmanager, r_reviewer))
    sdef.setPermission(p_modify, 0, ())

    sdef = wf.states['rejected']
    sdef.setProperties(
        title='Visible and editable only by forum administrators and reviewers',
        transitions=('submit','publish', 'private'))
    sdef.setPermission(p_access, 0, (r_manager, r_fmanager, r_reviewer, r_owner))
    sdef.setPermission(p_view, 0, (r_manager, r_fmanager, r_reviewer, r_owner))
    sdef.setPermission(p_areply, 0, (r_manager, r_fmanager))
    sdef.setPermission(ModifyTopic, 0, (r_manager, r_fmanager, r_owner))
    sdef.setPermission(DeleteTopic, 0, (r_manager, r_fmanager, r_owner))
    sdef.setPermission(p_modify, 0, ())

    sdef = wf.states['private']
    sdef.setProperties(
        title='',
        transitions=('submit','publish'))
    sdef.setPermission(p_access, 0, (r_manager, r_fmanager, r_reviewer, topicOwner, r_owner))
    sdef.setPermission(p_view, 0, (r_manager, r_fmanager, r_reviewer, topicOwner, r_owner))
    sdef.setPermission(p_areply, 0, (r_manager, r_fmanager, r_reviewer))
    sdef.setPermission(ModifyTopic, 0, (r_manager, r_fmanager, topicOwner, r_owner))
    sdef.setPermission(DeleteTopic, 0, (r_manager, r_fmanager, topicOwner, r_owner))
    sdef.setPermission(p_modify, 0, ())

    sdef = wf.states['locked']
    sdef.setProperties(
        title='Visible but replies only administrators and reviewers',
        transitions=('publish','reject'))
    sdef.setPermission(p_access, 1, ())
    sdef.setPermission(p_view, 1, ())
    sdef.setPermission(p_areply, 0, (r_manager, r_fmanager, r_reviewer))
    sdef.setPermission(ModifyTopic, 0, (r_manager, r_fmanager))
    sdef.setPermission(DeleteTopic, 0, (r_manager, r_fmanager))
    sdef.setPermission(p_modify, 0, ())

    sdef = wf.states['published']
    sdef.setProperties(
        title='Visible topic',
        transitions=('lock','reject', 'private'))
    sdef.setPermission(p_access, 1, ())
    sdef.setPermission(p_view, 1, ())
    sdef.setPermission(p_areply, 1, ())
    sdef.setPermission(ModifyTopic, 1, (r_manager, r_fmanager))
    sdef.setPermission(DeleteTopic, 0, (r_manager, r_fmanager))
    sdef.setPermission(p_modify, 0, ())

    sdef = wf.states['waitreview']
    sdef.setProperties(
        title='Waiting for reviewer',
        transitions=('publish', 'reject', 'private'))
    sdef.setPermission(p_access, 0, (r_manager, r_fmanager, r_reviewer, topicOwner, r_owner))
    sdef.setPermission(p_view, 0, (r_manager, r_fmanager, r_reviewer, topicOwner, r_owner))
    sdef.setPermission(p_areply, 0, ())
    sdef.setPermission(ModifyTopic, 0, (r_manager, r_fmanager, topicOwner, r_owner))
    sdef.setPermission(DeleteTopic, 0, (r_manager, r_fmanager, topicOwner, r_owner))
    sdef.setPermission(p_modify, 0, ())

    sdef = wf.states['retired']
    sdef.setProperties(
        title='Retired Announcement',
        transitions=('announce',),
        description='retired announcement.  Still view/editable by Manager, Reviewer.  Same as "private" state.')
    sdef.setPermission(p_access, 0, (r_manager, r_fmanager, r_reviewer, topicOwner, r_owner))
    sdef.setPermission(p_view, 0, (r_manager, r_fmanager, r_reviewer, topicOwner, r_owner))
    sdef.setPermission(p_areply, 0, (r_manager, r_fmanager, r_reviewer))
    sdef.setPermission(ModifyTopic, 0, (r_manager, r_fmanager, topicOwner, r_owner))
    sdef.setPermission(DeleteTopic, 0, (r_manager, r_fmanager, topicOwner, r_owner))
    sdef.setPermission(p_modify, 0, ())


    # transitions
    tdef = wf.transitions['autoadmin']
    tdef.setProperties(
        title='Automatic make this topic for Administrators',
        new_state_id='admin',
        trigger_type=TRIGGER_AUTOMATIC,
        actbox_name='Admin',
        after_script_name='stateChanged',
        props={ 'guard_expr': 'python:here.getTopicType()==2'} )

    tdef = wf.transitions['autoannounce']
    tdef.setProperties(
        title='Automatic make this topic for Administrators',
        new_state_id='announce',
        trigger_type=TRIGGER_AUTOMATIC,
        actbox_name='Announce',
        after_script_name='stateChanged',
        props={ 'guard_expr': 'python:here.getTopicType()==4'} )

    tdef = wf.transitions['autopublish']
    tdef.setProperties(
        title='Automatic make this topic published',
        new_state_id='published',
        trigger_type=TRIGGER_AUTOMATIC,
        actbox_name='Published',
        after_script_name='stateChanged',
        props={ 'guard_expr': 'python:(here.getTopicType()==1) and not here.isModerated()'})

    tdef = wf.transitions['autosubmit']
    tdef.setProperties(
        title='Automatic submit',
        new_state_id='waitreview',
        trigger_type=TRIGGER_AUTOMATIC,
        actbox_name='autosubmit',
        after_script_name='stateChanged',
        props={ 'guard_expr': 'python:(here.getTopicType()==1) and here.isModerated()'})
    
    tdef = wf.transitions['publish']
    tdef.setProperties(
        title='Reviewer publishes content',
        new_state_id='published',
        actbox_name='Publish',
        actbox_url='%(content_url)s/content_publish_form',
        after_script_name='stateChanged',
        props={'guard_permissions': p_review}) #p_mforum})

    tdef = wf.transitions['reject']
    tdef.setProperties(
        title='Reject topic',
        new_state_id='rejected',
        actbox_name='Reject',
        actbox_url='%(content_url)s/forum_content_disable',
        after_script_name='stateChanged',
        props={'guard_permissions': p_review}) #p_mforum})

    tdef = wf.transitions['private']
    tdef.setProperties(
        title='Private topic',
        new_state_id='private',
        actbox_name='Private',
        actbox_url='%(content_url)s/forum_content_disable',
        after_script_name='stateChanged',
        props={'guard_permissions':p_request})

    tdef = wf.transitions['lock']
    tdef.setProperties(
        title='Lock topic',
        new_state_id='locked',
        actbox_name='Lock',
        actbox_url='%(content_url)s/forum_content_lock',
        after_script_name='stateChanged',
        props={'guard_permissions': p_review}) #p_mforum})

    tdef = wf.transitions['submit']
    tdef.setProperties(
        title='Submit state',
        new_state_id='waitreview',
        actbox_name='Submit',
        after_script_name='stateChanged',
        props={'guard_permissions': p_request})
    
    tdef = wf.transitions['announce']
    tdef.setProperties(
        title='',
        description='re-announce a retired announcement',
        new_state_id='announce',
        actbox_name='Announce',
        after_script_name='stateChanged',
        props={'guard_permissions': p_review}) #p_mforum})
    
    tdef = wf.transitions['retire']
    tdef.setProperties(
        title='',
        description='Retire an announcement',
        new_state_id='retired',
        actbox_name='Retire',
        after_script_name='stateChanged',
        props={'guard_permissions': p_review}) #p_mforum})


    # variables
    wf.variables.setStateVar('review_state')

    vdef = wf.variables['action']
    vdef.setProperties(description='The last transition',
                       default_expr='transition/getId|nothing',
                       for_status=1, update_always=1)

    vdef = wf.variables['actor']
    vdef.setProperties(description='The ID of the user who performed '
                       'the last transition',
                       default_expr='user/getId',
                       for_status=1, update_always=1)

    vdef = wf.variables['comments']
    vdef.setProperties(description='Comments about the last transition',
                       default_expr="python:state_change.kwargs.get('comment', '')",
                       for_status=1, update_always=1)

    vdef = wf.variables['review_history']
    vdef.setProperties(description='Provides access to workflow history',
                       default_expr="state_change/getHistory",
                       props={'guard_permissions': p_mtopic})

    vdef = wf.variables['time']
    vdef.setProperties(description='Time of the last transition',
                       default_expr="state_change/getDateTime",
                       for_status=1, update_always=1)


def createTopicWorkflow(id):
    ob = DCWorkflowDefinition(id)
    setupTopicWorkflow(ob)
    return ob

addWorkflowFactory(createTopicWorkflow, id='forum_topic_workflow',
                   title='Topic Workflow [CMFBoard]')


topic_workflow = 'forum_topic_workflow (Topic Workflow [CMFBoard])'
topic_workflow_id = 'forum_topic_workflow'



def setupMessageWorkflow(wf):
    wf.setProperties(title='Message Workflow [CMFBoard]')

    for s in ('initial', 'waitreview', 'published', 'rejected', 'visible', 'private'):
        wf.states.addState(s)
    for t in ('publish', 'reject', 'submit', 'autosubmit', 'autopublish',
              'autovisible', 'visible', 'private'):
        wf.transitions.addTransition(t)
    for v in ('action', 'actor', 'comments', 'review_history', 'time'):
        wf.variables.addVariable(v)
    for p in (p_access, p_view, DeleteReply, p_modify):
        wf.addManagedPermission(p)

    manage_addPythonScript(wf.scripts, 'stateChanged')
    scr = getattr(wf.scripts, 'stateChanged')
    scr.write(
"""##parameters=sci
from Products.CMFBoard.utils import forum_stateChanged
forum_stateChanged(sci)
""")

    wf.states.setInitialState('initial')

    ############## workflow states ################
    sdef = wf.states['initial']
    sdef.setProperties(
        title='Initial state',
        transitions=('autosubmit', 'autovisible', 'autopublish'))
    sdef.setPermission(p_access,  0, (r_manager, r_fmanager, r_owner, r_reviewer))
    sdef.setPermission(p_view,    0, (r_manager, r_fmanager, r_owner, r_reviewer))
    sdef.setPermission(p_modify,  0, (r_manager, r_fmanager))
    sdef.setPermission(DeleteReply,  0, (r_manager, r_fmanager))

    sdef = wf.states['waitreview']
    sdef.setProperties(
        title='Waiting for reviewer',
        transitions=('publish', 'reject', 'lock', 'private'))
    sdef.setPermission(p_access,  0, (r_manager, r_fmanager, r_owner, r_reviewer))
    sdef.setPermission(p_view,    0, (r_manager, r_fmanager, r_owner, r_reviewer))
    sdef.setPermission(p_modify,  0, (r_manager, r_fmanager, r_owner))
    sdef.setPermission(DeleteReply,  0, (r_manager, r_fmanager, r_owner, r_reviewer))

    sdef = wf.states['published']
    sdef.setProperties(
        title='Public',
        transitions=('reject', 'lock', 'visible', 'private'))
    sdef.setPermission(p_access,  1, ())
    sdef.setPermission(p_view,    1, ())
    sdef.setPermission(p_modify,  0, (r_manager, r_fmanager))
    sdef.setPermission(DeleteReply,  0, (r_manager, r_fmanager))

    sdef = wf.states['rejected']
    sdef.setProperties(
        title='Non-visible and editable only by owner',
        transitions=('submit', 'publish', 'private'))
    sdef.setPermission(p_access,  0, (r_manager, r_fmanager, r_reviewer, r_owner))
    sdef.setPermission(p_view,    0, (r_manager, r_fmanager, r_reviewer, r_owner))
    sdef.setPermission(p_modify,  0, (r_manager, r_fmanager, r_owner))
    sdef.setPermission(DeleteReply,  0, (r_manager, r_fmanager, r_owner))

    sdef = wf.states['private']
    sdef.setProperties(
        title='Non-visible and editable only by owner',
        transitions=('submit', 'publish',))
    sdef.setPermission(p_access,  0, (r_manager, r_fmanager, r_reviewer, r_owner))
    sdef.setPermission(p_view,    0, (r_manager, r_fmanager, r_reviewer, r_owner))
    sdef.setPermission(p_modify,  0, (r_manager, r_fmanager, r_owner))
    sdef.setPermission(DeleteReply,  0, (r_manager, r_fmanager, r_owner))

    sdef = wf.states['visible']
    sdef.setProperties(
        title='Visible for all, editable by owner',
        transitions=('lock','publish','reject','private'))
    sdef.setPermission(p_access, 1,(r_anon, r_manager, r_fmanager, r_reviewer, r_owner))
    sdef.setPermission(p_view,   1,(r_anon, r_manager, r_fmanager, r_reviewer, r_owner))
    sdef.setPermission(p_modify,  0,(r_manager, r_fmanager, r_owner))
    sdef.setPermission(DeleteReply, 0,(r_manager, r_fmanager))

    ############# transitions ##############
    tdef = wf.transitions['publish']
    tdef.setProperties(
        title='Reviewer publishes content',
        new_state_id='published',
        actbox_name='Publish',
        after_script_name='stateChanged',
        props={'guard_permissions': p_review}) #p_mforum})

    tdef = wf.transitions['autovisible']
    tdef.setProperties(
        title='Automatic publishing in unmoderated boards',
        new_state_id='visible',
        trigger_type=TRIGGER_AUTOMATIC,
        after_script_name='stateChanged',
        props={ 'guard_expr':'python:not here.isModerated() and here.isEditable()'})

    tdef = wf.transitions['autopublish']
    tdef.setProperties(
        title='Automatic publishing in unmoderated boards',
        new_state_id='published',
        trigger_type=TRIGGER_AUTOMATIC,
        after_script_name='stateChanged',
        props={ 'guard_expr':'python:not here.isModerated() and not here.isEditable()'})

    tdef = wf.transitions['autosubmit']
    tdef.setProperties(
        title='Automatic submit in moderated boards',
        new_state_id='waitreview',
        trigger_type=TRIGGER_AUTOMATIC,
        after_script_name='stateChanged',
        props={ 'guard_expr':'here/isModerated'})

    tdef = wf.transitions['reject']
    tdef.setProperties(
        title='Reviewer rejects submission',
        new_state_id='rejected',
        actbox_name='Reject',
        after_script_name='stateChanged',
        props={'guard_permissions': p_review}) #p_mforum})

    tdef = wf.transitions['private']
    tdef.setProperties(
        title='Private',
        new_state_id='private',
        actbox_name='Private',
        after_script_name='stateChanged',
        props={'guard_permissions': p_request})

    tdef = wf.transitions['submit']
    tdef.setProperties(
        title='Member requests publishing',
        new_state_id='waitreview',
        actbox_name='Submit',
        after_script_name='stateChanged',
        props={'guard_permissions': p_request})

    tdef = wf.transitions['visible']
    tdef.setProperties(
        title='',
        new_state_id='visible',
        actbox_name='Visible',
        after_script_name='stateChanged',
        props={'guard_permissions': p_review}) #p_mforum})

    wf.variables.setStateVar('review_state')

    ############## variables #############
    vdef = wf.variables['action']
    vdef.setProperties(description='The last transition',
                       default_expr='transition/getId|nothing',
                       for_status=1, update_always=1)

    vdef = wf.variables['actor']
    vdef.setProperties(description='The ID of the user who performed '
                       'the last transition',
                       default_expr='user/getId',
                       for_status=1, update_always=1)

    vdef = wf.variables['comments']
    vdef.setProperties(description='Comments about the last transition',
                       default_expr="python:state_change.kwargs.get('comment', '')",
                       for_status=1, update_always=1)

    vdef = wf.variables['review_history']
    vdef.setProperties(description='Provides access to workflow history',
                       default_expr="state_change/getHistory",
                       props={'guard_permissions':
                              p_request + ';' + p_review})

    vdef = wf.variables['time']
    vdef.setProperties(description='Time of the last transition',
                       default_expr="state_change/getDateTime",
                       for_status=1, update_always=1)


def createMessageWorkflow(id):
    ob=DCWorkflowDefinition(id)
    setupMessageWorkflow(ob)
    ob.setProperties(title='Message Workflow [CMFBoard]')
    return ob

message_workflow = 'forum_message_workflow (Message Workflow [CMFBoard])'
message_workflow_id = 'forum_message_workflow'

addWorkflowFactory(createMessageWorkflow, message_workflow_id,
                   title='Message Workflow [CMFBoard]')
