# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Author: Nikolay Kim <fafhrd@legco.biz> 
"""
$Id: HitsCounter.py,v 1.8.2.1 2004/12/27 18:15:45 fafhrd91 Exp $
"""

import os, thread, traceback, Globals
from webdav.common import rfc1123_date
from DateTime import DateTime
from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import View

try:
    from App.config import getConfiguration
    cfg = getConfiguration()
    INSTANCE_HOME = cfg.instancehome
except:
    from Globals import INSTANCE_HOME

from config import hc_dir, hc_use_instance

join = os.path.join
var_dir = os.path.join(INSTANCE_HOME, 'var')

class HitsCounter:

    def __init__(self):
        """ """
        if hc_use_instance:
            path = join(var_dir, hc_dir)
        else:
            path = hc_dir

        if not os.path.exists(path):
            os.mkdir(path)

        self._path = path

        sync = thread.allocate_lock()

        self._a = sync.acquire
        self._r = sync.release

    def get(self, uid, parent='forums'):
        try:
            path = join(self._path, str(parent))
            if not os.path.exists(path):
                os.mkdir(path)

            path = join(path, str(uid))
           
            f = open(path, 'rb')
            data = f.read()
            f.close()
            return int(data)
        except:
            return 0

    def set(self, uid, count, parent='forums'):
        self._a()

        try:
            path = join(self._path, str(parent), str(uid))
            try:
                f = open(path, 'wb')
            except:
                self.get(uid, parent)
                f = open(path, 'wb')

            f.write(str(count))
            f.close()
        finally:
            self._r()

    def del_item(self, uid, parent='forums'):
        self._a()
        
        try:
            os.unlink(join(self._path, str(parent), str(uid)))
        finally:
            self._r()

    def inc(self, uid, parent='forums'):
        count = self.get(uid, parent) + 1
        self.set(uid, count, parent)

        return count

    raw = get


hits_counter = HitsCounter()


img = 'GIF89a\x01\x00\x01\x00\x80\x00\x00\x00\x00\x00\x00\x00\x00!\xf9\x04\x01\x00\x00\x00\x00,\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;'
len_img = len(img)

def hits_image(response):
    response.setHeader('Content-Type', 'image/gif')
    response.setHeader('Last-Modified', rfc1123_date(DateTime('01/01/2000')))
    response.setHeader('Content-Length', len_img)
    return img


class HitsCounterForum:

    __implements__ = ()
    
    security = ClassSecurityInfo()

    def _getParent(self):
        return 'forums'

    security.declareProtected(View, 'Hits')
    def Hits(self):
        """ return hits and inc """
        parent = self._getParent()
        return hits_counter.inc(self.UID(), parent)

    security.declareProtected(View, 'RawHits')
    def RawHits(self):
        """ returns hits counter without changing it """
        parent = self._getParent()
        return hits_counter.get(self.UID(), parent)

    security.declareProtected(View, 'HitsImg')
    def HitsImg(self):
        """ returns 1x1 transparent image """
        self.Hits()
        return hits_image(self.REQUEST.RESPONSE)

    security.declarePrivate('setHits')
    def setHits(self, num):
        """ set initial number """
        parent = self._getParent()
        return hits_counter.set(self.UID(), num, parent)

    def manage_beforeDelete(self, item, container):
	""" remove hits record"""
        hits_counter.del_item(self.UID(), self._getParent())

Globals.InitializeClass(HitsCounterForum)


class HitsCounterTopic(HitsCounterForum):

    def _getParent(self):
        return str(self.aq_parent.UID())

