# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nikolay Kim <fafhrd@legco.biz>
"""
$Id: ListSupport.py,v 1.13.2.11 2005/05/26 03:52:06 fafhrd91 Exp $
"""

import sys, traceback
from zLOG import LOG, ERROR
from types import FloatType, StringType
from DateTime import DateTime
from ExtensionClass import Base
from Acquisition import aq_base
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from AccessControl.PermissionRole import rolesForPermissionOn
from BTrees.IOBTree import IOBTree
from BTrees.IIBTree import IIBTree, IISet, IITreeSet, \
     union, intersection, weightedIntersection
from Products.CMFCore.utils import _mergedLocalRoles, _getAuthenticatedUser
from Products.CMFCore.CMFCorePermissions import View

from config import PROJECTNAME
from ForumPermissions import ManageForum

def allowedRolesAndUsers(ob):
    allowed = {}
    for r in rolesForPermissionOn('View', ob):
        allowed[r] = 1

    localroles = _mergedLocalRoles(ob)
    for user, roles in localroles.items():
        for role in roles:
            if allowed.has_key(role):
                allowed['user:' + user] = 1
    if allowed.has_key('Owner'):
        del allowed['Owner']
        owner = ob.getOwnerTuple()
        if owner is not None:
            allowed['user:' + owner[1]] = 1

    return tuple(allowed.keys())


class ListSupport(Base):

    security = ClassSecurityInfo()

    security.declareProtected(ManageForum, 'printIndex')
    def printIndex(self):
        """ """
        for key, val in self._list_index.items():
            print key, val

        print '==================================='
        for key, val in self._list_unindex.items():
            print key, val        

        print '==================================='
        for key, val in self._hash_map.items():
            print key, val

    def __init__(self):
        self._clear()

    def _clear(self):
        self._hash_map = IOBTree()

        # mtime index
        self._list_index = IIBTree()
        self._list_unindex = IIBTree()

        # allowRolesAndUsers index
        self._list_access_index = IOBTree()
        self._list_access_unindex = IOBTree()

    def _get_time_key(self, obj, id, meth='created', attr='creation_date'):
        # this code needed because int(DateTime)
        # can take equel integer for diffirent dates
        date = getattr(obj, meth)()
        mtime = int(date)
        stored_id = self._list_index.get(mtime, None)
        if stored_id is None:
            return mtime
        elif stored_id == id:
            return mtime

        # conflict resolution 
        setattr(obj, attr, date + 0.00002)
        return self._get_time_key(obj, id, meth, attr)

    def _get_user_roles(self):
        user = _getAuthenticatedUser(self)
            
        roles = list( user.getRoles() )
        if hasattr(aq_base(user), 'getGroups'):
            roles = roles + ['user:%s' % x for x in user.getGroups()]
        id = user.getId()
        if id:
            roles.append('user:%s' % id)

        #print '_get_user_roles', roles
        #print '_get_user_roles', [hash(r) for r in roles]
        
        return [hash(r) for r in roles]
        
    def _insert(self, id, mtime, object):
        self._remove(id, mtime, object)

        self._list_index[mtime] = id
        self._list_unindex[id] = mtime

        index = self._list_access_index
        roles = [hash(r) for r in allowedRolesAndUsers(object)]
        for r in roles:
            row = index.get(r, None)
            if row is None:
                index[r] = IITreeSet((id,))
            else:
                row.insert(id)

        self._list_access_unindex[id] = roles

    def _remove(self, id, mtime, object):
        t = self._list_unindex.get(id, None)
        if t is not None:
            try:
                del self._list_index[t]
            except:
                pass
            try:
                del self._list_unindex[id]
            except:
                pass

        index = self._list_access_index
        unindex = self._list_access_unindex
        
        for r in unindex.get(id, ()):
            row = index.get(r, None)
            if row is not None:
                try:
                    row.remove(id)
                except KeyError:
                    pass
        try:
            del unindex[id]
        except KeyError:
            pass

    security.declarePrivate('insertObject')
    def insertObject(self, object):
        """ insert object into list """
        try:
            self.removeObject(object)

            obId = object.getId()
            hashId = hash(obId)
            mtime = self._get_time_key(object, hashId)

            self._hash_map[hashId] = obId
            self._insert(hashId, mtime, object)
        except Exception, msg:
            LOG(PROJECTNAME, ERROR, str(msg), error = sys.exc_info())

    security.declarePrivate('removeObject')
    def removeObject(self, object):
        """ remove object from list """
        try:
            id = object.getId()
            hashId = hash(id)
            mtime = self._get_time_key(object, hashId)

            try:
                del self._hash_map[hashId]
            except KeyError:
                pass
            except AttributeError:
                self._clear()

            self._remove(hashId, mtime, object)
        except Exception, msg:
            LOG(PROJECTNAME, ERROR, str(msg), error = sys.exc_info())

    security.declareProtected(View, 'getFirstObject')
    def getFirstObject(self):
        """ returns first message """
        try:
            index = self._list_index
            return self._hash_map[index[index.minKey()]]
        except:
            return None

    security.declareProtected(View, 'getLastObject')
    def getLastObject(self):
        """ returns first message """
        try:
            r = None
            if self.getForum().isModerated():
                index = self._list_access_index
                for role in self._get_user_roles():
                    set = index.get(role, None)
                    if set is not None:
                        r = union(r, set)

            list_index = self._list_index

            #print list(list_index.items())
            
            if r is not None:
                index = IISet(weightedIntersection(
                    self._list_unindex, r, 1, 0)[1].values()).keys()[-1]
            else:
                index = self._list_index.maxKey()

            #print index
            
            return self._hash_map[list_index[index]]
        except:
            return None
    
    security.declareProtected(View, 'getObjectsList')
    def getObjectsList(self, order='direct'):
        """ return list topics sortet by creation time
        with direct/reverse order """

        # if forum is moderated then
        # first select only viewable messages
        # otherwise select all (speed optimization)
        r = None
        if self.getForum().isModerated():
            index = self._list_access_index
            for role in self._get_user_roles():
                set = index.get(role, None)
                if set is not None:
                    r = union(r, set)

        if r is not None:
            results = IISet(weightedIntersection(
                self._list_unindex, r, 1, 0)[1].values()).keys()
        else:
            results = self._list_index.keys()

        if order == 'reverse':
            # fixme: something strange with LazyListReverse
            #return LazyListReverse(self, results)
            l = list(LazyList(self, results))
            l.reverse()
            return l
        else:
            return LazyList(self, results)

    security.declareProtected(ManageForum, 'rebuildIndex')
    def rebuildIndex(self, rebuild0000=1):
        """ drop index and rebuild new """
        self._clear()

        for obj in self.contentValues():
            self.insertObject(obj)

        if rebuild0000:
            self.insertObject(getattr(self, '0000'))

InitializeClass(ListSupport)


class LazyList:

    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, list, seq):
        self._list = list
        self._seq = seq
        self._len = len(seq)

    def __len__(self):
        return self._len

    def __str__(self):
        return 'LazyList "%s"'%list(self)

    def __getitem__(self, key):
        list = self._list
        hashId = list._list_index[self._seq[key]]
        return list._hash_map[hashId]


class LazyListReverse(LazyList):

    def __getitem__(self, key):
        list = self._list
        hashId = list._list_index[self._seq[self._len - key - 1]]

        return list._hash_map[hashId]
