# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
"""
$Id: Subscribeable.py,v 1.44.6.2 2005/05/23 17:35:00 fafhrd91 Exp $
"""

import re, traceback
from Acquisition import aq_parent, aq_inner
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from AccessControl.SecurityManagement import newSecurityManager, noSecurityManager

from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.CMFCore.CMFCorePermissions import View
from Products.Archetypes.config import TOOL_NAME

from Products.PortalTransport.SubscribeableMember import SubscribeableMember
from Products.PortalTransport.SubscribeableAnonymous import SubscribeableAnonymous
from Products.PortalTransport.PortalTransportPermissions import *
from Products.PortalTransport.utils import *

from config import PROJECTNAME
from debug import log_exc
from ForumPermissions import *

siType = 'PT_SIMember'

class Subscribeable(SubscribeableMember):
    """ """
    security = ClassSecurityInfo()

    def manage_afterAdd(self, item, container):
        self.manage_permission(
            AllowSubscription, ('Manager', 'Member', 'Owner',
                                forumReader, forumWriter, replyWriter), 0)

    security.declareProtected(ManageForum, 'getSubscribersList')
    def getSubscribersList(self):
        """ """
        stool = getToolByName(self, 'portal_subscriptions')
        return stool.getSubscriptions(hint=self.getForum().UID())

    security.declareProtected(ManageForum, 'removeSubscribers')
    def removeSubscribers(self, ids, noemail=0):
        """ """
        subs = getToolByName(self, 'portal_subscriptions')
        subs.removeSubscriptions(ids, noemail)

    security.declareProtected(ManageForum, 'addSubscribers')
    def addSubscribers(self, members):
	"""
        Subscribe a given sequence of MemberData objects to the current item.
        """
        hint = self.getForum().UID()
        
        for member in members:
            mtool = getToolByName(self, 'portal_membership')
            # Check if user is a portal member
            if mtool.getMemberById(member.getId()) is None:
                raise Exception, "Can only subscribe portal member."

            subs = getToolByName(self, 'portal_subscriptions')
        
            if not subs.isSubscribed(self, member_id=member.getMemberId()):
                subs.addSubscriptions(siType, ({'member': member, 'content': self,
                                                'hint': hint, 'text_format': 'plain'},) )
            else:
                raise Exception, '%s already subscribed.' % member.getId()

    security.declareProtected(AllowSubscription, 'subscribe')
    def subscribe(self):
        """ """
        SubscribeableMember.subscribe(self, hint=self.getForum().UID())

    security.declarePrivate('sendMessage')
    def sendMessage(self, message, parent=None):
        """ send reply to all subsribed members """
        portal = getToolByName(self, 'portal_url').getPortalObject()
        pmt = portal.portal_mailtemplates
        mtool = portal.portal_membership
        mspool = portal.portal_mailspool
        stool = portal.portal_subscriptions
        email_from_address = portal.email_from_address
        
        topic = message.getTopic()
        forum = message.getForum()

        subs = (stool.getSubscriptions(topic, review_state=('active', 'deactivation')) +
                stool.getSubscriptions(forum, review_state=('active', 'deactivation')))

        if not subs:
            return

        # generate users list and check security
        plain = []
        html = []
        acl_users = portal.acl_users
        for si in subs:
            member_id = si.member_id
            member = mtool.getMemberById(member_id)
            
            if member is not None:
                # check if user can view this message
                user = member.getUser()
                user = user.__of__(acl_users)
                newSecurityManager(None, user)
                if _checkPermission(View, topic):
                    if member.getProperty('fc_mail_format', 'plain') == 'html':
                        html.append(member)
                    else:
                        plain.append(member)
                noSecurityManager()

        if not (plain or html):
            return

        # generate headers
        headers = {}

        # from header
        member = mtool.getMemberById(message.Creator())
        if member:
            if not member.getProperty('show_fullname', 0):
                fullname = str(member)
            else:
                fullname = member.getProperty('fullname', str(member)).strip()
                if not fullname:
                    fullname = str(member)

            email = member.getProperty('email', email_from_address)
            if not email:
                email = email_from_address

            headers['From'] = '%s <%s>' % (fullname, email)
        else:
            headers['From'] = 'Anonymous <%s>' % email_from_address

        # message-id header
        headers['Message-ID'] = '<%s@cmfboard.org>'%message.UID()
        if parent:
            headers['In-Reply-To'] = '<%s@cmfboard.org>'%parent.UID()


        forum_title = forum.Title()
        forum_url = forum.absolute_url()

	# set default return path
        return_path = email_from_address
	
        # lists headers
        mtransport = getattr(portal, 'portal_mailtransport', None)
        if mtransport is not None:

            # get additional headers from portal_mailtransport
            mtinfo = self.getBRefs('MailTransport')
            if mtinfo and mtinfo[0]:
                mt_headers = mtinfo[0].getHeaders()

                return_path = mtinfo[0].getReturnPath()

                forum_email = mt_headers['reply-to']
                if forum_email and (forum_email != email_from_address):
                    headers['To'] = '%s'%formataddr((forum_title, forum_email))
                    headers['List-Id'] = '%s, %s'%(forum_title, forum.getId())
                    headers['List-Post'] = '<mailto:%s>'%forum_email
                    headers['List-Unsubscribe'] = '<%s>'%forum_url
                    headers['List-Subscribe'] = '<%s>'%forum_url
                    headers['List-Archive'] = '<%s>'%forum_url

                headers.update(mt_headers)

        info = {
            'message': message,
            'message_title': message.Title() or '[No subject]',
            'message_url': message.absolute_url(),
            'topic': topic,
            'topic_url': topic.absolute_url(),
            'topic_title': topic.Title(),
            'forum': forum,
            'forum_url': forum_url,
            'forum_title': forum_title,
            }
        headers['Date'] = message.created().rfc822()

        # attachements
        files = []

        for attach in message.objectValues('ForumAttachment'):
            size = attach.getSize()
            if size:
                file = attach.getField('file')
                files.append([str(file.get(attach)), file.getContentType(attach),
                              attach.getId()])

        # send html messages
        if html:
            bcc = []
            for m in html:
                try:
                    bcc.append(m.getProperty('email'))
                except:
                    pass

            info['message_body'] = message.CookedBody()

            mspool.sendmail(return_path, bcc, pmt.createMessage(
                PROJECTNAME, 'forum_message_html', info, headers, text_format='html',
                files=files))

        # send plain messages
        if plain:
            bcc = []
            for m in plain:
                try:
                    bcc.append(m.getProperty('email'))
                except:
                    pass

            info['message_body'] = message.getText(mimetype='text/plain')

            message = pmt.createMessage(
                PROJECTNAME, 'forum_message_plain', info, headers, files=files)
            mspool.sendmail(return_path, bcc, message)


    security.declarePrivate('sendModeratedNotification')
    def sendModeratedNotification(self, message, parent):
        """ send notification to author """
        portal = getToolByName(self, 'portal_url').getPortalObject()
        pmt = portal.portal_mailtemplates
        mtool = portal.portal_membership
        mspool = portal.portal_mailspool
        stool = portal.portal_subscriptions
        email_from_address = portal.email_from_address

        forum = message.getForum()
        topic = message.getTopic()

        member = mtool.getAuthenticatedMember()
        member_id = member.getUserName()

        if not self.isSubscribed():
            return

        # generate headers
        headers = {}

        # from header
        if member:
            if not member.getProperty('show_fullname', 0):
                fullname = str(member)
            else:
                fullname = member.getProperty('fullname', str(member))
                if not fullname:
                    fullname = str(member)

            email = member.getProperty('email', email_from_address)
            if not email:
                email = email_from_address

            headers['From'] = '%s <%s>' % (fullname, email)
        else:
            headers['From'] = 'Anonymous <%s>' % email_from_address

        # message-id header
        headers['Message-ID'] = '<%s@cmfboard.org>'%message.UID()
        if parent:
            headers['In-Reply-To'] = '<%s@cmfboard.org>'%parent.UID()

        forum_title = forum.Title()
        forum_url = forum.absolute_url()

	# set default return path
        return_path = email_from_address

        # lists headers
        mtransport = getattr(portal, 'portal_mailtransport', None)
        if mtransport is not None:

            # get additional headers from portal_mailtransport
            mtinfo = self.getBRefs('MailTransport')
            if mtinfo and mtinfo[0]:
                mt_headers = mtinfo[0].getHeaders()

                return_path = mtinfo[0].getReturnPath()

                if forum_email and (forum_email != email_from_address):
                    headers['List-Id'] = '%s\n\t%s'%(forum_title, forum.getId())
                    headers['List-Post'] = '<mailto:%s>'%forum_email
                    headers['List-Unsubscribe'] = '<%s>'%forum_url
                    headers['List-Subscribe'] = '<%s>'%forum_url
                    headers['List-Archive'] = '<%s>'%forum_url

                headers.update(mt_headers)

        email = member.getProperty('email')
        headers['To'] = '%s <%s>'%(member.getProperty('fullname'), email)
        headers['Date'] = message.created().rfc822()

        info = {
            'message': message,
            'message_title': message.Title() or '[No subject]',
            'message_body': message.EditableBody(),
            'message_url': message.absolute_url(),
            'topic': topic,
            'topic_url': topic.absolute_url(),
            'topic_title': topic.Title(),
            'forum': forum,
            'forum_url': forum.absolute_url(),
            'forum_title': forum.Title(),
            }

        msg = pmt.createMessage(PROJECTNAME, 'forum_moderated_message', info, headers)
        mspool.sendmail(return_path, email, msg)


InitializeClass(Subscribeable)
