# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nikolay Kim <fafhrd@legco.biz>
"""
$Id: TreeSupport.py,v 1.8.2.11 2005/03/25 12:00:45 fafhrd91 Exp $
"""

import traceback
from Globals import InitializeClass
from BTrees.IOBTree import IOBTree
from BTrees.IIBTree import IIBTree, IITreeSet
from AccessControl import ClassSecurityInfo

from ListSupport import ListSupport

class TreeSupport(ListSupport):

    security = ClassSecurityInfo()

    security.declarePublic('printTree')
    def printTree(self):
        """ """
        print 'printTree'
        for k, v in self._tree_index.items():
            print k, list(v.keys())

        print '-----------------------------------'

        for k, v in self._tree_unindex.items():
            print k, v
        print '-----------------------------------'

        for key, val in self._hash_map.items():
            print key, val
        print '==================================='


    def _clear(self):
        self._tree_index = IOBTree()
        self._tree_unindex = IIBTree()
        ListSupport._clear(self)

    security.declarePrivate('getRootId')
    def getRootId(self):
        return self.id

    def getParent(self, msgId):
        """ return parent of message """
        try:
            return self._hash_map[self._tree_unindex[hash(msgId)]]
        except:
            return self.getRootId()

    def getChildren(self, msgId):
        """ returns all children """
        try:
            hash_map = self._hash_map
            return [hash_map.get(val) for val in self._tree_index[hash(msgId)].keys()
                    if val != 1970255172]
        except:
            return ()

    def isInTree(self, msgId):
        """ checks if message id in tree """
        return self._tree_unindex.has_key(hash(msgId))

    def insertMessageIntoTree(self, msg, parent=None):
        """ insert message into tree """
        m_id = msg.getId()
        root_id = self.getRootId()
        
        if m_id == root_id:
            return
        
        msgId = hash(m_id)

        parentId = None
        if ((parent is None) or
            (parent.portal_type not in ('ForumMessage','ForumContentMessage'))):
            index = self._list_index
            try:
                parentId = index[index.keys()[-2]]
                if msgId == parentId:
                    parentId = None
            except IndexError:
                pass
        else:
            parentId = hash(parent.getId())

        if parentId is None:
            parentId = hash(self.getRootId())

        if msgId != parentId:
            # insert parent index
            self._tree_unindex[msgId] = parentId

            # insert children index
            index = self._tree_index
            data = index.get(parentId, None)
            if data is None:
                data = IITreeSet((msgId,))
                index[parentId] = data
            else:
                data.insert(msgId)

    def removeMessageFromTree(self, msg):
        """ removes message from tree """
        msgId = hash(msg.getId())

        try:
            parent = self._tree_unindex[msgId]
        except KeyError:
            parent = hash(self.getRootId())

        index = self._tree_index
        unindex = self._tree_unindex

        parentId = unindex.get(msgId, None)
        if parentId is None:
            return parent
        
        # reparent childrens
        for chId in index.get(msgId, {}).keys():
            unindex[chId] = parentId
            index[parentId].insert(chId)

        # remove index records
        del unindex[msgId]
        index[parentId].remove(msgId)

        return parent

    def getObjectsTree(self, base=None):
        """ return messages tree """
        root = hash(self.getRootId())
        return LazyTree(self, root, root)

InitializeClass(TreeSupport)


class LazyTree:

    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, support, base, root):
        self._support = support
        self._parent = None
        self._base = base
        self._root = root
        index = support._tree_index

        hash_map = support._hash_map

        if base == root:
            self.id = support.getRootId()
        else:
            self.id = hash_map[base]

        children_ids = []
        for key in index.get(base, {}).keys():
            try:
                id = hash_map.get(key)
                children_ids.append(id)
            except KeyError:
                pass
            
        self._len = len(children_ids)
        self._children_ids = children_ids
        
    def __len__(self):
        return self._len

    def __nonzero__(self):
        return self._len > 0

    def __getitem__(self, key):
        #return self._children_ids[key]
        hashId = hash(self._children_ids[key])
        return LazyTree(self._support, hashId, hash(self.id))

    def isRoot(self):
        return self._base == self._root
    
    def getParent(self):
        pass

    def getChildrenIds(self):
        """ returns children ids """
        return self._children_ids
