# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.kz> 
"""
$Id: UserPreferences.py,v 1.16.4.1.2.6 2005/05/20 11:54:15 fafhrd91 Exp $
"""

import random, traceback, time
from cPickle import loads, dumps
from types import InstanceType

from ExtensionClass import Base
from Globals import InitializeClass
from Acquisition import aq_base, aq_acquire
from AccessControl import ClassSecurityInfo
from DateTime import DateTime

from OFS.SimpleItem import SimpleItem
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.utils import getToolByName, _checkPermission

from ForumCatalog import *
from ForumPermissions import *
from config import CACHE_ID2
from debug import log_exc
from utils import cb_isinstance

class UserPreferences(SimpleItem):

    security = ClassSecurityInfo()

    security.declareProtected(View, 'get_user_role')
    def get_user_role(self, user, default=''):
        """ """
        user = getToolByName(self, 'portal_membership').getMemberById(user)
        try:
            if user:
                roles = user.getRoles()
                if ('Forum manager' in roles) or ('Manager' in roles):
                    return 'Manager'
                elif 'Forum reviewer' in roles:
                    return 'Reviewer'
                else:
                    return 'Member'
        except:
            pass

        return 'Anonymous'


    security.declareProtected(View, 'get_user_properties')
    def get_user_properties(self, user, portal=None, mtool=None):
        """ """
        user = str(user)

        if portal is None:
            portal = getToolByName(self, 'portal_url').getPortalObject()

        cache = None
        try:
            manager = aq_acquire(portal, CACHE_ID2, containment=1, default=None)
        except:
            manager = None
        if manager is not None:
            cache = manager.ZCacheManager_getCache()
            if cache is not None:
                data = cache.ZCache_get(portal, 'user_props', keywords={'user': user})
                if data is not None:
                    return data
            
        if mtool is None:
            mtool = portal.portal_membership
        
        data = {'fc_posts': 'n/a',
                'fc_emailtome': 0,
                'fc_forum_vm': 'flat',
                'fc_topic_vm': 'flat',
                'homepage': '',
                'fullname': user,
                'avatar': '',
                'anonymous': 1,
                }

        # maybe fix for LdapUserFolder
        if (user == 'None') or not user:
            return data
        
        user_ob = mtool.getMemberById(user)
        if user_ob:
            data['anonymous'] = 0
	    try: #catch Unauthorise exception from CMFMember
        	if user_ob.getProperty('show_email', 0):
            	    email = user_ob.getProperty('email', '')
            	    new_email = ''.join(['&#%s;'%ord(ch) for ch in email])
            	    data['email'] = new_email
            	    data['email_tag'] = '<a href="mailto:%s">%s</a>'%(new_email, new_email)
            	    data['email_tag_start'] = '<a href=" mailto:%s">'%new_email
            	    data['email_tag_end'] = '</a>'

        	if user_ob.getProperty('show_im', 0):
            	    data['im_jabber'] = user_ob.getProperty('im_jabber', '')
            	    data['im_yahoo'] = user_ob.getProperty('im_yahoo', '')
                    data['im_icq'] = user_ob.getProperty('im_icq', '')
                    data['im_msn'] = user_ob.getProperty('im_msn', '')
                    data['im_aim'] = user_ob.getProperty('im_aim', '')
                    data['im_ichat'] = user_ob.getProperty('im_ichat', '')

                if (user_ob.getProperty('fc_emailtome', 0) and
                    user_ob.getProperty('email', '')):
                    data['fc_emailtome'] = 1

                data['fc_posts'] = user_ob.getProperty('fc_posts', 'n/a')
                data['fc_forum_vm'] = user_ob.getProperty('fc_forum_vm', 'flat')
                data['fc_topic_vm'] = user_ob.getProperty('fc_topic_vm', 'flat')

                data['first_login_time'] = user_ob.getProperty('first_login_time', '')
                data['last_login_time'] = user_ob.getProperty('last_login_time', '')
                data['location'] = user_ob.getProperty('location', '')
                data['occupation'] = user_ob.getProperty('occupation', '')
                data['interests'] = user_ob.getProperty('interests', '')
                data['homepage'] = user_ob.getProperty('homepage', '')
                data['timezone'] = user_ob.getProperty('timezone', '')

                if user_ob.getProperty('show_fullname', 0):
                    data['fullname'] = user_ob.getProperty('fullname','') or user

                avatar = mtool.getPersonalPortrait(user_ob.getId())
                if avatar.getId() != 'defaultUser.gif':
                    data['avatar'] = avatar.tag(width=75, height=100)
            except:
                pass

            # remove instances
            filtered = {}
            for k, v in data.items():
                if cb_isinstance(v, Base):
                    filtered[k] = ''
                else:
                    filtered[k] = v
                    
            data = filtered
            
            if cache is not None:
                try:
                    cache.ZCache_set(portal, data, 'user_props', keywords={'user': user})
                except:
                    pass
                    #log_exc('Unkown error')
                    
        return data


    security.declareProtected(View, 'get_read_status')
    def get_read_status(self, object, request):
        """ check object read status """
        last_login = self._get_last_login_time(request)

        topic = object.getTopic()
        object_modified = topic.modified().timeTime()

        if last_login > object_modified:
            return 1

        try:
            session = request.SESSION
        except:
            # sometimes error happen, i don't know why
            return 1

        if session.has_key('read_status'):
            read_status = session['read_status']
        else:
            read_status = {}

        if read_status.get(topic.UID(), 0) > object_modified:
            return 1

        return 0

    security.declareProtected(View, 'set_read_status')
    def set_read_status(self, object, request):
        """ mark topic as read, session is used for storing status """
        try:
            session = request.SESSION
        except:
            # sometimes error happen, i don't know why
            return

        last_login = self._get_last_login_time(request)

        if session.has_key('read_status'):
            read_status = session['read_status']
        else:
            read_status = {}

        topic = object.getTopic()
        object_uid = topic.UID()
        object_modified = topic.modified().timeTime()

        # check is it really need save access time
        if last_login > object_modified:
            if read_status.has_key(object_uid):
                del read_status[object_uid]
                session['read_status'] = read_status
        else:
            read_status[object_uid] = time.time()
            session['read_status'] = read_status

    def _get_last_login_time(self, request):
        """ determin user last login time, only works with cookie enabled browsers
        i use 2 cookie, one permanent, second only for one session.
        so if first cooki exist but second is not then this is new session
        and we need change last_login_time
        """
        var1 = 'last_login_1'
        var2 = 'last_login_2'

        cookies = request.cookies

        now = DateTime()
        expires = (now + 365).toZone('GMT').rfc822()
        setCookie = request.RESPONSE.setCookie

        if cookies.has_key(var1):
            try:
                last_login = DateTime(cookies[var1])
            except:
                last_login = now
                
            if not cookies.has_key(var2):
                # this code for update member property, but does it need?
                #mtool = self.portal_membership
                #if not mtool.isAnonymousUser():
                #    member = mtool.getAuthenticatedMember()
                #    member.setProperties('last_login_time', login_time)
                setCookie(var1, now, path='/', expires = expires)
                setCookie(var2, 1, path='/')
                
            return last_login
        else:
            mtool = self.portal_membership
            if not mtool.isAnonymousUser():
                member = mtool.getAuthenticatedMember()
                login_time = member.getProperty('last_login_time', now)
            else:
                login_time = now

            setCookie(var1, login_time, path='/', expires = expires)
            setCookie(var2, 1, path='/')
            return login_time

InitializeClass(UserPreferences)
