# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nikolay Kim <fafhrd@legco.biz>
"""
$Id: UtilitiesTool.py,v 1.16.2.8 2005/04/07 18:24:08 fafhrd91 Exp $
"""
import os, traceback
from Globals import InitializeClass
from Acquisition import aq_inner, aq_parent
from AccessControl import ClassSecurityInfo
from ZPublisher import BeforeTraverse
from ZPublisher.HTTPRequest import HTTPRequest
from Products.CMFCore.utils import UniqueObject, SimpleItemWithProperties
from Products.CMFCore.CMFCorePermissions import View, ManagePortal
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.CMFPlone.PloneUtilities import translate_wrapper

from UserPreferences import UserPreferences
from Extensions.migrations import migrate

import smiles, config
from config import CMFBOARD_TOOL

_zpt = os.path.join(os.path.dirname(__file__), 'zpt')

_cache_data = {}

class UtilitiesTool(UniqueObject, SimpleItemWithProperties, UserPreferences):
    """ utilities tool for access various iternal methods """

    id = CMFBOARD_TOOL
    meta_type = 'CMFBoard Utilities Tool'

    security = ClassSecurityInfo()

    _properties = (
        {'id': 'poll_slot_configuration', 'type': 'string', 'mode': 'rw'},
        {'id': 'allow_signatures', 'type': 'boolean', 'mode': 'rw'},
        {'id': 'allow_outside_forumnb', 'type': 'boolean', 'mode': 'rw'},
        {'id': 'view_mode_selectable', 'type': 'boolean', 'mode': 'rw'},
        {'id': 'latest_portlet_count', 'type': 'int', 'mode': 'rw'},
        {'id': 'latest_portlet_all_forums', 'type': 'boolean', 'mode': 'rw'},
        {'id': 'latest_portlet_forums', 'type': 'lines', 'mode': 'rw'},
        {'id': 'filtered_discussion_types', 'type': 'lines', 'mode': 'rw'},
        )

    allow_signatures = 1
    allow_outside_forumnb = 0
    view_mode_selectable = 1
    poll_slot_configuration = 'newest'
    latest_portlet_count = 6
    latest_portlet_forums = ()
    latest_portlet_all_forums = 1
    filtered_discussion_types = config.filtered_discussion_types

    manage_options = (
        (
        { 'label'  : 'Properties',
          'action' : 'manage_propertiesForm',
          },
        {  'label'  : 'Migration',
           'action' : 'manage_migrationForm',
           },
        ))
    
    security.declareProtected(ManagePortal, 'manage_migrationForm')
    manage_migrationForm = PageTemplateFile('migrationForm',_zpt)

    def manage_afterAdd(self, item, container):
        SimpleItemWithProperties.manage_afterAdd(self, item, container)

        try:
            portal = aq_parent(aq_inner(self))
            props = portal.portal_properties.cmfboard_properties

            allow_signatures = props.allow_signatures
            allow_outside_forumnb = props.allow_outside_forumnb
            view_mode_selectable = props.view_mode_selectable
            poll_slot_configuration = props.poll_slot_configuration
        except:
            pass

    def manage_beforeDelete(self, item, container):
        if item is self:
            handle = self.meta_type + '/' + self.getId()
            BeforeTraverse.unregisterBeforeTraverse(container, handle)
        
    security.declareProtected(ManagePortal, 'manage_migrate')
    def manage_migrate(self, force = 0, REQUEST=None):
        """Migrate tool"""
        migrate(self, force)
        
        if REQUEST is not None:
            message = "Migration completed."
            REQUEST.RESPONSE.redirect('%s/manage_migrationForm?manage_tabs_message=%s'%(
                self.absolute_url(), message))

    security.declareProtected(View, 'forumIcons')
    def forumIcons(self):
        """ smiles from skins """
        return smiles.icons_cats

    security.declarePrivate('getView')
    def getView(self, uid, view):
        if _cache_data.has_key(uid):
            return _cache_data[uid].get(view, None)

        return None

    security.declarePrivate('setView')
    def setView(self, uid, view, data):
        rec = _cache_data.setdefault(uid, {})
        rec[view] = data

    security.declarePrivate('clearView')
    def clearView(self, uid, view):
        rec = _cache_data.get(uid, {})
        try:
            del rec[view]
        except KeyError:
            pass

    security.declarePublic('initCMFBoardViews')
    def initCMFBoardViews(self, obj):
        """ set globals vars for request """
        request = self.REQUEST

        if not getattr(request, 'cmfboard', 0):
            # globals
            data = {}

            portal = aq_parent(aq_inner(self))

            data['portal'] = portal
            data['portal_url'] = portal.absolute_url()
            data['ctool'] = portal.portal_catalog
            data['tool'] = self

            mtool = portal.portal_membership
            data['mtool'] = mtool
            
            data['wtool'] = portal.portal_workflow

            member = mtool.getAuthenticatedMember()

            data['member'] = member
            data['timezone'] = member.getProperty('timezone','')

            properties = portal.portal_properties.site_properties

            data['isAnon'] = mtool.isAnonymousUser()
            data['encoding'] = properties.default_charset

            forumTime = translate_wrapper('plone', 'date_format_long', context=portal)
            if not forumTime:
                forumTime = properties.localLongTimeFormat
                
            data['forumTime'] = forumTime

            request.cmfboard_tool = self
            request.cmfboard_data = data
            request.cmfboard = 1

        data = request.cmfboard_data
        
        try:
            if not hasattr(request, 't_data'):
                topic = obj.getTopic()
                data['topic'] = topic
                request.t_data = topic.view_getGlobals(request)
        except:
            pass
            
        try:
            if not hasattr(request, 'f_data'):
                forum = obj.getForum()
                data['forum'] = forum
                request.f_data = forum.view_getGlobals(request)
        except:
            pass

        return request.cmfboard_data


InitializeClass(UtilitiesTool)
