# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nikolay Kim <fafhrd@legco.biz> 
"""
$Id: forum_codes.py,v 1.17.4.4.2.2 2005/05/25 05:04:59 fafhrd91 Exp $
"""

import re, traceback
import string

interesting = re.compile('\[', re.UNICODE)
incomplete = re.compile('&([a-zA-Z][a-zA-Z0-9]*|#[0-9]*)?|'
                        '\[([a-zA-Z][^\[\]]*|'
                        '/([a-zA-Z][^\[\]]*)?|'
                        '![^\[\]]*)?', re.UNICODE)

starttagopen = re.compile('\[[\]a-zA-Z]', re.UNICODE)
shorttagopen = re.compile('\[[a-zA-Z][a-zA-Z0-9]*/', re.UNICODE)
shorttag = re.compile('\[([a-zA-Z][a-zA-Z0-9]*)/([^/]*)/', re.UNICODE)
endtagopen = re.compile('\[/[\[\]a-zA-Z]', re.UNICODE)
endbracket = re.compile('[\[\]]', re.UNICODE)

startexttag = re.compile(':[a-zA-Z0-9][a-zA-Z0-9+\-_]*', re.UNICODE)

tagfind = re.compile('[a-zA-Z][a-zA-Z0-9]*', re.UNICODE)
attrfind = re.compile(
    r'[ \t\r\n]*([a-zA-Z_][-.a-zA-Z_0-9]*)'
    r'([ \t\r\n]*=[ \t\r\n]*'
    r'(\'[^\']*\'|"[^"]*"|\&quot;[^"]*&quot;|[-a-zA-Z0-9./:+*%?!\(\)_#=~]*))?', re.UNICODE)


class Parser:

    def __init__(self, verbose=1):
        self.verbose = verbose
        self.reset()

    def reset(self):
        self.rawdata = ''
        self.stack = []
        self.lasttag = '???'
        self.nomoretags = 0
        self.literal = 0

    def setnomoretags(self):
        self.nomoretags = self.literal = 1

    def setliteral(self, *args):
        self.literal = 1

    def feed(self, data):
        self.rawdata = self.rawdata + data
        self.goahead(0)

    # Interface -- handle the remaining data
    def close(self):
        self.goahead(1)

    def goahead(self, end):
        rawdata = self.rawdata
        i = 0
        n = len(rawdata)
        while i < n:
            if self.nomoretags:
                self.handle_data(rawdata[i:n])
                i = n
                break
            match = interesting.search(rawdata, i)
            if match: j = match.start(0)
            else: j = n
            if i < j: self.handle_data(rawdata[i:j])
            i = j
            if i == n: break

            if starttagopen.match(rawdata, i):
                if self.literal:
                    self.handle_data(rawdata[i])
                    i = i+1
                    continue
                k = self.parse_starttag(i)
                if k < 0: break
                i = k
                continue

            if endtagopen.match(rawdata, i):
                k = self.parse_endtag(i)
                if k < 0: break
                i =  k
                self.literal = 0
                continue
            
            # We get here only if incomplete matches but
            # nothing else
            match = incomplete.match(rawdata, i)
            if not match:
                self.handle_data(rawdata[i])
                i = i+1
                continue
            j = match.end(0)
            if j == n:
                break # Really incomplete
            self.handle_data(rawdata[i:j])
            i = j
        # end while
        if end and i < n:
            self.handle_data(rawdata[i:n])
            i = n
        self.rawdata = rawdata[i:]

        stack = self.stack
        for idx in range(len(stack)-1, -1, -1):
            self.finish_endtag(stack[idx][0])

    def parse_starttag(self, i):
        rawdata = self.rawdata
        if shorttagopen.match(rawdata, i):
            match = shorttag.match(rawdata, i)
            if not match:
                return -1
            tag, data = match.group(1, 2)
            tag = string.lower(tag)
            self.finish_shorttag(tag, data)
            k = match.end(0)
            return k
        # XXX The following should skip matching quotes (' or ")
        match = endbracket.search(rawdata, i+1)
        if not match:
            return -1
        j = match.start(0)
        # Now parse the data between i+1 and j into a tag and attrs
        attrs = []
        if rawdata[i:i+2] == '[]':
            # SGML shorthand: <> == <last open tag seen>
            k = j
            tag = self.lasttag
        else:
            match = tagfind.match(rawdata, i+1)
            if not match:
                raise RuntimeError, 'unexpected call to parse_starttag'
            k = match.end(0)
            tag = string.lower(rawdata[i+1:k])
            self.lasttag = tag

        match = startexttag.match(rawdata, k)
        if match:
            k1 = match.end(0)
            exttag = string.lower(rawdata[k+1:k1])
            k = k1
        else:
            exttag = ''

        while k < j:
            match = attrfind.match(rawdata, k)
            if not match: break
            attrname, rest, attrvalue = match.group(1, 2, 3)
            if not rest:
                attrvalue = attrname
            elif attrvalue[:1] == '\'' == attrvalue[-1:] or \
                 attrvalue[:1] == '"' == attrvalue[-1:]:
                attrvalue = attrvalue[1:-1]
            elif attrvalue[:6] == '&quot;' == attrvalue[-6:]:
                attrvalue = attrvalue[6:-6]
            attrs.append((string.lower(attrname), attrvalue))
            k = match.end(0)
        if rawdata[j] == ']':
            j = j+1

        nattrs = {}
        for key, val in attrs:
            nattrs[key] = val

        self.finish_starttag(tag, exttag, nattrs)
        return j

    # Internal -- parse endtag
    def parse_endtag(self, i):
        rawdata = self.rawdata
        match = endbracket.search(rawdata, i+1)
        if not match:
            return -1
        j = match.start(0)
        tag = string.lower(string.strip(rawdata[i+2:j]))
        if rawdata[j] == ']':
            j = j+1
        self.finish_endtag(tag)
        return j

    # Internal -- finish parsing of <tag/data/ (same as [tag]data[/tag])
    def finish_shorttag(self, tag, data):
        try:
            self.finish_starttag(tag, [])
            self.handle_data(data)
            self.finish_endtag(tag)
        except:
            traceback.print_exc()

    def finish_starttag(self, tag, exttag, attrs):
        try:
            method = getattr(self, 'start_' + tag)
        except AttributeError:
            try:
                method = getattr(self, 'do_' + tag)
            except AttributeError:
                self.stack.append([tag, exttag, attrs])
                self.unknown_starttag(tag, exttag, attrs)
                return 0

        self.stack.append([tag, exttag, attrs])
        self.handle_starttag(tag, exttag, method, attrs)
        return 1

    # Internal -- finish processing of end tag
    def finish_endtag(self, tag):
        if not tag:
            found = len(self.stack) - 1
            if found < 0:
                self.unknown_endtag(tag)
                return
        else:
            found = len(self.stack)
            for i in range(found):
                if self.stack[i][0] == tag:
                    found = i

            if len(self.stack) == found:
                try:
                    method = getattr(self, 'end_' + tag)
                except AttributeError:
                    self.unknown_endtag(tag)
                return

        while len(self.stack) > found:
            tag = self.stack[-1][0]
            try:
                method = getattr(self, 'end_' + tag)
            except AttributeError:
                method = None
            if method:
                self.handle_endtag(tag, method)
            else:
                self.unknown_endtag(tag)
            del self.stack[-1]

    def handle_starttag(self, tag, exttag, method, attrs):
        method(exttag, attrs)

    def handle_endtag(self, tag, method):
        method()

    def report_unbalanced(self, tag):
        if self.verbose:
            print '*** Unbalanced </' + tag + '>'
            print '*** Stack:', self.stack

    # Example -- handle data, should be overridden
    def handle_data(self, data):
        pass

    # To be overridden -- handlers for unknown objects
    def unknown_starttag(self, tag, exttag, attrs): pass
    def unknown_endtag(self, tag): pass
    

from config import icons
from utils import html_unquote
from smiles import parse_smiles
from DocumentTemplate.DT_Var import \
     html_quote, newline_to_br, url_quote_plus

start_tags = {
    'b': ' <strong>',
    'strong': ' <strong>',
    'i': ' <em>',
    'em': ' <em>',
    'u': ' <span class="underline">',
    'ul': ' <ul>',
    }

end_tags = {
    'b': '</strong> ',
    'strong': '</strong> ',
    'i': '</em> ',
    'em': '</em> ',
    'quote': '</blockquote></div>',
    'code': '</code></div></div>',
    'big': '</span> ',
    'size': '</span> ',
    'color': '</span> ',
    'u': '</span> ',
    'ul': '</ul>',
    'ol': '</ol>',
    }

end_tags_desc = {
    'b': '</strong> ',
    'strong': '</strong> ',
    'i': '</em> ',
    'em': '</em> ',
    'ul': '</ul> '
    }

end_tags_text = { }
end_tags_email = { }

class CMFBoardCodes(Parser):

    def parse_text(self, text, context, target='www', mimetype='text/plain', emoticons=1):
        self.reset()
        text = self.feed(text, context, target, mimetype, emoticons)
        self.reset()
        return text

    def feed(self, text, context, target='www', mimetype='text/plain', emoticons=1,
             size_limit=0):

        self.body = ''
        self.target = target
        self.context = context
        self.mimetype = mimetype
        self.emoticons = emoticons
        self.size_limit = size_limit
        self.block = 0

        self.portal_url = context.portal_url()
        
        Parser.feed(self, text)
        self.context = None

        body = self.body

        self.body = None
        return body

    def unknown_starttag(self, tag, exttag, attrs):
        if start_tags.has_key(tag):
            self.body += start_tags[tag]
    
    def unknown_endtag(self, tag):
        target = self.target

        if (target == 'www') and end_tags.has_key(tag):
            self.body += end_tags[tag]
        if (target == 'desc') and end_tags_desc.has_key(tag):
            self.body += end_tags_desc[tag]

    def handle_data(self, data):
        if self.block:
            return

        try:
            tag = self.stack[-1][0]
            method = getattr(self, 'handle_data_%s'%tag, None)
            if callable(method):
                method(data)
                return
        except:
            #traceback.print_exc()
            pass

        body = data

        if self.size_limit:
            body_len = len(self.body)

            new_len = self.size_limit - (body_len + len(body))

            if new_len < 0:
                body = body[:new_len]
                self.block = 1

        if self.emoticons and (self.target != 'email'):
            body = parse_smiles(self.context, body)

        self.body += body
        #print 'len', len(self.body)

    def handle_data_img(self, text):
        if text and not self.block:
            target = self.target
            if target == 'www':
                self.body += ' <img src="%s" border="0" /> '%text
            elif target in ('desc', 'email'):
                self.body += ' Image:%s'%text

    def handle_data_email(self, text):
        if text and not self.block:
            target = self.target
            if target == 'email':
                self.body += ' ' + text
            else:
                self.body += ' <a href="mailto:%s">%s</a> '%(text, text)

    def start_quote(self, exttag, attrs):
        target = self.target

        if self.block or (target == 'desc'):
            return
        
        if target == 'www':
            self.body += '<div class="cquote">'
            if exttag:
                self.body += '<div class="codeHeader">%s wrote:</div>'%exttag

            self.body += '<blockquote>'
        elif target == 'email':
            if exttag:
                self.body += '%s wrote:\n'%exttag

    def end_quote(self):
        target = self.target
        
        if self.block or (target == 'desc'):
            return

        if target == 'email':
            self.body += '\n\n'
        else:
            self.body += end_tags['quote']

    def handle_data_quote(self, text):
        if self.target == 'desc' or self.block:
            return

        attrs = self.stack[-1][2]

        format = attrs.get('format', '')
        if format == 'text/html':
            text = html_unquote(text)
        elif format == 'text/plain':
            if self.mimetype == 'text/html':
                text = html_quote(text)
                text = newline_to_br(text)

        if self.emoticons and (self.target != 'email'):
            self.body += parse_smiles(self.context, text)
        else:
            if self.target == 'email':
                self.body += ''.join(['\n> %s'%line for line in text.split('\n')])
            else:
                self.body += text

    def start_code(self, exttag, attrs):
        if self.block:
            return
        
        count = 0
        for tag in self.stack:
            if tag[0] == 'code':
                count += 1
        if count > 1:
            if exttag:
                self.body += '[code:%s]'%exttag
            else:
                self.body += '[code]'
        else:
            if self.target == 'email':
                self.body += '\n'
                    
            else:
                self.body += '<div class="code">'

                if exttag:
                    self.body += '<div class="codeHeader">code (%s):</div>'%exttag
                else:
                    self.body += '<div class="codeHeader">code:</div>'

                self.body += '<div class="codeBody"><code>'

    def end_code(self):
        if self.block:
            return

        count = 0
        for tag in self.stack:
            if tag[0] == 'code':
                count += 1
        
        if count > 1:
            self.body += '[/code]'
        else:
            if self.target == 'email':
                self.body += '/n/n'
            else:
                self.body += end_tags['code']

    def handle_data_code(self, text):
        if self.block:
            return
        
        text = text.strip()
        target = self.target
        
        count = 0
        for tag in self.stack:
            if tag[0] == 'code':
                count += 1
        if count <= 1:
            if target != 'email':
                if text.startswith('<br />'):
                    text = text[6:]
                elif text.startswith('<br/>'):
                    text = text[5:]

        if target != 'email':
            text = text.replace(' ', '&nbsp;')
            text = text.replace('<br&nbsp;/>', '<br />')
            
        self.body += text

    def start_url(self, exttag, attrs):
        if self.block:
            return

        target = self.target

        count = 0
        for tag in self.stack:
            if tag[0] == 'url':
                count += 1
        if count <= 1:
            if attrs.has_key('href'):
                if target == 'email':
                    self.body += ' %s '%attrs['href']
                else:
                    self.body += ' <a href="%s" target="_blank">'%attrs['href']
            else:
                if target != 'email':
                    self.body += ' <a href="#" target="_blank">'
                    
                self.stack[-1][2]['hint'] = len(self.body) - 19

    def end_url(self):
        if self.block or (self.target == 'email'):
            return

        count = 0
        for tag in self.stack:
            if tag[0] == 'url':
                count += 1
        if count <= 1:
            self.body += '</a> '

    def handle_data_url(self, text):
        if self.block:
            return
        
        text = text.strip()
        tag_info = self.stack[-1]

        if self.target == 'email':
            if tag_info[2].has_key('hint'):
                self.body += ' %s '%text
        else:
            if tag_info[2].has_key('hint'):
                hint = tag_info[2]['hint']
                self.body = self.body[:hint] + text + self.body[hint+1:]

            if self.emoticons:
                self.body += parse_smiles(self.context, text)
            else:
                self.body += text

    def start_size(self, exttag, attrs):
        if self.target in ('desc', 'email') or self.block:
            return

        if exttag:
            try:
                s = int(exttag)
                exttag = '%spx'%exttag
            except:
                pass
            self.body += ' <span style="font-size:%s; line-height:normal">'%exttag
        else:
            self.body += ' <span style="font-size:large; line-height:normal">'

    start_big = start_size

    def start_color(self, exttag, attrs):
        if self.target in ('desc', 'email') or self.block:
            return

        if exttag:
            self.body += ' <span style="color: %s">'%exttag

    def start_ol(self, exttag, attrs):
        if self.target in ('desc', 'email') or self.block:
            return

        if exttag in ('A', 'a', 'I', 'i', '1'):
            self.body += '<ol type="%s">'%exttag
        else:
            self.body += '<ol>'

    def start_li(self, exttag, attrs):
        if self.target == 'desc' or self.block:
            return

        try:
            if self.stack[-2][0] in ('ul', 'ol'):
                if self.target == 'email':
                    self.body += ' *'
                else:
                    self.body += '<li>'
        except:
            pass

    def end_li(self):
        if self.target == 'desc' or self.block:
            return

        try:
            if self.stack[-2][0] in ('ul', 'ol'):
                if self.target == 'email':
                    if self.body[-1] != '/n':
                        self.body += '/n'
                else:
                    self.body += '</li>'
        except:
            pass

    def start_icon(self, exttag, attrs):
        if self.block:
            return
        
        if icons.has_key(exttag):
            data = icons[exttag]
            target = self.target
            
            if target in ('www', 'desc'):
                tag = ' <img src="%s/%s" alt=":%s:" title="%s" width="%s" height="%s" /> '% (
                    self.portal_url, data['path'], data['title'], data['title'],
                    data['width'], data['height'])
                self.body += tag
            elif target == 'email':
                self.body += ':%s:'%data['title']
            

parser = CMFBoardCodes()
