# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 

import zlib, traceback
import types, os
from types import StringType, UnicodeType
from cPickle import loads, dumps
from DateTime import DateTime
from Acquisition import aq_parent, aq_base, aq_inner
from Products.CMFCore.utils import _checkPermission, getToolByName
from Products.CMFCore.CMFCorePermissions import ManagePortal

try:
    from App.config import getConfiguration
    cfg = getConfiguration()
    INSTANCE_HOME = cfg.instancehome
except:
    from Globals import INSTANCE_HOME

import StringIO, binascii
from xml.sax.saxutils import XMLGenerator
from xml.sax.xmlreader import AttributesNSImpl, AttributesImpl

from config import memberData_properties

StringTypes = (StringType, UnicodeType)

attrs_list = [
    '_Access_contents_information_Permission',
    '_List_folder_contents_Permission',
    '_Modify_portal_content_Permission',
    '_Request_review_Permission',
    '_Review_portal_content_Permission',
    '_View_Permission',
    '_CMFBoard__Add_forum_topic_Permission',
    '_CMFBoard__Add_forum_topic_reply_Permission',
    '_CMFBoard__Add_forum_poll_Permission',
    '_CMFBoard__Manage_forum_Permission',
    '_CMFBoard__Manage_forum_topic_Permission',
    '_CMFBoard__Modify_forum_topic_Permission',
    '_CMFBoard__Modify_topic_attachements_Permission',
    '_CMFBoard__Modify_forum_topic_reply_Permission',
    '_CMFBoard__Modify_reply_attachements_Permission',
    '_CMFBoard__View_admin_topic_Permission',
    '_CMFBoard__Delete_forum_topic_Permission',
    '_CMFBoard__Delete_forum_reply_Permission',

    '_owner',
    '__ac_local_roles__',
    'portal_type',
    'workflow_history',
    'creation_date',
    'modification_date',
    'topic_type',
    'hits',
    '_anonymous',
    '_subscription',
    '_votes',
    ]

types_map = {
    types.NoneType: '_NoneType', types.IntType: '_IntType',
    types.LongType: '_LongType', types.FloatType: '_FloatType',
    types.StringType: '_StringType', types.UnicodeType: '_UnicodeType',
    types.TupleType: '_TupleType', types.ListType: '_ListType',
    types.DictType: '_DictType',
    DateTime.__name__: '_DateTimeType',
    'PersistentMapping': '_PersistentMappingType',
    'Length': '_LengthType',
    }

try:
    types_map[types.BooleanType] = '_BooleanType'
except:
    pass


class ForumExporter:

    def __init__(self, output, portal, encoding='utf-8'):
        """ """
        xmlgen = XMLGenerator(output, encoding)
        xmlgen.startDocument()
        self._xmlgen = xmlgen
        self._output = output
        self._encoding = encoding
        self.portal = portal

        self._dict = {'id': 1}

    def startElement(self, name, attrs):
        self._xmlgen.startElement(name, AttributesImpl(attrs))

    def endElement(self, name):
        self._xmlgen.endElement(name)

    def characters(self, val):
        self._xmlgen.characters(val)

    def _IntegerField(self, object, field, charset):
        attrs = AttributesImpl({u'id': unicode(field.getName())})

        self._xmlgen.startElement(u'IntegerField', attrs)
        self.export_value('', field.get(object), charset)
        self._xmlgen.endElement(u'IntegerField')

    def _BooleanField(self, object, field, charset):
        attrs = AttributesImpl({u'id': unicode(field.getName())})

        self._xmlgen.startElement(u'BooleanField', attrs)
        self.export_value('', field.get(object), charset)
        self._xmlgen.endElement(u'BooleanField')

    def _StringField(self, object, field, charset):
        attrs = AttributesImpl({u'id': unicode(field.getName())})

        self._xmlgen.startElement(u'StringField', attrs)
        self.export_value('', field.get(object), charset)
        self._xmlgen.endElement(u'StringField')

    def _TextField(self, object, field, charset):
        field_id = field.getName()
        mimetype = object.getContentType(field_id)
        attrs = AttributesImpl({u'id': unicode(field_id),
                                u'mimetype': mimetype})

        self._xmlgen.startElement(u'TextField', attrs)
        self.export_value('', field.get(object, mimetype=mimetype), charset)
        self._xmlgen.endElement(u'TextField')

    def _ImageField(self, object, field, charset):
        field_id = field.getName()

        image = aq_base(field.get(object))
        if not image:
            return            

        attrs_val = {u'id': unicode(field_id),
                     u'content_type': unicode(image.content_type)}
        try:
            attrs_val[u'filename'] = unicode(image.filename)
        except:
            pass

        self._xmlgen.startElement(u'ImageField', AttributesImpl(attrs_val))
        self.export_value('', binascii.b2a_base64(str(image.data)), charset)
        self._xmlgen.endElement(u'ImageField')

    def _ForumFileField(self, object, field, charset):
        field_id = field.getName()

        file = aq_base(field.get(object))
        if not file:
            return

        attrs_val = {u'id': unicode(field_id),
                     u'content_type': unicode(file.content_type)}
        try:
            attrs_val[u'filename'] = unicode(file.filename)
        except:
            pass
        
        attrs = AttributesImpl(attrs_val)

        self._xmlgen.startElement(u'ForumFileField', attrs)
        self.export_value('', binascii.b2a_base64(str(file.data)), charset)
        self._xmlgen.endElement(u'ForumFileField')

    def _LinesField(self, object, field, charset):
        attrs = AttributesImpl({u'id': unicode(field.getName())})

        self._xmlgen.startElement(u'LinesField', attrs)
        self.export_value('', field.get(object), charset)
        self._xmlgen.endElement(u'LinesField')

    def _DateTimeField(self, object, field, charset):
        attrs = AttributesImpl({u'id': unicode(field.getName())})

        self._xmlgen.startElement(u'DateTimeField', attrs)
        self.export_value('', field.get(object), charset)
        self._xmlgen.endElement(u'DateTimeField')

    def export_Fields(self, instance):
        attrs = AttributesImpl({})
        self._xmlgen.startElement(u'fields', attrs)

        for field in instance.Schema().values():
            if field.getName() == 'id':
                continue
            
            charset = instance.getCharset()

            name = field.__class__.__name__
            m = getattr(self, '_%s'%name, None)
            if m is not None:
                m(instance, field, charset)

        self._xmlgen.endElement(u'fields')

    def export_Attributes(self, instance):
        attrs = AttributesImpl({})
        self._xmlgen.startElement(u'attributes', attrs)

        ob = aq_base(instance)
        charset = instance.getCharset()

        for key, val in ob.__dict__.items():
            if key in attrs_list:
                self.export_value('', key, charset)
                self.export_value('', val, charset)
        
        self._xmlgen.endElement(u'attributes')

    def export_value(self, id, value, charset):
        mname = types_map.get(type(value), None)

        if mname is None:
            try:
                mname = types_map.get(value.__class__.__name__, None)
            except:
                pass

        if mname is None:
            mname = '_Pickled'

        m = getattr(self, mname, None)
        if m:
            m(id, value, charset)

    def _Pickled(self, id, value, charset):
        if id:
            attrs = AttributesImpl({u'id': unicode(id, charset)})
        else:
            attrs = AttributesImpl({})
        self._xmlgen.startElement(u'Pickled', attrs)
        self._xmlgen.characters(binascii.b2a_base64(dumps(value, 1)))
        self._xmlgen.endElement(u'Pickled')

    def _NoneType(self, id, value, charset):
        if id:
            attrs = AttributesImpl({u'id': unicode(id, charset)})
        else:
            attrs = AttributesImpl({})
        self._xmlgen.startElement(u'none', attrs)
        self._xmlgen.characters(u'None')
        self._xmlgen.endElement(u'none')

    def _IntType(self, id, value, charset):
        if id:
            attrs = AttributesImpl({u'id': unicode(id, charset)})
        else:
            attrs = AttributesImpl({})
        self._xmlgen.startElement(u'int', attrs)
        self._xmlgen.characters(unicode(str(value)))
        self._xmlgen.endElement(u'int')

    def _LengthType(self, id, value, charset):
        if id:
            attrs = AttributesImpl({u'id': unicode(id, charset)})
        else:
            attrs = AttributesImpl({})
        self._xmlgen.startElement(u'Length', attrs)
        self._xmlgen.characters(unicode(int(value.value)))
        self._xmlgen.endElement(u'Length')

    def _LongType(self, id, value, charset):
        if id:
            attrs = AttributesImpl({u'id': unicode(id, charset)})
        else:
            attrs = AttributesImpl({})
        self._xmlgen.startElement(u'long', attrs)
        self._xmlgen.characters(unicode((value)))
        self._xmlgen.endElement(u'long')

    def _FloatType(self, id, value, charset):
        if id:
            attrs = AttributesImpl({u'id': unicode(id, charset)})
        else:
            attrs = AttributesImpl({})
        self._xmlgen.startElement(u'float', attrs)
        self._xmlgen.characters(unicode((value)))
        self._xmlgen.endElement(u'float')

    def _BooleanType(self, id, value, charset):
        if id:
            attrs = AttributesImpl({u'id': unicode(id, charset)})
        else:
            attrs = AttributesImpl({})
        self._xmlgen.startElement(u'bool', attrs)
        self._xmlgen.characters(unicode((value)))
        self._xmlgen.endElement(u'bool')

    def _StringType(self, id, value, charset):
        if id:
            attrs = AttributesImpl({u'id': unicode(id, charset)})
        else:
            attrs = AttributesImpl({})
        self._xmlgen.startElement(u'string', attrs)
        self._xmlgen.characters(unicode(value, charset))
        self._xmlgen.endElement(u'string')

    def _UnicodeType(self, id, value, charset):
        if id:
            attrs = AttributesImpl({u'id': unicode(id, charset)})
        else:
            attrs = AttributesImpl({})
        self._xmlgen.startElement(u'unicode', attrs)
        self._xmlgen.characters(value)
        self._xmlgen.endElement(u'unicode')

    def _TupleType(self, id, value, charset):
        if id:
            attrs = AttributesImpl({u'id': unicode(id, charset)})
        else:
            attrs = AttributesImpl({})
        self._xmlgen.startElement(u'tuple', attrs)
        for val in value:
            self.export_value('', val, charset)
        self._xmlgen.endElement(u'tuple')

    def _ListType(self, id, value, charset):
        if id:
            attrs = AttributesImpl({u'id': unicode(id, charset)})
        else:
            attrs = AttributesImpl({})
        self._xmlgen.startElement(u'list', attrs)
        for val in value:
            self.export_value('', val, charset)
        self._xmlgen.endElement(u'list')

    def _DictType(self, id, value, charset):
        if id:
            attrs = AttributesImpl({u'id': unicode(id, charset)})
        else:
            attrs = AttributesImpl({})
        self._xmlgen.startElement(u'dictionary', attrs)
        for key, val in value.items():
            self.export_value('', key, charset)
            self.export_value('', val, charset)
        self._xmlgen.endElement(u'dictionary')

    def _PersistentMappingType(self, id, value, charset):
        if id:
            attrs = AttributesImpl({u'id': unicode(id, charset)})
        else:
            attrs = AttributesImpl({})
        self._xmlgen.startElement(u'PersistentMapping', attrs)
        for key, val in value.items():
            self.export_value('', key, charset)
            self.export_value('', val, charset)
        self._xmlgen.endElement(u'PersistentMapping')

    def _DateTimeType(self, id, value, charset):
        if id:
            attrs = AttributesImpl({u'id': unicode(id, charset)})
        else:
            attrs = AttributesImpl({})
        self._xmlgen.startElement(u'DateTime', attrs)
        self._xmlgen.characters(unicode(value))
        self._xmlgen.endElement(u'DateTime')

    def export_ForumFolder(self, folder):
        attr_vals = {u'id': unicode(folder.getId()),
                     u'uid': unicode(folder.UID())}

        self._xmlgen.startElement(u'ForumFolder', AttributesImpl(attr_vals))
        
        self.export_Fields(folder)
        self.export_Attributes(folder)

        # export forums
        self._xmlgen.startElement(u'forums', AttributesImpl({}))
        for forum in folder.contentValues():
	    if forum.portal_type == 'ForumNB':
		self.export_Forum(forum)
        self._xmlgen.endElement(u'forums')

        # import nested folders
        self._xmlgen.startElement(u'folders', AttributesImpl({}))
        for folder in folder.contentValues():
	    if folder.portal_type == 'ForumFolder':
                self.export_ForumFolder(folder)
        self._xmlgen.endElement(u'folders')
        
        self._xmlgen.endElement(u'ForumFolder')

    def export_Forum(self, forum):
        attr_vals = {u'id': unicode(forum.getId()),
                     u'uid': unicode(forum.UID())}

        self._xmlgen.startElement(u'ForumNB', AttributesImpl(attr_vals))

        self.export_Fields(forum)
        self.export_Attributes(forum)

        self._xmlgen.startElement(u'topics', AttributesImpl({}))
        for topic in forum.contentValues():
            if topic.portal_type == 'ForumTopic':
                self.export_ForumTopic(topic)
            elif topic.portal_type == 'ForumPoll':
                self.export_ForumPoll(topic)
            elif topic.portal_type == 'ForumContentTopic':
                self.export_ForumContentTopic(topic)
                
        self._xmlgen.endElement(u'topics')
        self._xmlgen.endElement(u'ForumNB')
    

    def export_ForumTopic(self, topic, attrs={}):
        attr_vals = {}
        attr_vals.update(attrs)
        attr_vals.update({u'id': unicode(topic.getId()),
                          u'uid': unicode(topic.UID())})

        pt = topic.portal_type

        exporter = self._xmlgen
        
        exporter.startElement(unicode(pt), AttributesImpl(attr_vals))

        self.export_Fields(topic)
        self.export_Attributes(topic)

        exporter.startElement(u'children', AttributesImpl({}))

        proccesed_uids = []
        for msg in topic.getChildren():
            self.export_ForumMessage(msg, proccesed_uids)

        for msg in topic.contentValues():
            if msg.UID() not in proccesed_uids:
                self.export_ForumMessage(msg, proccesed_uids)
                
        exporter.endElement(u'children')

        exporter.endElement(unicode(pt))


    def export_ForumContentTopic(self, topic):
        object = topic._get_content()

        attrs = {}
        if object is not None:
            attrs[u'object'] = unicode('/'.join(
                self.portal.portal_url.getRelativeContentPath(object)))
            attrs[u'isDiscussionAllowed'] = (
                unicode(self.portal.portal_discussion.isDiscussionAllowedFor(object)))
            
        self.export_ForumTopic(topic, attrs)

    export_ForumPoll = export_ForumTopic


    def export_ForumMessage(self, msg, proccesed_uids=None):
        attr_vals = {u'id': unicode(msg.getId()),
                     u'uid': unicode(msg.UID())}

        if proccesed_uids is None:
            proccesed_uids = []
        
        if msg.UID() in proccesed_uids:
            return
        
        proccesed_uids.append(msg.UID())

        exporter = self._xmlgen

        exporter.startElement(u'ForumMessage', AttributesImpl(attr_vals))

        self.export_Fields(msg)
        self.export_Attributes(msg)

        exporter.startElement(u'children', AttributesImpl({}))
        for msg in msg.getChildren():
            self.export_ForumMessage(msg, proccesed_uids)
        exporter.endElement(u'children')

        exporter.endElement(u'ForumMessage')


    def close(self):
        self._xmlgen.endDocument()
        return


def forum_export(self, forum_folder):
    if not _checkPermission(ManagePortal, self):
        raise 'Unauthorized'

    portal = getToolByName(self, 'portal_url').getPortalObject()
    atool = portal.archetype_tool

    folder = atool.lookupObject(forum_folder)
    if folder is None:
        self.REQUEST.RESPONSE.redirect(
            '%s/forum_import_export_form?portal_status_message=Forum Folder not found.'%
            portal.absolute_url())

    filename = '%s.data'%'_'.join(folder.getPhysicalPath()[1:])

    filepath = os.path.join(INSTANCE_HOME, 'var', filename)
    
    out = open(filepath, 'wb')

    exporter = ForumExporter(out, portal)

    exporter.startElement(
        u'Data', {'path': '/'.join(folder.getPhysicalPath())})

    if folder.portal_type == 'ForumFolder':
        exporter.export_ForumFolder(folder)
    elif folder.portal_type == 'ForumNB':
        exporter.export_Forum(folder)
    
    export_users_data(exporter, portal)

    exporter.endElement(u'Data')

    exporter.close()

    out.close()


def export_users_data(exporter, portal):
    mtool = portal.portal_membership

    exporter.startElement(u'UsersData', {})

    charset = portal.portal_properties.site_properties.default_charset

    for user_id in portal.acl_users.getUserNames():
	user = mtool.getMemberById(user_id)
	if user:
            data = {}
            for id, tp, de in memberData_properties:
                if id != 'fc_internal_data':
                    data[unicode(id)] = user.getProperty(id, '')

            exporter.startElement(u'User', {u'id': unicode(user_id)})
            exporter.export_value('', data, charset)
            exporter.endElement(u'User')

    exporter.endElement(u'UsersData')
