# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 

import zlib, StringIO, binascii, traceback
from types import StringType, UnicodeType
from cPickle import loads, dumps
from DateTime import DateTime
from BTrees.Length import Length
from Acquisition import aq_parent, aq_base, aq_inner
from Products.CMFCore.utils import _checkPermission, getToolByName
from Products.CMFCore.CMFCorePermissions import ManagePortal
from ZODB.PersistentMapping import PersistentMapping

from xml.sax import saxlib, saxexts, saxutils
from xml.marshal.generic import Unmarshaller

from debug import log, log_exc

StringTypes = (StringType, UnicodeType)

attrs_mapping = {
    '_Add_forum_topic_Permission': '_CMFBoard__Add_forum_topic_Permission',
    '_Add_forum_topic_reply_Permission': '_CMFBoard__Add_forum_topic_reply_Permission',
    '_Manage_forum_Permission': '_CMFBoard__Manage_forum_Permission',
    '_Manage_forum_topic_Permission': '_CMFBoard__Manage_forum_topic_Permission',
    '_Modify_forum_topic_Permission': '_CMFBoard__Modify_forum_topic_Permission',
    '_Modify_forum_topic_reply_Permission': '_CMFBoard__Modify_forum_topic_reply_Permission',
    '_View_admin_topic_Permission': '_CMFBoard__View_admin_topic_Permission',
    '_Delete_forum_topic_Permission': '_CMFBoard__Delete_forum_topic_Permission',
    '_Delete_forum_reply_Permission': '_CMFBoard__Delete_forum_reply_Permission',
    }

string_fields = ['title', 'desscription', 'text', 'authorname']

uid_mapping = {}

class ForumImporter(Unmarshaller, saxutils.DefaultHandler):
    unmarshal_meth = {
        'Data': (None, None),

        'fields': (None, None),
        'folders': (None, None),
        'forums': (None, None),
        'attributes': ('start_attributes', 'end_attributes'),
        'topics': (None, None),
        'children': (None, None),
        
        'ForumFolder': ('start_ForumFolder', 'end_generic_ForumType'),
        'ForumNB': ('start_ForumNB', 'end_ForumNB'),
        'ForumTopic': ('start_ForumTopic', 'end_generic_ForumType'),
        'ForumPoll': ('start_ForumTopic', 'end_generic_ForumType'),
        'ForumContentTopic': ('start_ForumContentTopic', 'end_generic_ForumType'),
        'ForumMessage': ('start_ForumMessage', 'end_generic_ForumType'),

        'IntegerField': ('start_IntegerField', 'end_IntegerField'),
        'BooleanField': ('start_BooleanField', 'end_BooleanField'),
        'StringField': ('start_StringField', 'end_StringField'),
        'TextField': ('start_TextField', 'end_TextField'),
        'ImageField': ('start_ImageField', 'end_ImageField'),
        'ForumFileField': ('start_ForumFileField', 'end_ForumFileField'),
        'LinesField': ('start_LinesField', 'end_LinesField'),
        'DateTimeField': ('start_DateTimeField', 'end_DateTimeField'),

        'bool': ('start_boolean', 'end_boolean'),
        'unicode': ('start_unicode', 'end_unicode'),
        'Pickled': ('start_Pickled', 'end_Pickled'),

        'Length': ('start_Length', 'end_Length'),
        'PersistentMapping': ('start_PersistentMapping', 'end_PersistentMapping'),
        'DateTime': ('start_DateTime', 'end_DateTime'),
        
        'User': ('start_User', 'end_User'),
        'UsersData': (None, None),
        }
    unmarshal_meth.update(Unmarshaller.unmarshal_meth)
    
    def __init__(self, portal):
        self.portal = portal
        self.fields = {}

        saxutils.DefaultHandler.__init__(self)
        Unmarshaller.__init__(self)

    def encode(self, value):
        root = self.root_stack[-1]
        try:
            charset = root.getCharset()
        except AttributeError:
            charset = root.portal_properties.site_properties.default_charset

        return value.encode(charset)

    def import_data(self, file):
        p = saxexts.make_parser()
        p.setDocumentHandler(self)
        p.parseFile(file)
        self._clear()

    def _clear(self):
        Unmarshaller._clear(self)
        self.root_stack = [self.portal]
        self.fields = {}

    start_DateTime = start_Length = Unmarshaller.um_start_generic
    start_unicode = start_boolean = start_Pickled = Unmarshaller.um_start_generic
    start_attributes = start_PersistentMapping = Unmarshaller.um_start_dictionary
    end_PersistentMapping = Unmarshaller.um_end_dictionary

    def end_DateTime(self, name):
        ds = self.data_stack
        ds[-1] = ''.join(ds[-1])
        ds[-1] = DateTime(ds[-1])
        self.accumulating_chars = 0

    def end_Length(self, name):
        ds = self.data_stack
        ds[-1] = ''.join(ds[-1])
        ds[-1] = Length(int(ds[-1]))
        self.accumulating_chars = 0
    
    def end_Pickled(self, name):
        ds = self.data_stack
        ds[-1] = ''.join(ds[-1])
        data = binascii.a2b_base64(ds[-1])
        ds[-1] = loads(data)
        self.accumulating_chars = 0
    
    def end_unicode(self, name):
        ds = self.data_stack
        ds[-1] = ''.join(ds[-1])
        ds[-1] = unicode(ds[-1])
        self.accumulating_chars = 0

    def end_boolean(self, name):
        ds = self.data_stack
        ds[-1] = ''.join(ds[-1])
        ds[-1] = int(bool(ds[-1]))
        self.accumulating_chars = 0

    def end_attributes(self, name):
        self.um_end_dictionary(name)

        attrs = self.data_stack[-1]

        object = self.root_stack[-1]
        items = attrs.items()
        items.sort()
        for key, val in items:
            if (key == 'hits') and (type(val) == type(1)):
                val = Length(val)
            
            if attrs_mapping.has_key(key):
                setattr(object, attrs_mapping[key], val)
            else:
                setattr(object, key, val)

        if '_owner' not in attrs.keys():
            try:
                del object._owner
            except:
                pass

        del self.data_stack[-1]

    def end_generic_ForumType(self, name):
        root = self.root_stack[-1]
        self.root_stack.pop()

        self.portal.portal_catalog.indexObject(root)

    def start_ForumFolder(self, name, attrs):
        id = str(attrs['id'])
        uid = str(attrs['uid'])

        root = self.root_stack[-1]

        root.invokeFactory('ForumFolder', id, uid=uid)

        ob = getattr(root, id)
        try:
            ob._delObject('members')
            ob._delObject('public')
        except:
            pass

        ob._p_changed = 1
        self.root_stack.append(ob)

    def start_ForumNB(self, name, attrs):
        id = str(attrs['id'])
        uid = str(attrs['uid'])

        root = self.root_stack[-1]

        root.invokeFactory('ForumNB', id, uid=uid)

        ob = getattr(root, id)
        ob._p_changed = 1

        self.root_stack.append(ob)

    def end_ForumNB(self, name):
        root = self.root_stack[-1]

        hits = 0
        for topic in root.contentValues():
            hits += topic.RawHits()

        root.hits.set(hits)
        #print root.RawHits()
        
        root.forum_catalog.refreshCatalog(clear=1)
        self.portal.portal_catalog.indexObject(root)
        self.root_stack.pop()

    def start_ForumTopic(self, name, attrs):
        root = self.root_stack[-1]
        
        id = str(attrs['id'])

        kw = {'title': 'no title',
              'portal_type': str(name),
              'uid': str(attrs['uid'])}

        ob = root._addTopic(id, **kw)
        ob._p_changed = 1
        
        self.root_stack.append(ob)

    def start_ForumContentTopic(self, name, attrs):
        root = self.root_stack[-1]
        
        id = str(attrs['id'])

        kw = {'title': 'no title',
              'portal_type': str(name),
              'uid': str(attrs['uid'])}

        object_path = str(attrs.get('object', ''))
        try:
            object = self.portal.unrestrictedTraverse(object_path)
            if object is not None:
                dtool = self.portal.portal_discussion
                dtool.overrideDiscussionFor(object, str(attrs[u'isDiscussionAllowed']))
                talkback = dtool.getDiscussionFor(object)

                kw['content_uid'] = talkback.UID()
        except:
            pass
        
        try:
            ob = root._addTopic(id, **kw)
        except:
            log('Import error creating Forum Content Topic')
            raise

        self.root_stack.append(ob)


    def start_ForumMessage(self, name, attrs):
        root = self.root_stack[-1]
        topic = root.getTopic()

        ob = topic._addReply(
            id=str(attrs['id']), parent = root, uid=str(attrs['uid']))
        ob._p_changed = 1
        self.root_stack.append(ob)

    ###################
    ## Fields
    ###################
    def start_generic_Field(self, name, attrs):
        self.data_stack.append(attrs)

    def end_generic_Field(self, name):
        ds = self.data_stack
        root = self.root_stack[-1]

        field_id = str(ds[-2]['id'])

        field = root.getField(field_id)
        if field is not None:
            mutator = field.getMutator(root)

            if mutator:
                try:
                    mutator(ds[-1])
                except:
                    log('Import error: %s' % field.getName())
            else:
                field.set(root, ds[-1])
        else:
            log('Import error: field not found "%s"' % ds[-2]['id'])
            
        self.data_stack = ds[:-2]

    start_IntegerField = start_BooleanField = start_StringField = start_generic_Field
    start_TextField = start_ImageField = start_ForumFileField = start_generic_Field
    start_LinesField = start_DateTimeField = start_generic_Field

    end_IntegerField = end_BooleanField = end_StringField = end_generic_Field
    end_LinesField = end_DateTimeField = end_generic_Field
        
    def end_TextField(self, name):
        ds = self.data_stack
        root = self.root_stack[-1]
        attrs = ds[-2]

        field = root.getField(str(attrs['id']))

        mutator = field.getMutator(root)

        if attrs.has_key('mimetype'):
            mimetype = attrs['mimetype']
            if type(mimetype) == type([]):
                try:
                    mimetype = str('/'.join(mimetype))
                except:
                    mimetype = 'text/plain'

            if mimetype == 'text':
                mimetype = 'text/plain'

            mimetype = str(mimetype)

            if mutator:
                mutator(ds[-1], mimetype=mimetype)
            else:
                field.set(root, ds[-1], mimetype=mimetype)
        else:
            if mutator:
                mutator(ds[-1])
            else:
                field.set(root, ds[-1])
                
        self.data_stack = ds[:-2]

    def end_ImageField(self, name):
        ds = self.data_stack
        root = self.root_stack[-1]
        
        attrs = ds[-2]
        data = binascii.a2b_base64(str(ds[-1]))

        field = root.getField('image')

        try:
            field.set(root, data)
        except:
            log_exc('Image import error, message path: %s'%('/'.join(root.getPhysicalPath())))

        attach = field.get(root)
        try:
            attach.filename = self.encode(attrs.get('filename', u''))
            attach.content_type = self.encode(attrs.get(
                'content_type', u'text/x-unknown-content-type'))
        except:
            pass
        self.data_stack = ds[:-2]
    
    def end_ForumFileField(self, name):
        ds = self.data_stack
        root = self.root_stack[-1]

        attrs = ds[-2]
        data = binascii.a2b_base64(str(ds[-1]))

        field = root.getField('file')
        field.set(root, data)

        attach = field.get(root)
        try:
            attach.filename = self.encode(attrs.get('filename', u''))
            attach.content_type = self.encode(attrs.get(
                'content_type', u'text/x-unknown-content-type').encode(charset))
        except:
            pass
        self.data_stack = ds[:-2]

    def um_end_string(self, name):
        ds = self.data_stack
        ds[-1] = self.encode(u''.join(ds[-1]))
        self.accumulating_chars = 0

    def start_User(self, name, attrs):
        member = self.portal.portal_membership.getMemberById(str(attrs['id']))

        self.member = member

    def end_User(self, none):
        props = self.data_stack[-1]

        try:
            self.member.setMemberProperties(props)
        except:
            pass

        self.data_stack.pop()
    

def forum_import(self, import_file):

    if not _checkPermission(ManagePortal, self):
            raise 'Unauthorized'
	    
    portal = getToolByName(self, 'portal_url').getPortalObject()
    ttool = portal.portal_types
    atool = portal.archetype_tool

    importer = ForumImporter(portal)

    try:
        importer.import_data(import_file)
    except:
        traceback.print_exc()
        raise
