# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
"""
$Id: MessageView.py,v 1.14.2.15 2005/05/24 20:07:38 fafhrd91 Exp $
"""

import traceback
from Acquisition import aq_base
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import _checkPermission, getToolByName
from Products.CMFCore.CMFCorePermissions import \
     ModifyPortalContent, ReviewPortalContent, View

from Products.CMFBoard.config import *
from Products.CMFBoard.ForumPermissions import *

from utils import opt_localized_time, get_message_icon

class MessageView:

    security = ClassSecurityInfo()

    def view_getData(self, request):
        tool = request.cmfboard_tool
        g_data = request.cmfboard_data
        f_data = request.f_data
        t_data = request.t_data

        encoding = g_data['encoding']
        topic_url = t_data['url']
        portal = g_data['portal']
        
        data = {}
        
        title = self.title.encode(encoding)
        msg_id = self.id
        msg_uid = self.UID()
        
        data['id'] = msg_id
        data['uid'] = msg_uid
        data['title'] = title

        url = '%s/%s'%(topic_url, msg_id)
        data['url'] = url
        data['body'] = self.CookedBody(encoding)
        data['signature'] = self.getSignature()
        data['review_state'] = g_data['wtool'].getInfoFor(self, 'review_state', '')

        # links
        links = []
        subject = self.Subject() or []
        if subject:
            for lb in g_data['ctool'](Subject=subject):
                try:
                    if url == lb.getURL():
                        continue
                    link = lb.getObject()
                    if link is None:
                        continue
                except:
                    pass

                if link.meta_type != 'Discussion Item':
                    links.append((link.Title(), link.absolute_url()))

        data['links'] = links

        # icon
        data['icon'] = get_message_icon(self, msg_uid, None,
                                        g_data['portal'], self, '&nbsp;&bull;')

        created = self.created()

        if g_data['timezone']:
            created = created.toZone(g_data['timezone'])

	try:
	    data['created'] = opt_localized_time(created, g_data['forumTime'])
        except:
            data['created'] = created

        creator = self.Creator()
        data['creator'] = creator

        anonymous = getattr(self, 'anonymous', 0)
        data['anonymous'] = anonymous

        if not anonymous:
            creator_info = tool.get_user_properties(creator)

            avatar = creator_info.get('avatar', None)
            if avatar:
                creator_info['avatar'] = avatar
                creator_info['avatar_orig'] = avatar

            data['creator_info'] = creator_info
            data['creator_role'] = tool.get_user_role(creator, 'Unavailable')

        # member menu
        if (f_data['show_userinfo'] and not anonymous):
            info = {'id': 'mm_%s'%creator,
                    'menus': [
                {'url': '%s/forum_user_profile?member_id=%s'%(g_data['portal_url'], creator),
                 'title': 'View public profile'},]
                    }
            menus = info['menus']

            if creator_info['fc_emailtome']:
                menus.append({'url':'%s/forum_contact_member_form?member_id=%s'%(topic_url, creator),
                              'title': 'Send email to member'})

        # attachments
        files = []
        images = []

        mtr = getattr(portal, 'mimetypes_registry', None)

        for attach in self.contentValues():
            hasThumbnail = False
            meth = getattr(aq_base(attach), 'hasThumbnail', None)
            if (meth is not None) and callable(meth):
                hasThumbnail = meth()
            
            if hasThumbnail:
                size = attach.getSize()
                if size:
                    images.append(
                        {'url': '%s/%s' % (url, attach.getId()),
                         'size': '%0.1f Kb' % (float(size)/1024),
                         'tag': attach.getThumbnailTag()
                         })
            else:
                size = attach.getSize()

                if size:
                    attach_file = {}
                    attach_file['url'] = '%s/%s' % (url, attach.getId())
                    attach_file['size'] = '%0.1f Kb' % (float(size)/1024)
                    attach_file['filename'] = attach.getId()
                    attach_file['title'] = attach.Title()

                    if mtr is None:
                        attach_file['icon'] = 'unknown.png'
                    else:
                        mti = mtr.lookupExtension(attach.getId())
                        if not mti:
                            mti = mtr.lookup(attach.content_type)
                            if mti:
                                mti = mti[0]

                        if mti:
                            attach_file['icon'] = mti.icon_path
                        else:
                            attach_file['icon'] = 'unknown.png'

                    files.append(attach_file)

        data['files'] = files
        data['images'] = images

        textField = self.getField('text')

        format = self.Format()

        if format in ('text/structured','text/x-rst', ):
            cls = 'messageBody stx'
        else:
            cls = 'messageBody plain'

        if data['images']:
            data['body_class'] = cls + ' messageBodyWithThumbnails'
        else:
            data['body_class'] = cls

        if f_data['show_userinfo'] and not anonymous:
            data['user_info'] = (
                '<!--user_info-->%s<!--end-->'%creator)
        else:
            data['user_info'] = ''

        return data

    def view_getActions(self, request):
        url = self.absolute_url()
        g_data = request.cmfboard_data
        f_data = request.f_data
        t_data = request.t_data
        encoding = g_data['encoding']
        msg_id = self.getId()

        # actions
        actions = {'data': {'msg_id': msg_id, 'url': url}}

        if t_data['replyable']:
            actions['reply'] = '%s/forum_reply_form'%url

        if self.portal_type == 'ForumContentMessage':
            # for content message allow only reply link
            return actions

        if _checkPermission(ModifyPortalContent, self):
            actions['edit'] = '%s/forum_message_edit'%url
            actions['attach'] = '%s/folder_contents'%url

        #if self.id == 'first':
        #    if _checkPermission(DeleteTopic, t_data['topic']):
        #        actions['delete'] = '%s/forum_message_delete_form'%t_data['url']
        if _checkPermission(DeleteReply, self) and (self.id != '0000'):
            actions['delete'] = '%s/forum_message_delete_form'%url

        if f_data['isModerated'] and (msg_id != '0000'):
            #or (tp not in ('ForumMessage', 'ForumContentTopic')):
            transitions = g_data['wtool'].getTransitionsFor(self)
            if transitions:
                actions['review_state'] = g_data['wtool'].getInfoFor(self, 'review_state', '')
                actions['transitions'] = transitions

        # quote link
        if t_data['replyable']:
            try:
                textField = self.getField('text')
                format = self.Format()

                text = textField.get(self, mimetype=format, encoding=encoding)
                text = text.replace('\n', '\\n')
                text = text.replace('\r', '')
                text = text.replace("'", "\\'")
                text = '[quote:%s format="%s"]%s[/quote]\\n' % (
                    self.Creator(), format, text)
            except:
                text = 'Test text'

            try:
                title = self.getField('title').get(self, encoding=encoding)
                q_title = title.replace('\n', '\\n')
                q_title = q_title.replace('\r', '')
                q_title = q_title.replace("'", "\\'")
                q_title =  (title[:3]=='Re:') and q_title or 'Re: '+ q_title
            except:
                q_title = 'Re:'

            q = 'quote(\'%s\',\'%s\',\'%s\');;return false;;'%(text, self.id, q_title)
            actions['quote'] = q

        return actions

    def view_getBodyClass(self):
        pass

    def view_getViewable(self):
        pass

    security.declareProtected(View, 'view_getIcon')
    def view_getIcon(self, forum, portal, default='&nbsp;', cache=None):
        """ return icon """
        msg_uid = self.UID()
        msg_icon = None

        if cache is not None:
            msg_icon = cache.ZCache_get(portal, 'message_icon', keywords={'UID': msg_uid})
            if msg_icon is not None:
                if msg_icon:
                    return msg_icon
                else:
                    return default

        try:
            msg_icon = getattr(self, 'message_icon')
            msg_icon = getattr(forum.fcIcons, msg_icon).tag()
        except:
            msg_icon = ''

        if cache is not None:
            cache.ZCache_set(portal, msg_icon, 'message_icon', keywords={'UID': msg_uid})

        return msg_icon or default

    security.declareProtected(View, 'view')
    def view(self, REQUEST=None):
        """ generate page """
        if REQUEST is None:
            REQUEST = self.REQUEST

        tool = getToolByName(self, CMFBOARD_TOOL)
        g_data = tool.initCMFBoardViews(self)

        f_data = REQUEST.f_data
        t_data = REQUEST.t_data
	topic = self.getTopic()
        
        msg_id = self.getId()

        # find view mode
	view_mode = topic.view_getViewMode(REQUEST)

        # redirect to topic for nested
        if view_mode == 'nested':
            REQUEST.RESPONSE.redirect('%s#%s'%(t_data['url'], msg_id))
            return

        # redirect to topic for flat
        elif view_mode == 'flat':
	    mode = topic.view_getSortMode(REQUEST)

            if mode == 'older_first':
                replies = g_data['topic'].getObjectsList()
            else:
                replies = g_data['topic'].getObjectsList('reverse')

            replies_len = len(replies)
            page_size = f_data['topic_page_size']
            url = t_data['url']

            for idx in range(replies_len):
                if replies[idx] == msg_id:
                    page = int(round(float(idx)/page_size + 0.5)) - 1
                    url = '%s?b_start:int=%s#%s'%(url, page*page_size, msg_id)
                    break

            #print url
            REQUEST.RESPONSE.redirect(url)
            return

        elif view_mode == 'thread':
            # render flat and nested view
            forum = self.getForum()
            
            header, footer = topic.forum_static()

            msg = forum.forum_status_message()
            body = self.forum_message()

            return header + msg + body + footer


    __call__ = view
    index_html = None


InitializeClass(MessageView)
