# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
"""
$Id: TopicView.py,v 1.17.2.9 2005/05/25 04:43:16 fafhrd91 Exp $
"""

import traceback
from Globals import InitializeClass
from Acquisition import aq_inner, aq_parent, aq_base
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import _checkPermission, getToolByName
from Products.CMFPlone.PloneUtilities import translate_wrapper

from Products.CMFBoard.config import *
from Products.CMFBoard.ForumPermissions import *

from utils import get_forums_tree, opt_localized_time, get_message_icon

class TopicView:

    security = ClassSecurityInfo()

    def view_getGlobals(self, request):
        #uid = self.UID()
        tool = request.cmfboard_tool
        g_data = request.cmfboard_data

        data = {}

        #data['uid'] = uid
        data['id'] = self.id
        data['topic'] = self
        data['url'] = self.absolute_url()
        data['title'] = self.Title()
        data['desc'] = self.Description()
        data['replyable'] = _checkPermission(AddTopicReply, self)
        data['rawhits'] = self.RawHits()
        data['sticky_flag'] = self.sticky_flag
        
        review_state = g_data['wtool'].getInfoFor(self, 'review_state', '')
        data['review_state'] = review_state

        if getattr(self, 'hits_counter', 1):
            # hits_counter from forum instance
            data['hits'] = self.RawHits() + 1
        else:
            data['hits'] = None

        if review_state == 'locked':
            if self.portal_type == 'ForumPoll':
                icon = 'fcPoll_locked'
                icon_title = 'Locked poll'
            else:
                icon = 'fcTopic_locked'
                icon_title = 'Locked topic'

            portal = g_data['portal']
            icon_title = translate_wrapper(
                'CMFBoard', icon_title, context=portal, default=icon_title)
            data['icon'] = getattr(portal,'%s.gif'%icon).tag(
                alt=icon_title, title=icon_title)
        else:
            try:
                data['icon'] = str(getattr(self.fcIcons, self.message_icon))
            except:
                data['icon'] = "&bull;"

        request.t_data = data
        return data

    def view_getData(self, request):
        """ """
        g_data = request.cmfboard_data
        t_data = request.t_data
        f_data = request.f_data
        
        data = {}
        view_mode = self.view_getViewMode(request)
        data['view_mode'] = view_mode
        data['hits'] = self.RawHits()

        replies = self.replyCount()
        data['replies'] = replies

        # pages links
        links = []
        if view_mode == 'flat':
            topic_page_size = f_data['topic_page_size']
            if replies >= topic_page_size:
                pages = int(round(float(replies)/topic_page_size + 0.5))
                
                if pages > 8:
                    for idx in (0,1,2):
                        links.append({'id':idx+1, 'url':'%s?%s'%(
                            url, 'b_start:int=%s'%(idx*topic_page_size))})
                        
                    links.append({'id':'...', 'url':''})
                    
                    for idx in range(pages)[-5:]:
                        links.append({'id':idx+1, 'url':'%s?%s'%(
                            url, 'b_start:int=%s'%(idx*topic_page_size))})
                else:
                    for idx in range(pages):
                        links.append({'id':idx+1, 'url':'%s?%s'%(
                            url, 'b_start:int=%s'%(idx*topic_page_size))})

        data['pages_links'] = links

        list = self.getObjectsList()

        first = list[0]
        last = list[-1]

        data['first'] = first
        if first != last:
            data['last'] = last
        else:
            data['last'] = None
        
        return data

    def view_getDataForum(self, request):
        """ """
        g_data = request.cmfboard_data
        f_data = request.f_data
        
        tool = g_data['tool']

        data = {}

        id = self.id

        data['review_state'] = g_data['wtool'].getInfoFor(self, 'review_state', '')
        data['url'] = '%s/%s/'%(f_data['url'], id)
        try:
            data['icon'] = str(getattr(self.fcIcons, getattr(self,'0000').message_icon))
        except:
            data['icon'] = "&bull;"

        data['title'] = self.Title()
        data['sticky_flag'] = self.sticky_flag
        data['desc'] = self.Description()

        view_mode = self.view_getViewMode(request)
        data['view_mode'] = view_mode
        data['hits'] = self.RawHits()

        replies = self.replyCount()
        data['replies'] = replies

        # pages links
        links = []
        topic_page_size = f_data['topic_page_size']
        if replies >= topic_page_size:
            pages = int(round(float(replies)/topic_page_size + 0.5))
                
            if pages > 8:
                for idx in (0,1,2):
                    links.append({'id':idx+1, 'url':'%s?%s'%(
                        url, 'b_start:int=%s'%(idx*topic_page_size))})
                        
                links.append({'id':'...', 'url':''})
                    
                for idx in range(pages)[-5:]:
                    links.append({'id':idx+1, 'url':'%s?%s'%(
                        url, 'b_start:int=%s'%(idx*topic_page_size))})
            else:
                for idx in range(pages):
                    links.append({'id':idx+1, 'url':'%s?%s'%(
                        url, 'b_start:int=%s'%(idx*topic_page_size))})

        data['pages_links'] = links

        list = self.getObjectsList()

        first = list[0]
        last = list[-1]

        data['first'] = first
        if first != last:
            data['last'] = last
        else:
            data['last'] = None

        def get_message_data(self, msgId, t_data):
            # data from first message
            msg = getattr(self, msgId)

            data = {}

            data['url'] = '%s%s/'%(t_data['url'], msgId)
            data['title'] = msg.Title()
            
            creator = msg.Creator()
            data['creator'] = creator

            anonymous = getattr(msg, 'anonymous', 0)
            data['anonymous'] = anonymous

            if not anonymous:
                creator_info = tool.get_user_properties(creator)

                avatar = creator_info.get('avatar', None)
                if avatar:
                    creator_info['avatar'] = avatar
                    creator_info['avatar_orig'] = avatar

                data['creator_info'] = creator_info
                data['creator_role'] = tool.get_user_role(creator, 'Unavailable')

            created = msg.created()

            if g_data['timezone']:
                created = created.toZone(g_data['timezone'])

            try:
                data['created'] = opt_localized_time(created, g_data['forumTime'])
            except:
                data['created'] = created
                
            return data

        data['first'] = get_message_data(self, data['first'], data)
        if data['last']:
            data['last'] = get_message_data(self, data['last'], data)
        
        return data
        
    security.declarePublic('view_getNavLinks')
    def view_getNavLinks(self, request):
        # unique for each member

        g_data = request.cmfboard_data
        t_data = request.t_data
        forum = g_data['forum']

        data = {}

        topic_id = t_data['id']
        encoding = g_data['encoding']
    
        topics_list = forum.getObjectsList()
        
        try:
            nextTopic = forum._getOb(topics_list.getNext(topic_id))
            data['next_topic_url'] = nextTopic.absolute_url()
            data['next_topic_title'] = nextTopic.title.encode(encoding)
        except:
            #traceback.print_exc()
            data['next_topic_url'] = None

        try:
            prevTopic = forum._getOb(topics_list.getPrev(topic_id))
            data['prev_topic_url'] = prevTopic.absolute_url()
            data['prev_topic_title'] = prevTopic.title.encode(encoding)
        except:
            #traceback.print_exc()
            data['prev_topic_url'] = None

        if _checkPermission(AddTopic, forum):
            data['new_topic_url'] = request.f_data['new_topic_url']

        return data

    def view_getViewMode(self, request):
        g_data = request.cmfboard_data
        
        # unique for each member
        view_mode_selectable = getattr(self, 'view_mode_selectable', 1)
        if view_mode_selectable:
            view_mode = request.cookies.get('viewmode', None)
            if view_mode is None:
                view_mode = g_data['member'].getProperty('fc_topic_vm', 'flat')
            if view_mode not in ('flat','nested','thread'):
                view_mode = 'flat'
        else:
            view_mode = getattr(self, 'view_mode', 'flat')

        return view_mode

    def view_getSortMode(self, request):
        g_data = request.cmfboard_data

        sort_mode_selectable = getattr(self, 'sort_mode_selectable', 1)
        if sort_mode_selectable:
            sort_mode = request.cookies.get('sortmode', None)
            if sort_mode is None:
                sort_mode = g_data['member'].getProperty('fc_topic_sm', 'default')
        else:
            sort_mode = g_data['member'].getProperty('fc_topic_sm', 'default')

        if sort_mode not in ('older_first','newer_first'):
            sort_mode = 'older_first'
        return sort_mode

    def view_getMessages(self, request):
        # unique for each member
        sort_mode = self.view_getSortMode(request)
        if sort_mode == 'newer_first':
            return self.getObjectsList('reverse')
        else:
            return self.getObjectsList()

    def view_getMessagesTree(self, request, base=None):
        return self.getObjectsTree(base)

    security.declarePublic('view_getMenu')
    def view_getMenu(self, request):
        g_data = request.cmfboard_data
        f_data = request.f_data
        t_data = request.t_data
        url = t_data['url']
        
        data = {}

        data['forums_tree'] = get_forums_tree(g_data['forum'], g_data['portal'])

        if f_data['allowTopicSubscription'] and not g_data['isAnon']:
            if self.isSubscribed():
                data['unsubscribe_topic'] = 1
            elif g_data['forum'].isSubscribed():
                data['unsubscribe_forum'] = 1
            else:
                data['subscribe_topic'] = 1
                
        # view mode
        view_mode = self.view_getViewMode(request)
        data['view_mode'] = view_mode

        if (view_mode == 'flat') and f_data['sort_mode_selectable']:
            # sort mode
            sort_mode = self.view_getSortMode(request)
            data['sort_mode'] = sort_mode
            data['sort_menu'] = 1
        else:
            data['sort_menu'] = 0

        # actions menu, Edit, Delete
        actions = []
        if _checkPermission(ModifyTopic, self):
            url = t_data['url']
            actions.append({'url':'%s/base_edit'%url, 'title': 'Edit'})
            actions.append({'url': '%s/forum_topic_delete'%url, 'title': 'Delete'})

        data['actions'] = actions

        # topic state
        transitions = g_data['wtool'].getTransitionsFor(self)
        if transitions:
            states = []
            for trans in transitions:
                states.append(
                    {'url':'%s/forum_change_state?workflow_action=%s'%(url,trans['id']),
                     'title':trans['name']})
            data['states'] = [
                g_data['wtool'].getInfoFor(self, 'review_state', ''), states]

        return data

    def view_getThread(self, request):
        g_data = request.cmfboard_data
        t_data = request.t_data

        tree = self.getObjectsTree()

        portal = g_data['portal']
        timezone = g_data['timezone']
        forumTime = g_data['forumTime']
        encoding = g_data['encoding']
        url = t_data['url']

        def generateMessageInfo(self, id):
            info = {}

            msg = getattr(self, id)

            created = msg.created()
            if timezone:
                created = created.toZone(timezone)
            try:
                info['date'] = opt_localized_time(created, forumTime)
            except:
                info['date'] = created

            info['id'] = id
            info['creator'] = msg.Creator()
            info['title'] = msg.title.encode(encoding)
            info['url'] = '%s/%s/'%(url, msg.id)

            # icon
            info['icon'] = get_message_icon(msg, msg.UID(), None,
                                            g_data['portal'], msg, '&nbsp;&bull;')


            return info
        
            # read status
	    #<b tal:omit-tag="python:record['read_status'] or record['isHere']">
	    #  <a tal:omit-tag="record/isHere" tal:content="record/title"
            #tal:attributes="href string:${record/url}#${record/id}"></a>
	    #</b>

            # info
            #<a tal:attributes="href record/file_url"
            #	tal:condition="nocall:record/file|nothing" title="View attached file"
            #	i18n:attributes="title" target="_blank">
            #	<img tal:replace="structure record/file_icon" src="mime_txt.png" /></a>
            #<a tal:attributes="href record/image_url"
            #tal:condition="nocall:record/image|nothing" title="View attached image"
            #i18n:attributes="title" target="_blank">
            #<img tal:replace="structure here/image.png" src="mime_txt.png" /></a>

        def generateTree(self, tree):
            info = generateMessageInfo(self, tree.id)

            subtree = []
            for stree in tree:
                subtree.append(generateTree(self, stree))

            return (info, subtree)

        return (generateTree(self, tree),)

    def view(self, REQUEST=None):
        """ generate page """
        if REQUEST is None:
            REQUEST = self.REQUEST
        
        tool = getToolByName(self, CMFBOARD_TOOL)
        g_data = tool.initCMFBoardViews(self)
        view_mode = self.view_getViewMode(REQUEST)

        # render thread
        if view_mode == 'thread':
            messages = self.getObjectsList()
            if messages:
                id = messages[0]
                return REQUEST.RESPONSE.redirect('%s/%s'%(self.absolute_url(), id))

        # render flat and nested view
        header, footer = self.forum_static()

        msg = self.forum_status_message()
        body = self.forum_topic()

        return header + msg + body + footer

    index_html = None

InitializeClass(TopicView)
