# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nikolay Kim <fafhrd@legco.biz>
"""
$Id: __init__.py,v 1.9.6.1 2004/12/27 18:15:47 fafhrd91 Exp $
"""

from ExtensionClass import Base
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_inner, aq_parent

from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFPlone.PloneUtilities import translate_wrapper

from ui_get_forum_data import \
     get_forum_data, get_forum_menu, get_forums_tree
from ui_get_topic_data import \
     get_topic_data, gen_thread, get_topic_menu
from ui_get_message_data import get_message_data
from ui_get_forum_folder_data import get_forum_folder_data


class UIHelpers(Base):
    """ """
    security = ClassSecurityInfo()

    security.declareProtected(View, 'getForumFolderData')
    def getForumFolderData(self, portal, member, ctool):
        """ """
        dateFormat_trans = translate_wrapper('plone', 'date_format_long', context=portal)
        if not dateFormat_trans:
            properties = portal.portal_properties.site_properties
            dateFormat_trans = properties.localLongTimeFormat

        return get_forum_folder_data(
            aq_inner(self), portal, member, ctool,
            member.getProperty('timezone', ''), dateFormat_trans,
            encoding=portal.portal_properties.site_properties.default_charset)

    security.declareProtected(View, 'getForumData')
    def getForumData(self, here, portal, member, basic=0):
        """ """
        return get_forum_data(here.getForum(), portal, member, basic)

    security.declareProtected(View, 'getForumMenu')
    def getForumMenu(self, here, portal, member):
        """ """
        return get_forum_menu(here.getForum(), portal, member)

    security.declareProtected(View, 'getTopicData')
    def getTopicData(self, message, portal, member):
        """ """
        return get_topic_data(
            message.getForum(), message.getTopic(),
            aq_inner(message), portal, member)

    def getTopicMenu(self, message, portal, member):
        """ """
        return get_topic_menu(
            message.getForum(), message.getTopic(),
            aq_inner(message), portal, member)

    security.declareProtected(View, 'getMessageData')
    def getMessageData(self, message, portal, member, actions=0, jumpto=1):
        """ """
        data = get_message_data(
            message.getForum(), aq_inner(message), portal, member, actions, jumpto=jumpto)

        view_mode = data['view_mode']
        if view_mode == 'thread':
            data['tree'] = self.genThreadTree(message, portal, member)
            data['replies_data'] = [data]
        
        return data

    security.declareProtected(View, 'genThreadTree')
    def genThreadTree(self, message, portal, member, base=None, remove_first=0):
        """ """
        topic = message.getTopic()
        if base is None:
            base = topic

        return gen_thread(
            topic, base, message, portal.forumTime,
            member.getProperty('timezone',''), portal, topic.getCharset(), remove_first)

    security.declareProtected(View, 'genForumsTree')
    def genForumsTree(self, here, portal):
        """ """
        return get_forums_tree(here.getForum(), portal)

InitializeClass(UIHelpers)
