# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nikolay Kim <fafhrd@legco.biz>
"""
$Id: ui_get_forum_data.py,v 1.42.4.2.2.5 2005/05/24 20:07:38 fafhrd91 Exp $
"""
from __future__ import nested_scopes

import traceback
from DateTime import DateTime
from AccessControl import getSecurityManager
from Acquisition import aq_base, aq_inner, aq_parent, aq_acquire
from DocumentTemplate.DT_Var import newline_to_br
from Products.PageTemplates.GlobalTranslationService import getGlobalTranslationService
from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.CMFCore.CMFCorePermissions import View, ReviewPortalContent

from Products.CMFPlone.PloneBatch import Batch
from Products.CMFPlone.PloneUtilities import translate_wrapper

from Products.CMFBoard.ForumPermissions import *
from Products.CMFBoard.debug import log, log_exc
from Products.CMFBoard.config import CACHE_ID, CACHE_ID2

from utils import opt_localized_time, get_message_icon


def get_forum_menu(forum, portal, member, session=None):
    if session is None:
        request = portal.REQUEST
        session = request.get('SESSION', {})

    data = {}

    if _checkPermission(AddTopic, forum):
        data['contact_admin'] = getattr(forum, 'contact_admin', 1)
    else:
        data['contact_admin'] = 0

    # new content creation links
    contents = {}
    if _checkPermission(AddTopic, forum):
        contents['topic'] = 1

    if _checkPermission(AddPoll, forum):
        contents['poll'] = 1

    if _checkPermission(AddAnnouncement, forum):
        contents['announce'] = 1
    data['contents'] = contents
        
    return data


def get_forum_data(forum, portal, member, basic=0):
    request = portal.REQUEST

    ctool = portal.portal_catalog
    mtool = portal.portal_membership
    wtool = portal.portal_workflow
    trans = getGlobalTranslationService()
    properties = portal.portal_properties.site_properties

    timezone = member.getProperty('timezone','')
    encoding = properties.default_charset
        
    #####################################
    ## optimization vars
    #####################################
    dateFormat_trans = translate_wrapper('plone', 'date_format_long', context=portal)
    if not dateFormat_trans:
        dateFormat_trans = properties.localLongTimeFormat

    default_vmode = member.getProperty('fc_topic_vm', 'flat')

    session = request.get('SESSION', {})
    #--------------------------------------------------------

    # cache
    cache = None
    try:
        manager = aq_acquire(portal, CACHE_ID2, containment=1, default=None)
    except:
        manager = None

    if manager is not None:
        cache = manager.ZCacheManager_getCache()

    forum_page_size = getattr(forum, 'forum_page_size', 20)
    topic_page_size = getattr(forum, 'topic_page_size', 20)
    view_last_reply = getattr(forum, 'view_last_reply_preview', 1)
    b_start = int(request.get('b_start', 0))
    
    now = DateTime()
    isAnon = mtool.isAnonymousUser()

    #f_catalog = forum.forum_catalog

    #topics1 = f_catalog(review_state='admin',
    #                    sort_on='modified', sort_order='reverse')

    #topics2 = f_catalog(review_state=['published', 'visible', 'locked', 'disabled',
    #                                  'rejected','waitreview', 'announce'],
    #                    sticky_flag=1, sort_on='modified', sort_order='reverse')
    #topics3 = f_catalog(review_state=['published', 'visible', 'locked', 'disabled',
    #                                  'rejected','waitreview', 'announce'],
    #                    sticky_flag=0, sort_on='modified', sort_order='reverse')
    #                    #sort_limit=forum_page_size+b_start)
    #topics = topics1 + topics2 + topics3
    #len(topics)
    topics = forum.getObjectsList()
    
    batch = Batch(topics, forum_page_size, b_start, orphan=0)
    
    data = {}

    data['b_start'] = b_start
    data['batch'] = batch
    data['page_size'] = topic_page_size
    data['forums_tree'] = get_forums_tree(forum, portal, cache)
    data['view_last_reply'] = view_last_reply

    try:
        data['forum_icon'] = forum.forum_icon.tag()
    except:
        data['forum_icon'] = ''

    # cmfboard propsfo
    data['show_userinfo'] = getattr(forum, 'show_userinfo', 1)
    data['show_wizards'] = getattr(forum, 'show_wizards', 1)
    data['show_message_icons'] = getattr(forum, 'show_message_icons', 1)
    data['allow_anon_names'] = getattr(forum, 'allow_anon_names', '')
    data['allow_signatures'] = getattr(forum, 'allow_signatures', 1)

    # hits counter
    hits_counter = getattr(forum, 'hits_counter', 1)
    data['hits_counter'] = hits_counter

    # parsing flag (emoticons)
    pf = getattr(forum, 'parsing_flag', '')
    if not pf:
        data['show_parsingflag'] = 1
    else:
        data['show_parsingflag'] = 0

    # read status
    read_status = getattr(forum, 'read_status', 1)
    data['read_status'] = read_status
        
    # topic status (new with replies, hot topic, etc)
    topic_hot = getattr(forum, 'topic_hot', 300)
    topic_noreplies = getattr(forum, 'topic_noreplies', 2)
    topic_nr_date = now - topic_noreplies

    # process topics
    topics_data = []
    data['topics_data'] = topics_data

    icon_title_cache = {}
    msg_icons_cache = {}

    if basic:
        return data
    
    for topicId in batch:
        rec = {}
	try:
            topic = forum._getOb(topicId)
    	    #topic = brain.getObject()
            #if topic is None:
            #    continue
	except:
	    continue
	
        topic_uid = topic.UID()
        rec['topic'] = topic
        rec['topicId'] = topicId
        #rec['brain'] = brain

        #url = brain.getURL()
        url = topic.absolute_url()
        rec['url'] = url

        # ------------------- micro date optimization ----------------
        created = topic.created()
        if timezone:
	    created = created.toZone(timezone)
        try:
            rec['created'] = opt_localized_time(created, dateFormat_trans)
        except:
            rec['created'] = created
        # -------------------------------------------------------------
        
        # ------------------- micro view mode optimization ------------
        name = '%s_viewmode'%topic.getId()
        if session.has_key(name):
            view_mode = session[name]
        else:
            view_mode = default_vmode

        if view_mode not in ('flat','nested','thread'):
            view_mode = 'flat'
       
        rec['view_mode'] = view_mode
        # -------------------------------------------------------------

        # actions
        actions = {}
        if _checkPermission(AddTopicReply, topic):
            actions['reply'] = '%s/forum_reply_form'%url
        if _checkPermission(DeleteTopic, topic):
            actions['delete'] = '%s/forum_message_delete_form'%url
        rec['actions'] = actions

        # find last reply
        msg_id = topic.getLastObject()
        #print 'ui_get_forum_data', topicId, msg_id
        msg = getattr(topic, msg_id)

        rec['msg'] = msg

        msg_uid = msg.UID()

        # basic attribues
        review_state = wtool.getInfoFor(topic, 'review_state', '')
        rec['review_state'] = review_state
        rec['sticky_flag'] = topic.sticky_flag

        rec['Title'] = topic.title.encode(encoding)
        rec['topicDesc'] = newline_to_br(topic.description.strip())
        rec['Description'] = newline_to_br(topic.Description())
        try:
            rec['topicIcon'] = getattr(topic, '0000').message_icon
        except:
            rec['topicIcon'] = ''

        # message icon
        msg_icon = get_message_icon(msg, msg_uid, cache, portal, forum, '&nbsp;')
        if msg_icon == '&nbsp;':
            rec['msgIcon'] = get_message_icon(topic, topic_uid, cache, portal, forum)
        else:
            rec['msgIcon'] = msg_icon
            
        # reply count
        replyCount = topic.replyCount()
        rec['replyCount'] = replyCount

        # read status
        if read_status:
            rstatus = forum.get_read_status(topic, request)
        else:
            rstatus = 0

        # hits counter
        if hits_counter:
            hits = topic.RawHits()
            rec['hits'] = hits
        else:
            hits = 0
            rec['hits'] = 0

        # topic status
        if review_state == 'announce':
            icon = 'announce'
            icon_title = 'Announcement'
        elif review_state == 'locked':
            if topic.portal_type == 'ForumPoll':
                icon = 'fcPoll_locked'
                icon_title = 'Locked poll'
            else:
                icon = 'fcTopic_locked'
                icon_title = 'Locked topic'
        else:
            s2 = 'Normal'
            s3 = ''
            if topic.portal_type == 'ForumPoll':
                icon = 'fcPoll_icon'
                s1 = 'poll'
            else:
                icon = 'fcTopic_icon'
                s1 = 'topic'

            if hits_counter:
                if not isAnon and read_status:
                    if hits >= topic_hot:
                        s2 = 'Hot'
                        if rstatus:
                            icon = '%s_hot_noans'%icon
                            s3 = 'without new replies'
                        else:
                            icon = '%s_hot'%icon
                            s3 = 'with new replies'
                    else:
                        if rstatus:
                            icon = '%s_noans'%icon
                            s3 = 'without new replies'
                else:
                    if hits >= topic_hot:
                        if topic.modified() > topic_nr_date:
                            s2 = 'Hot'
                            if replyCount:
                                icon = '%s_hot'%icon
                                s3 = 'with new replies'
                            else:
                                icon = '%s_hot_noans'%icon
                                s3 = 'without new replies'
                        else:
                            icon = '%s_hot_noans'%icon
                            s2 = 'Hot'
                            s3 = 'without new replies'

                    else:
                        if (topic.modified() < topic_nr_date) or not replyCount:
                            icon = '%s_noans'%icon
                            s3 = 'without new replies'
            icon_title = '%s %s %s'%(s2, s1, s3)

        try:
            if not icon_title_cache.has_key(icon_title):
                icon_title_cache[icon_title] = trans.translate(
                    'CMFBoard', icon_title, context=portal, default=icon_title)
    
            icon_title = icon_title_cache[icon_title]
            rec['icon'] = getattr(portal,'%s.gif'%icon).tag(alt=icon_title,title=icon_title)
        except:
            rec['icon'] = ''

        # pages links
        links = []
        if view_mode == 'flat':
            if replyCount >= topic_page_size:
                pages = int(round(float(replyCount)/topic_page_size + 0.5))
                
                if pages > 8:
                    for idx in (0,1,2):
                        links.append({'id':idx+1, 'url':'%s?%s'%(
                            url, 'b_start:int=%s'%(idx*topic_page_size))})
                        
                    links.append({'id':'...', 'url':''})
                    
                    for idx in range(pages)[-5:]:
                        links.append({'id':idx+1, 'url':'%s?%s'%(
                            url, 'b_start:int=%s'%(idx*topic_page_size))})
                else:
                    for idx in range(pages):
                        links.append({'id':idx+1, 'url':'%s?%s'%(
                            url, 'b_start:int=%s'%(idx*topic_page_size))})

        rec['pages_links'] = links

        # originator
        creator = topic.Creator()
        rec['creator'] = creator
        if getattr(topic, 'anonymous', 0):
            rec['anonymous'] = 1
        else:
            rec['anonymous'] = 0
            rec['creator_info'] = forum.get_user_properties(creator, portal, mtool)

        # link to last message
        msg_id = msg.getId()
        if view_mode in ('thread', 'full_thread'):
	    link = '%s#%s'%(msg.absolute_url(), msg_id)
        elif view_mode == 'nested':
            link = '%s#%s'%(url, msg_id)
        else:
            link = msg.absolute_url()
        rec['last_reply_link'] = link

        # last post user info
        if msg is topic:
            rec['last_reply_topic'] = 1

            rec['last_user'] = rec['creator']
            rec['last_user_isanon'] = rec['anonymous']
	    rec['last_reply_created'] = rec['created']
            if rec.has_key('creator_info'):
                rec['last_user_info'] = rec['creator_info']
        else:
            rec['last_reply_topic'] = 0
            rec['last_reply_title'] = msg.Title()

            # ------------------- date optimization ---------------------
            tm = msg.created()
            if timezone:
		tm = tm.toZone(timezone)
            try:
        	rec['last_reply_created'] = opt_localized_time(tm, dateFormat_trans)
            except:
        	rec['last_reply_created'] = tm
            # -------------------------------------------------------------
            
            creator = msg.Creator()
            rec['last_user'] = creator

            anonymous = getattr(msg, 'anonymous', 0)
            rec['last_user_isanon'] = anonymous
            if not anonymous:
                rec['last_user_info'] = forum.get_user_properties(creator,portal,mtool)

        topics_data.append(rec)


    data.update(get_forum_menu(forum, portal, member, session))

    return data


def get_forums_tree(self, portal, cache=None):
    found = 0
    parent = self
    while not found:
        try:
            if parent.portal_type != 'ForumFolder':
                parent = aq_parent(parent)
            else:
                found = 1
        except:
            return []

    parent_uid = parent.UID()

    if cache is None:
        try:
            manager = aq_acquire(portal, CACHE_ID2, containment=1, default=None)
        except:
            manager = None

        if manager is not None:
            cache = manager.ZCacheManager_getCache()

    if cache is not None:
        userid = str(getSecurityManager().getUser())
        data = cache.ZCache_get(portal, 'forums_tree', keywords={
            'UID': parent_uid, 'userid': userid})
        if data is not None:
            return data

    url = portal.absolute_url()
    folder_icon = '%s/%s'%(url, 'fcFolder_icon.gif')
    forum_icon = '%s/%s'%(url, 'fcForum_icon.gif')

    if parent.aq_parent.portal_type == 'ForumFolder':
        parent = aq_parent(aq_inner(parent))
   
    def add_rec(res, folder, mark1='', mark2='--'):
        if _checkPermission(View, folder):
            res.append({'title':folder.title_or_id(),
                        'disable':0, 'url':folder.absolute_url(),
                        'icon': folder_icon,
                        'mark': mark1})

            for forum in folder.objectValues('ForumNB'):
                if _checkPermission(View, forum):
                    res.append(
                        {'title':forum.title_or_id(),
                         'disable':1, 'url': forum.absolute_url(),
                         'icon': forum_icon,
                         'mark': mark2})

            for folder in folder.objectValues('ForumFolder'):
                add_rec(res, folder, mark1+'-', mark2+'--')

    res = []
    add_rec(res, parent)

    if cache is not None:
        cache.ZCache_set(portal, res, 'forums_tree',
                         keywords={'UID': parent_uid, 'userid': userid})

    return res
