# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nikolay Kim <fafhrd@legco.biz> 
"""
$Id: ui_get_forum_folder_data.py,v 1.4.6.3 2005/05/24 20:14:24 fafhrd91 Exp $
"""

from Products.CMFCore.utils import _checkPermission
from Products.CMFCore.CMFCorePermissions import View

from utils import opt_localized_time


def get_forum_folder_data(folder, portal, member, ctool, timezone, forumTime,
                          encoding='utf-8'):

    url = folder.absolute_url()
    data = {'folder': folder,
            'url': url,
            }

    try:
        data['folder_icon'] = folder.folder_icon.tag()
    except:
        pass

    title = folder.title
    if type(title) == type(u''):
	data['title'] = title.encode(encoding)
    else:
	data['title'] = title

    try:
        data['description'] = folder.description.getRaw(encoding=encoding)
    except:
	desc = folder.description
	if type(desc) == type(''):
	    data['description'] = desc
	else:
    	    data['description'] = folder.description.encode(encoding)

    forums = []
    no_hits_all = 0

    for forum_id in folder.objectIds():
        forum = getattr(folder, forum_id)
        if forum.meta_type != 'ForumNB':
            continue
            
        if not _checkPermission(View, forum):
            continue

        last_reply_id = forum.getLastObject()
        if last_reply_id is not None:

            last_reply = getattr(forum, last_reply_id)
            
            posts = len(forum.getObjectsList())

            created = last_reply.created()
            if timezone:
                created = created.toZone(timezone)
            try:
                created = opt_localized_time(created, forumTime)
            except:
                created = created

            forum_data = { 'posts': posts,
                           'replies': forum.replies(),
                           'last_reply': last_reply,
                           'last_user': last_reply.Creator(),
                           'last_user_isanon': getattr(last_reply, 'anonymous', 0),
                           'last_mtime': created, }
        else:
            forum_data = { 'posts': 0, 'replies': 0,
                           'last_reply': None, 'last_user': None,
                           'last_user_isanon': 0, 'last_mtime': None, }

        forum_data['forum'] = forum
        forum_data['title'] = forum.title.encode(encoding)

        try:
            forum_data['icon'] = forum.forum_icon.tag()
        except:
            forum_data['icon'] = '&nbsp;'

        try:
            forum_data['description'] = forum.description.getRaw(encoding=encoding)
        except:
            forum_data['description'] = forum.description.encode(encoding)
            
        forum_data['url'] = '%s/%s'%(url, forum.getId())
        forum_data['last_user_info'] = forum.get_user_properties(
            forum_data['last_user'], portal)

        hits_counter = getattr(forum, 'hits_counter', 1)
        forum_data['hits_counter'] = hits_counter

        if hits_counter:
            forum_data['hits'] = forum.RawHits()
            no_hits_all = 1
        else:
            forum_data['hits'] = '&nbsp;'
            
        forums.append(forum_data)

    data['forums'] = forums
    data['no_hits_all'] = no_hits_all

    folders = []
    for folder_id in folder.objectIds():
        sub_folder = getattr(folder, folder_id)
        if sub_folder.meta_type != 'ForumFolder':
            continue

        if _checkPermission(View, sub_folder):
            folders.append(
                get_forum_folder_data(sub_folder, portal, member,
                                      ctool, timezone, forumTime, encoding))

    data['folders'] = folders
    return data
