# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nikolay Kim <fafhrd@legco.biz>
"""
$Id: ui_get_message_data.py,v 1.28.4.2.2.1 2004/12/27 18:15:48 fafhrd91 Exp $
"""

from Acquisition import aq_base, aq_inner
from Products.PageTemplates.GlobalTranslationService \
     import getGlobalTranslationService
from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.CMFPlone.PloneBatch import Batch
from Products.CMFPlone.PloneUtilities import translate_wrapper
from Products.CMFCore.CMFCorePermissions import ReviewPortalContent
from Products.CMFBoard.ForumPermissions import *

from ui_get_forum_data import get_forums_tree
from ui_get_topic_data import gen_thread, get_message_attache_info
from utils import opt_localized_time, get_message_icon

def get_message_data(forum, message, portal, member, isActions=0, base=None, jumpto=1):
    topic = message.getTopic()
    request = topic.REQUEST

    wtool = portal.portal_workflow
    mtool = portal.portal_membership
    trans = getGlobalTranslationService()
    properties = portal.portal_properties.site_properties

    timezone = member.getProperty('timezone','')
    encoding = properties.default_charset
        
    #####################################
    ## optimization vars
    #####################################
    dateFormat_trans = translate_wrapper('plone', 'date_format_long', context=portal)
    if not dateFormat_trans:
        dateFormat_trans = properties.localLongTimeFormat

    cache = None
    try:
        manager = aq_acquire(portal, CACHE_ID2, containment=1, default=None)
    except:
        manager = None
    if manager is not None:
        cache = manager.ZCacheManager_getCache()

    session = request.get('SESSION', {})

    data = {}

    topic_id = topic.getId()
    topic_uid = topic.UID()
    topic_url = topic.absolute_url()
    data['topic'] = topic
    data['topic_id'] = topic.getId()
    data['topic_url'] = topic_url
    data['page_size'] = getattr(forum, 'topic_page_size', 20)

    isModerated = forum.moderated

    if jumpto:
        data['forums_tree'] = get_forums_tree(forum, portal)

    replyable = _checkPermission(AddTopicReply, topic)
    data['replyable'] = replyable

    quickreply =  forum.getField('quickreply_form').get(forum)
    data['quickreply'] =  quickreply

    # cmfboard propsfo
    show_userinfo = getattr(forum, 'show_userinfo', 1)
    data['show_userinfo'] = show_userinfo
    data['show_wizards'] = getattr(forum, 'show_wizards', 1)
    data['show_message_icons'] = getattr(forum, 'show_message_icons', 1)
    data['allow_anon_names'] = getattr(forum, 'allow_anon_names', 0)
    data['allow_signatures'] = getattr(forum, 'allow_signatures', 1)

    # view mode
    view_mode_selectable = getattr(forum, 'view_mode_selectable', 1)
    data['view_mode_selectable'] = view_mode_selectable
    if view_mode_selectable:
        name = '%s_viewmode'%topic_uid
        view_mode = session.get(name, member.getProperty('fc_topic_vm', 'flat'))
        if view_mode not in ('flat','nested','thread'):
            view_mode = 'flat'
        data['view_mode'] = view_mode
    else:
        view_mode = getattr(forum, 'view_mode', 'flat')
        data['view_mode'] = view_mode

    # parsing flag (emoticons)
    pf = getattr(forum, 'parsing_flag', 1)
    if not pf:
        data['show_parsingflag'] = 1
    else:
        data['show_parsingflag'] = 0

    # sort mode
    name = '%s_sortmode'%topic_uid
    sort_mode = session.get(name, member.getProperty('fc_topic_sm', 'default'))
    if sort_mode not in ('older_first','newer_first'):
        sort_mode = 'older_first'
    data['sort_mode'] = sort_mode

    # show userinfo
    data['show_userinfo'] = show_userinfo

    # hits counter
    hits_counter = getattr(forum, 'hits_counter', 1)
    data['hits_counter'] = hits_counter

    # basic attribues
    msg_id = message.getId()
    data['msg'] = message
    data['msg_id'] = msg_id

    msg_uid = message.UID()
    data['msg_uid'] = msg_uid
    
    url = message.absolute_url()
    data['url'] = url
    
    title = message.title.encode(encoding)
    data['Title'] = title
    
    data['Body'] = message.CookedBody(encoding)
    data['signature'] = message.getSignature()

    created = message.created()
    if timezone:
        created = created.toZone(timezone)
    try:
        data['created'] = opt_localized_time(created, dateFormat_trans)
    except:
        data['created'] = created
    
    # icon
    data['icon'] = get_message_icon(message, msg_uid, cache, portal, forum, '&nbsp;&bull;')

    # links
    links = []
    subject = message.Subject() or []
    if subject:
        for lb in portal.portal_catalog(Subject=subject):
            try:
                if url == lb.getURL():
                    continue
                link = lb.getObject()
                if link is None:
                    continue
            except:
                pass

            if link.meta_type != 'Discussion Item':
                links.append((link.Title(), link.absolute_url()))

    data['links'] = links

    # review state
    review_state = wtool.getInfoFor(message, 'review_state', '')
    data['review_state'] = review_state

    # Creator
    creator = message.Creator()
    data['creator'] = creator

    isAnonymousCreator = getattr(message, '_anonymous', 0)
    data['anonymous'] = isAnonymousCreator

    if show_userinfo:
        creator_info = forum.get_user_properties(creator, portal, mtool)

        # resize avatar
        avatar = creator_info.get('avatar', None)
        if avatar:
            creator_info['avatar_orig'] = avatar
            #avatar_title = trans.translate(
            #    'CMFBoard', 'label_avatar', context=forum,
            #    default="Member's photo")
            creator_info['avatar'] = avatar

        data['creator_info'] = creator_info
        data['creator_role'] = topic.get_user_role(creator,'Unavailable')

    # member menu
    if not isAnonymousCreator:
        info = {'id': 'mm_%s'%creator,
                'menus': [
            {'url': '%s/forum_user_profile?member_id=%s'%(topic_url, creator),
             'title': 'View public profile'},]
                }
        menus = info['menus']
        if ((getattr(portal, 'portal_messagetool', None) is not None) and
            mtool.getHomeFolder(creator)):
            menus.append({'url':'%s/forum_send_message_form?member_id=%s'%(url, creator),
                          'title':'Send a private message'},)
        if show_userinfo and creator_info['fc_emailtome']:
            menus.append({'url':'%s/forum_contact_member_form?member_id=%s'%(url, creator),
                          'title': 'Send email to member'})
        data['creator_menus'] = {creator: info}
    else:
        data['creator_menus'] = {}


    # actions
    actions = {}
    if isActions:
        if replyable:
            actions['reply'] = '%s/forum_reply_form'%url

        if (message.portal_type == 'ForumMessage'):
            if _checkPermission(ModifyPortalContent, message):
                actions['edit'] = '%s/forum_message_edit'%url
            
            if _checkPermission(DeleteReply, message):
                actions['delete'] = '%s/forum_message_delete_form'%url

        else:
            if _checkPermission(ModifyTopic, message):
                actions['edit'] = '%s/forum_message_edit'%url

            if _checkPermission(DeleteTopic, message):
                actions['delete'] = '%s/forum_message_delete_form'%url

        if isModerated:
            transitions = wtool.getTransitionsFor(message)
            if transitions:
                actions['review_state'] = review_state
                actions['transitions'] = transitions

    # quote link
    if replyable:
        q = None
        if cache is not None:
            q = cache.ZCache_get(portal, 'quote', keywords={'UID': msg_uid})

        if q:
            actions['quote'] = q
        else:
            textField = message.getField('text')
            format = textField.getContentType(message)
            text = textField.get(message, mimetype=format)
            text = text.replace('\n', '\\n')
            text = text.replace('\r', '')
            text = text.replace("'", "\\'")
            text = '[quote:%s format="%s"]%s[/quote]\\n' % (
                creator, message.getContentType('text'), text)

            q_title = title.replace('\n', '\\n')
            q_title = q_title.replace('\r', '\\r')
            q_title = q_title.replace("'", "\\'")

            q = 'quote(\'%s\',\'%s\',\'%s\');;return false;;'%(
                text, msg_uid, (q_title[:3]=='Re:') and q_title or 'Re: '+q_title)
            actions['quote'] = q

            if cache is not None:
                cache.ZCache_set(portal, q, 'quote', keywords={'UID': msg_uid})

    data['actions'] = actions
    
    data.update(get_message_attache_info(message, msg_uid, portal, forum, cache))

    return data
