# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nikolay Kim <fafhrd@legco.biz>
"""
$Id: ui_get_topic_data.py,v 1.43.4.6.2.3 2005/03/03 12:45:20 fafhrd91 Exp $
"""
from __future__ import nested_scopes

import traceback
from Acquisition import aq_base, aq_inner, aq_acquire
from AccessControl import getSecurityManager
from Products.PageTemplates.GlobalTranslationService \
     import getGlobalTranslationService
from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.CMFPlone.PloneBatch import Batch
from Products.CMFPlone.PloneUtilities import translate_wrapper
from Products.CMFCore.CMFCorePermissions import ReviewPortalContent
from Products.CMFBoard.ForumPermissions import *
from Products.CMFBoard.config import CACHE_ID2

from utils import opt_localized_time, get_message_icon
from ui_get_forum_data import get_forums_tree

def get_topic_menu(forum, topic, message, portal, member, request=None):
    if request is None:
        request = portal.REQUEST
	
    session = request.get('SESSION', {})

    topic_uid = topic.UID()

    data = {}

    data['forums_tree'] = get_forums_tree(forum, portal)

    # sort mode
    name = '%s_sortmode'%topic_uid
    sort_mode = session.get(name, member.getProperty('fc_topic_sm', 'default'))
    if sort_mode not in ('older_first','newer_first'):
        sort_mode = 'older_first'
    data['sort_mode'] = sort_mode

    # view mode
    view_mode_selectable = getattr(forum, 'view_mode_selectable', 1)
    data['view_mode_selectable'] = view_mode_selectable
    if view_mode_selectable:
        name = '%s_viewmode'%topic_uid
        view_mode = session.get(name, member.getProperty('fc_topic_vm', 'flat'))
        if view_mode not in ('flat', 'nested', 'thread'):
            view_mode = 'flat'
        data['view_mode'] = view_mode
    else:
        view_mode = getattr(forum, 'view_mode', 'flat')
        data['view_mode'] = view_mode

    return data


def get_topic_data(forum, topic, message, portal, member):
    request = topic.REQUEST

    ctool = portal.portal_catalog
    mtool = portal.portal_membership
    wtool = portal.portal_workflow
    hasMsgtool = hasattr(portal, 'portal_messagetool')

    timezone = member.getProperty('timezone','')

    encoding = portal.portal_properties.site_properties.default_charset

    #####################################
    ## optimization vars
    #####################################
    forumTime = translate_wrapper('plone', 'date_format_long', context=portal)
    if not forumTime:
        properties = portal.portal_properties.site_properties
        forumTime = properties.localLongTimeFormat

    #default_vmode = member.getProperty('fc_topic_vm', 'flat')
    #session = request.get('SESSION', {})
    #--------------------------------------------------------
    
    data = {}
    topic_id = topic.getId()
    topic_uid = topic.UID()

    # load menu
    data.update(get_topic_menu(forum, topic, message, portal, member, request))

    # prev thread
    # need add read status
    topic_mod = topic.modified()
    
    topics = forum.getObjectsList()
    try:
        prev_id = topics.getPrev(topic_id)
        if prev_id:
            ob = forum[prev_id]
            data['prev_topic_url'] = ob.absolute_url()
            data['prev_topic_title'] = ob.title.encode(encoding)
    except:
        pass
    try:
        next_id = topics.getNext(topic_id)
        if next_id:
            ob = forum[next_id]
            data['next_topic_url'] = ob.absolute_url()
            data['next_topic_title'] = ob.title.encode(encoding)
    except:
        pass

    forum_url = forum.absolute_url()
    # New topic link
    if _checkPermission(AddTopic, forum):
        data['new_topic_url'] = '%s/forum_add_topic_form'%forum_url

    # parsing flag (emoticons)
    pf = getattr(forum, 'parsing_flag', 1)
    if not pf:
        data['show_parsingflag'] = 1
    else:
        data['show_parsingflag'] = 0

    sort_mode = data['sort_mode']
    view_mode = data['view_mode']

    data['topic'] = topic

    topic_id = topic.getId()
    data['topic_id'] = topic_id

    topic_url = '%s/%s'%(forum_url, topic_id)
    data['topic_url'] = topic_url

    isModerated = forum.moderated

    # replyable
    replyable = _checkPermission(AddTopicReply, topic)
    data['replyable'] = replyable

    # show userinfo
    show_userinfo = getattr(forum, 'show_userinfo', 1)
    data['show_userinfo'] = show_userinfo

    # cmfboard properties
    data['show_wizards'] = getattr(forum, 'show_wizards', 1)
    data['show_message_icons'] = getattr(forum, 'show_message_icons', 1)
    data['allow_anon_names'] = getattr(forum, 'allow_anon_names', '')
    data['allow_signatures'] = getattr(forum, 'allow_signatures', 1)

    # quickreply
    quickreply =  getattr(forum, 'quickreply_form', '')
    data['quickreply'] = quickreply

    # hits counter
    hits_counter = getattr(forum, 'hits_counter', 1)
    data['hits_counter'] = hits_counter

    if hits_counter:
        # change hits counter
        data['hits'] = topic.RawHits()

    # creator_menus
    data['creator_menus'] = {}

    # local vars
    cr_roles = {}

    if view_mode == 'flat':
        data['replies_data'] = gen_flat(
            data, forum, topic, [], forumTime, timezone, cr_roles, replyable,
            isModerated, topic_id, quickreply,
            sort_mode, wtool, ctool, mtool, hasMsgtool,
            request, show_userinfo, portal, encoding)
        
    elif view_mode ==  'thread':
        data['replies_data'] = gen_flat(
            data, forum, topic, [message], forumTime, timezone, cr_roles, replyable,
            isModerated, topic_id, quickreply,
            sort_mode, wtool, ctool, mtool, hasMsgtool,
            request, show_userinfo, portal, encoding, 0)
        data['replies_data'][0]['tree'] = gen_thread(
            topic, topic, topic, forumTime, timezone, portal, encoding)

    elif view_mode == 'nested':
        data['record'] = gen_nested(
            data, forum, topic, [message], forumTime, timezone, cr_roles,
            replyable, isModerated, topic_id, quickreply,
            sort_mode, wtool, ctool, mtool, hasMsgtool,
            request, show_userinfo, portal, encoding)

    return data


def gen_flat(data, forum, topic, replies, forumTime, timezone, cr_roles,
             replyable, isModerated, topic_id, quickreply, sort_mode, 
             wtool, ctool, mtool, hasMsgtool,
             request, show_userinfo, portal, encoding, isBrain=1):

    #trans = getGlobalTranslationService()

    cache = None
    try:
        manager = aq_acquire(portal, CACHE_ID2, containment=1, default=None)
    except:
        manager = None
    if manager is not None:
        cache = manager.ZCacheManager_getCache()
        
    userid = str(getSecurityManager().getUser())

    topic_page_size = getattr(forum, 'topic_page_size', 20)

    if not replies:
        #b_start = int(request.get('b_start', 0))

        # replies
        replies = []
        if sort_mode == 'newer_first':
            replies = ctool(path='/'.join(topic.getPhysicalPath()),
                            sort_on='created', sort_order='reverse')
            #sort_limit = b_start*topic_page_size+topic_page_size)
        else:
            replies = ctool(path='/'.join(topic.getPhysicalPath()), sort_on='created')
            #sort_limit = b_start*topic_page_size+topic_page_size)

        b_start = int(request.get('b_start', 0))
        batch = Batch(replies, topic_page_size, b_start, orphan=0)

        replies = batch
        data['b_start'] = b_start
        data['batch'] = batch
        data['page_size'] = topic_page_size

    # creator menus
    creator_menus = data['creator_menus']
    topic_url = data['topic_url']

    # process topics
    replies_data = []
    for brain in replies:
        rec = {}
        if isBrain:
            reply = brain.getObject()
            title = brain.Title
        else:
            reply = brain
            title = reply.title.encode(encoding)

        # sometimes zcatalog has broken links :(
        if reply is None:
            continue
            
        msg_id = reply.getId()
        rec['msg'] = reply
        rec['msg_id'] = msg_id

        msg_uid = reply.UID()
        rec['msg_uid'] = msg_uid
        rec['brain'] = brain

        url = reply.absolute_url()
        rec['url'] = url

        # basic attribues
        rec['Title'] = title
        rec['Body'] = reply.CookedBody(encoding)
        rec['signature'] = reply.getSignature()

        # icon
        rec['icon'] = get_message_icon(reply, msg_uid, cache, portal, forum, '&nbsp;&bull;')

        # date
        # ------------------- date optimization ----------------
        if isBrain:
            created = brain.created
        else:
            created = brain.created()

        if timezone:
            created = created.toZone(timezone)

	try:
	    rec['created'] = opt_localized_time(created, forumTime)
        except:
            rec['created'] = created
        # -------------------------------------------------------------

        if isBrain:
            review_state = brain.review_state
        else:
            review_state = wtool.getInfoFor(reply, 'review_state', '')
            
        rec['review_state'] = review_state

        # links
        links = []
        subject = reply.Subject() or []
        if subject:
            for lb in portal.portal_catalog(Subject=subject):
                try:
		    #print url, lb.getURL()
                    if url == lb.getURL():
                        continue
                    link = lb.getObject()
                    if link is None:
                        continue
                except:
                    pass

                if link.meta_type != 'Discussion Item':
                    links.append((link.Title(), link.absolute_url()))

        rec['links'] = links
            
        # Creator
        if isBrain:
            creator = brain.Creator
        else:
            creator = brain.Creator()

        rec['creator'] = creator
        
        anonymous = getattr(reply, '_anonymous', 0)
        rec['anonymous'] = anonymous

        if show_userinfo and not anonymous:
            creator_info = forum.get_user_properties(creator, portal)

            avatar = creator_info.get('avatar', None)
            if avatar:
                creator_info['avatar_orig'] = avatar
                #avatar_title = trans.translate(
                #    'CMFBoard', 'label_avatar', context=forum,
                #    default="Member's photo")
                creator_info['avatar'] = avatar
                
            rec['creator_info'] = creator_info

            if cr_roles.has_key(creator):
                rec['creator_role'] = cr_roles[creator]
            else:
                role = topic.get_user_role(creator,'Unavailable')
                rec['creator_role'] = role
                cr_roles[creator] = role

        # member menu
        if (show_userinfo and not creator_menus.has_key(creator) and not anonymous):
            info = {'id': 'mm_%s'%creator,
                    'menus': [
                {'url': '%s/forum_user_profile?member_id=%s'%(topic_url, creator),
                 'title': 'View public profile'},]
                }
            menus = info['menus']
            if hasMsgtool and mtool.getHomeFolder(creator):
                menus.append({'url':'%s/forum_send_message_form?member_id=%s'%(url, creator),
                              'title':'Send a private message'},)

            if creator_info['fc_emailtome']:
                menus.append({'url':'%s/forum_contact_member_form?member_id=%s'%(url, creator),
                              'title': 'Send email to member'})
            creator_menus[creator] = info

        # actions
        actions = None

        if cache is not None:
            actions = cache.ZCache_get(portal, 'message_actions', keywords={
                'UID': msg_uid, 'userid': userid})
                
        if actions is None:
            actions = {}
            
            if replyable:
                actions['reply'] = '%s/forum_reply_form'%url

            tp = reply.portal_type
            if (tp == 'ForumMessage'):
                if _checkPermission(ModifyPortalContent, reply):
                    actions['edit'] = '%s/forum_message_edit'%url

                if _checkPermission(DeleteReply, reply):
                    actions['delete'] = '%s/forum_message_delete_form'%url
            else:
                if _checkPermission(ModifyTopic, reply):
                    if tp == 'ForumPoll':
                        actions['edit'] = '%s/forum_poll_edit'%url
                    elif tp == 'ForumContentTopic':
                        pass
                    else:
                        actions['edit'] = '%s/forum_message_edit'%url

                if _checkPermission(DeleteTopic, reply):
                    actions['delete'] = '%s/forum_message_delete_form'%url

            if isModerated or (tp not in ('ForumMessage', 'ForumContentTopic')):
                transitions = wtool.getTransitionsFor(reply)
                if transitions:
                    actions['review_state'] = wtool.getInfoFor(reply,'review_state','')
                    actions['transitions'] = transitions

            if cache is not None:
                cache.ZCache_set(portal, actions, 'message_actions',
                                 keywords={'UID': msg_uid, 'userid': userid})

        rec['actions'] = actions

        # quote link
        if replyable and quickreply:
            q = None
            if cache is not None:
                q = cache.ZCache_get(portal, 'quote', keywords={'UID': msg_uid})

            if q:
                actions['quote'] = q
            else:
                textField = reply.getField('text')
                format = textField.getContentType(reply)
                text = textField.get(reply, mimetype=format, encoding=encoding)
                text = text.replace('\n', '\\n')
                text = text.replace('\r', '')
                text = text.replace("'", "\\'")
                text = '[quote:%s format="%s"]%s[/quote]\\n' % (creator, format, text)

                q_title = title.replace('\n', '\\n')
                q_title = q_title.replace('\r', '\\r')
                q_title = q_title.replace("'", "\\'")
                q = 'quote(\'%s\',\'%s\',\'%s\');;return false;;'%(
                    text, reply.UID(), (q_title[:3]=='Re:') and
                    q_title or 'Re: '+q_title)
                actions['quote'] = q

                if cache is not None:
                    cache.ZCache_set(portal, q, 'quote', keywords={'UID': msg_uid})

                    
        rec.update(get_message_attache_info(reply, msg_uid, portal, forum, cache))

        replies_data.append(rec)
    
    return replies_data


def gen_thread(topic, base, message, forumTime, timezone, portal, encoding, remove_first=0):

    mesage_id = message.getId()
    topic_url = topic.absolute_url()

    cache = None
    try:
        manager = aq_acquire(portal, CACHE_ID2, containment=1, default=None)
    except:
        manager = None
    if manager is not None:
        cache = manager.ZCacheManager_getCache()

    request = portal.REQUEST

    def gen_level(msg, base_id):
        rec = {}

        msg_id = msg.getId()
        msg_uid = msg.UID()
        url = '%s/%s'%(topic_url, msg_id)

        rec['id'] = msg_id
        rec['url'] = url
        rec['read_status'] = topic.get_read_status(msg, request) and (msg_id != mesage_id)

        rec['title'] = msg.title.encode(encoding)
        rec['isHere'] = msg_id == mesage_id

        rec.update(get_message_attache_info(msg, msg_uid, portal, topic, cache))

        rec['icon'] = get_message_icon(msg, msg_uid, cache, portal, topic, '&nbsp;&bull;')

        creator = msg.Creator()
        rec['creator'] = creator

        # ------------------- micro date optimization ----------------
        created = msg.created()
	if timezone:
	    created = created.toZone(timezone)
        try:
            rec['date'] = opt_localized_time(created, forumTime)
        except:
            rec['date'] = created
        # -------------------------------------------------------------        

        subtree = []
        for ch in msg.getChildren():
            subtree.append(gen_level(ch, base_id))

        rec['subtree'] = subtree
        return rec

    base_id = base.getId()

    data = gen_level(base, base.getId())

    if remove_first:
        return data['subtree']
    else:
        return [data]


def gen_nested(data, forum, topic, replies, forumTime, timezone, cr_roles, replyable,
               isModerated, topic_id, quickreply, sort_mode,
               wtool, ctool, mtool, hasMsgtool,
               request, show_userinfo, portal, encoding):

    topic_url = data['topic_url']
    creator_menus = data['creator_menus']

    def gen_level(message, creator_menus, topic_url):
        data1 = {'topic_url': topic_url,
                 'creator_menus': creator_menus}

        d = gen_flat(data1, forum, topic, [message], forumTime, timezone,
                     cr_roles, replyable,
                     isModerated, topic_id, quickreply, sort_mode, wtool,
                     ctool, mtool, hasMsgtool,
                     request, show_userinfo, portal, encoding, 0)

        rec = d[0]
        
        subtree = []
        for msg in message.getChildren():
            subtree.append(gen_level(msg, creator_menus, topic_url))

        rec['subtree'] = subtree
        return rec


    return gen_level(topic, creator_menus, topic_url)


def get_message_attache_info(msg, msg_uid, portal, forum, cache=None):
    attach_data = None
    if cache is not None:
        attach_data = cache.ZCache_get(portal, 'attach', keywords={'UID': msg_uid})

    if attach_data is not None:
        return attach_data
        
    else:
        attach_data = {}

        url = msg.absolute_url()

        # image
        if hasattr(aq_base(msg), 'image') and (msg.image is not None):
            image = msg.image
            size = image.getSize()
            
            if size:
                #attach_data['image'] = image
                attach_data['image_url'] = '%s/image'%url
                attach_data['image_size'] = '%0.1f Kb'%(float(size)/1024)
                    
                if hasattr(msg,'image_thumbnail'):
                    attach_data['thumbnail_tag'] = msg.image_thumbnail.tag(title='')
                else:
                    scale = 0
                    
                    if image.width > image.height:
                       scale = 128.0 / image.width
                    else:
                       scale = 128.0 / image.height
                       
                    attach_data['thumbnail_tag'] = msg.image.tag(scale=scale,title='')

        # file
        if hasattr(aq_base(msg), 'file') and (msg.file is not None):
            file = msg.file
            size = file.getSize()
            if size > 0:
                #attach_data['file'] = file
                attach_data['file_url'] = '%s/file'%url
                attach_data['file_size'] = '%0.1f Kb'%(float(size)/1024)
            
                info = {}
                try:
                    info['filename'] = file.Filename()
                except:
                    info['filename'] = ''

                try:
                    icon = file.getMimetypeIcon()
                    if icon:
                        info['icon'] = icon
                    else:
                        info['icon'] = 'unknown.png'
                except:
                    info['icon'] = 'unknown.png'

                icon = getattr(forum, info['icon'], '')

                if type(icon) != type(''):
                    icon = icon.tag(title='')

                info['icon'] = icon
                attach_data['file_info'] = info

        if cache is not None:
            cache.ZCache_set(portal, attach_data, 'attach', keywords={'UID': msg_uid})

        return attach_data
