# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nikolay Kim <fafhrd@legco.biz> 

from types import StringType
from Acquisition import aq_parent, aq_inner
from Products.CMFCore.utils import _checkPermission
from Products.CMFCore.CMFCorePermissions import View

def opt_localized_time(time, trans):
    if type(trans) == StringType:
        return time.strftime(trans)
   
    dateParts = time.parts()
    day = '%02d' % dateParts[2]
    month = '%02d' % dateParts[1]
    year = dateParts[0]
    hour = '%02d' % dateParts[3]
    minute = '%02d' % dateParts[4]

    # substitute variables with actual values
    localized_time = trans.replace('${DAY}', str(day))
    localized_time = localized_time.replace('${MONTH}', str(month))
    localized_time = localized_time.replace('${YEAR}', str(year))
    localized_time = localized_time.replace('${HOUR}', str(hour))
    localized_time = localized_time.replace('${MINUTE}', str(minute))

    return localized_time


def get_message_icon(msg, msg_uid, cache, portal, forum, default='&nbsp;'):
    msg_icon = None

    if cache is not None:
        msg_icon = cache.ZCache_get(portal, 'message_icon', keywords={'UID': msg_uid})
        if msg_icon is not None:
            if msg_icon:
                return msg_icon
            else:
                return default

    try:
        msg_icon = getattr(msg, 'message_icon')
        msg_icon = getattr(forum.fcIcons, msg_icon).tag()
    except:
        msg_icon = ''

    if cache is not None:
        cache.ZCache_set(portal, msg_icon, 'message_icon', keywords={'UID': msg_uid})

    return msg_icon or default


def get_forums_tree(self, portal):
    found = 0
    parent = self
    while not found:
        try:
            if parent.portal_type != 'ForumFolder':
                parent = aq_parent(parent)
            else:
                found = 1
        except:
            return []

    parent_uid = parent.UID()

    url = portal.absolute_url()
    folder_icon = '%s/%s'%(url, 'fcFolder_icon.gif')
    forum_icon = '%s/%s'%(url, 'fcForum_icon.gif')

    if parent.aq_parent.portal_type == 'ForumFolder':
        parent = aq_parent(aq_inner(parent))
   
    def add_rec(res, folder, mark1='', mark2='--'):
        if _checkPermission(View, folder):
            res.append({'title':folder.title_or_id(),
                        'disable':0, 'url':folder.absolute_url(),
                        'icon': folder_icon,
                        'mark': mark1})

            for forum in folder.objectValues('ForumNB'):
                if _checkPermission(View, forum):
                    res.append(
                        {'title':forum.title_or_id(),
                         'disable':1, 'url': forum.absolute_url(),
                         'icon': forum_icon,
                         'mark': mark2})

            for folder in folder.objectValues('ForumFolder'):
                add_rec(res, folder, mark1+'-', mark2+'--')

    res = []
    add_rec(res, parent)

    return res
