# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
"""
$Id: utils.py,v 1.22.4.3.2.4 2005/05/20 11:54:15 fafhrd91 Exp $
"""

import re, os
from debug import log_exc
from types import TypeType, ClassType
from types import TypeType, ClassType
from DocumentTemplate.DT_Util import html_quote
from Products.CMFDefault.utils import bodyfinder, scrubHTML

from htmlentitydefs import entitydefs
entitydefs['nbsp'] = ' '
entities_data = [('&%s;'%k, v) for k, v in entitydefs.items() if k != 'amp']
def html_unquote(text):
    for s, r in entities_data:
        try:
            text = text.replace(s, r)
        except:
            pass

    return text.replace('&amp;', '&')

from forum_codes import parser

# matches an "xxxx://yyyy" URL at the start of a line, or after a space.
# xxxx can only be alpha characters.
# yyyy is anything up to the first space, newline, comma, double quote or <
re1 = re.compile("(^|[\n ])([\w]+?://[^ \"\n\r\t<]*)")

# matches a "www|ftp.xxxx.yyyy[/zzzz]" kinda lazy URL thing
# Must contain at least 2 dots. xxxx contains either alphanum, or "-"
# zzzz is optional.. will contain everything up to the first space, newline, 
# comma, double quote or <.
re2 = re.compile("(^|[\n ])((www|ftp)\.[^ \"\t\n\r<]*)")

# matches an email@domain type address at the start of a line, or after a space.
# Note: Only the followed chars are valid; alphanums, "-", "_" and or ".".
re3 = re.compile("(^|[\n ])([a-z0-9&\-_.]+?)@([\w\-]+\.([\w\-\.]+\.)*[\w]+)")


def forum_parser(self, content, emoticons=1, mimetype='text/plain', convert_br=0):
    if convert_br:
        content = html_quote(str(content).strip())

    if mimetype == 'text/plain':
        try:
            end = 0
            new_content = ''
            for m in re1.finditer(content):
                new_content += content[end:m.start()]
                grps = m.groups()
                new_content += '%s<a href="%s" target="_blank">%s</a>'%(grps[0],grps[1],grps[1])
                end = m.end()

            new_content += content[end:]
            content = new_content

            end = 0
            new_content = ''
            for m in re2.finditer(content):
                new_content += content[end:m.start()]
                grps = m.groups()
                new_content += '%s<a href="http://%s" target="_blank">%s</a>'%(grps[0],grps[1],grps[1])
                end = m.end()

            new_content += content[end:]
            content = new_content

            end = 0
            new_content = ''
            for m in re3.finditer(content):
                new_content += content[end:m.start()]
                grps = m.groups()
                new_content += '%s<a href="mailto:%s@%s">%s@%s</a>'%(
                    grps[0],grps[1],grps[2],grps[1],grps[2])
                end = m.end()

            new_content += content[end:]
            content = new_content            
        except:
            pass

    if convert_br:
        content = content.replace('\n', '<br/>')

    text = parser.parse_text(content, self, mimetype=mimetype, emoticons=emoticons)
    return text

def cb_issubclass (class1, class2):
    """A version of 'issubclass' that works with extension classes
    as well as regular Python classes.
    """
    if type(class1) is ClassType and type(class2) is ClassType:
        return issubclass(class1, class2)

    elif hasattr(class1, '__bases__') and hasattr(class2, '__bases__'):
        stack = [class1]
        while stack:
            if stack[0] is class2:
                return 1
            stack.extend(list(stack[0].__bases__))
            del stack[0]
        else:
            return 0
    else:
        raise TypeError, "arguments must be class or ExtensionClass objects"
				
def cb_isinstance (object, klass):
    """A version of 'isinstance' that works with extension classes
    as well as regular Python classes."""

    if type(klass) is TypeType:
        return isinstance(object, klass)
    elif hasattr(object, '__class__'):
        return cb_issubclass(object.__class__, klass)
    else:
        return 0

# word-wrap function that preserves line breaks and spaces
# (from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/148061)
def wrap(s, width=72):
    return reduce(lambda line, word, width=width: '%s%s%s' %
                  (line,
                   ' \n'[(len(line[line.rfind('\n')+1:])
                          + len(word.split('\n',1)[0]
                                ) >= width)],
                   word),
                  s.split(' ')
                  )


def wrap_filename(f_name):
    dir, f_name = os.path.split(f_name)
    f_name = f_name.split('\\')[-1].split('/')[-1]
    for key in '~,\'':
        try:
            f_name = f_name.replace(key, '_')
        except:
            pass
    return f_name


def forum_stateChanged(sci):
    try:
        sci.object.notifyStateChanged(sci)
    except:
        log_exc()

# Regexp for snipping the body of a document
try:
    from mx import Tidy
    mxTidyIsAvailable = 1
except ImportError:
    mxTidyIsAvailable = 0

def forum_html_tidy(self, html):
    """ """
    errors = 0
    output = html
    errordata = ""

    if mxTidyIsAvailable:
        input = html.encode('utf-8')
        
        (errors, warnings, output, errordata) = Tidy.tidy(
            input,
            drop_font_tags=1,
            drop_empty_paras=1,
            indent_spaces=0,
            output_xhtml = 1,
            indent='no',
            word_2000=1,
            wrap=0,
            logical_emphasis = 1,
            char_encoding = 'utf8')

        output = unicode(output, 'utf-8')
        
        if errors:
            output = html

    result = scrubHTML(bodyfinder(output))
    output = result.strip()

    return (errors, output, errordata)
